@@ -63,6 +63,23 @@ static int result = -1;
 static struct TALER_AUDITORDB_Plugin *plugin;
 
 
+static int
+select_denomination_info_result (void *cls,
+                                 const struct TALER_DenominationKeyValidityPS *issue2)
+{
+  const struct TALER_DenominationKeyValidityPS *issue1 = cls;
+
+  if (0 != memcmp (issue1, issue2, sizeof (*issue2)))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "select_denomination_info_result: issue does not match\n");
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
+}
+
+
 /**
  * Main function that will be run by the scheduler.
  *
@@ -73,6 +90,7 @@ run (void *cls)
 {
   struct GNUNET_CONFIGURATION_Handle *cfg = cls;
   struct TALER_AUDITORDB_Session *session;
+  uint64_t rowid;
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "loading database plugin\n");
@@ -89,7 +107,7 @@ run (void *cls)
       plugin->create_tables (plugin->cls))
   {
     result = 77;
-    goto drop;
+    goto unload;
   }
   if (NULL ==
       (session = plugin->get_session (plugin->cls)))
@@ -98,10 +116,18 @@ run (void *cls)
     goto drop;
   }
 
+  FAILIF (GNUNET_OK !=
+          plugin->start (plugin->cls,
+                         session));
+
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "initializing\n");
 
-  struct TALER_Amount value, fee_withdraw, fee_deposit, fee_refresh, fee_refund;
+  struct TALER_Amount value;
+  struct TALER_Amount fee_withdraw;
+  struct TALER_Amount fee_deposit;
+  struct TALER_Amount fee_refresh;
+  struct TALER_Amount fee_refund;
 
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":1.000010",
@@ -175,22 +201,6 @@ run (void *cls)
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: select_denomination_info\n");
 
-  int
-  select_denomination_info_result (void *cls,
-                                   const struct TALER_DenominationKeyValidityPS *issue2)
-  {
-    const struct TALER_DenominationKeyValidityPS *issue1 = cls;
-
-    if (0 != memcmp (issue1, issue2, sizeof (*issue2)))
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "select_denomination_info_result: issue does not match\n");
-      GNUNET_break (0);
-      return GNUNET_SYSERR;
-    }
-    return GNUNET_OK;
-  }
-
   FAILIF (GNUNET_OK !=
           plugin->select_denomination_info (plugin->cls,
                                             session,
@@ -201,52 +211,43 @@ run (void *cls)
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: insert_auditor_progress\n");
 
-  uint64_t
-      last_reserve_in_serial_id = 1234,
-      last_reserve_out_serial_id = 5678,
-      last_deposit_serial_id = 123,
-      last_melt_serial_id = 456,
-      last_refund_serial_id = 789,
-      last_prewire_serial_id = 555,
-
-      last_reserve_in_serial_id2 = 0,
-      last_reserve_out_serial_id2 = 0,
-      last_deposit_serial_id2 = 0,
-      last_melt_serial_id2 = 0,
-      last_refund_serial_id2 = 0,
-      last_prewire_serial_id2 = 0;
+  struct TALER_AUDITORDB_ProgressPoint pp = {
+    .last_reserve_in_serial_id = 1234,
+    .last_reserve_out_serial_id = 5678,
+    .last_deposit_serial_id = 123,
+    .last_melt_serial_id = 456,
+    .last_refund_serial_id = 789,
+    .last_wire_out_serial_id = 555
+  };
+  struct TALER_AUDITORDB_ProgressPoint pp2 = {
+    .last_reserve_in_serial_id = 0,
+    .last_reserve_out_serial_id = 0,
+    .last_deposit_serial_id = 0,
+    .last_melt_serial_id = 0,
+    .last_refund_serial_id = 0,
+    .last_wire_out_serial_id = 0
+  };
 
   FAILIF (GNUNET_OK !=
           plugin->insert_auditor_progress (plugin->cls,
                                            session,
                                            &master_pub,
-                                           last_reserve_in_serial_id,
-                                           last_reserve_out_serial_id,
-                                           last_deposit_serial_id,
-                                           last_melt_serial_id,
-                                           last_refund_serial_id,
-                                           last_prewire_serial_id));
-
+                                           &pp));
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: update_auditor_progress\n");
 
-  last_reserve_in_serial_id++;
-  last_reserve_out_serial_id++;
-  last_deposit_serial_id2++;
-  last_melt_serial_id2++;
-  last_refund_serial_id2++;
-  last_prewire_serial_id2++;
+  pp.last_reserve_in_serial_id++;
+  pp.last_reserve_out_serial_id++;
+  pp.last_deposit_serial_id++;
+  pp.last_melt_serial_id++;
+  pp.last_refund_serial_id++;
+  pp.last_wire_out_serial_id++;
 
   FAILIF (GNUNET_OK !=
           plugin->update_auditor_progress (plugin->cls,
                                            session,
                                            &master_pub,
-                                           last_reserve_in_serial_id,
-                                           last_reserve_out_serial_id,
-                                           last_deposit_serial_id,
-                                           last_melt_serial_id,
-                                           last_refund_serial_id,
-                                           last_prewire_serial_id));
+                                           &pp));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: get_auditor_progress\n");
@@ -255,19 +256,13 @@ run (void *cls)
           plugin->get_auditor_progress (plugin->cls,
                                         session,
                                         &master_pub,
-                                        &last_reserve_in_serial_id2,
-                                        &last_reserve_out_serial_id2,
-                                        &last_deposit_serial_id2,
-                                        &last_melt_serial_id2,
-                                        &last_refund_serial_id2,
-                                        &last_prewire_serial_id2));
-
-  FAILIF (last_reserve_in_serial_id2 != last_reserve_in_serial_id
-          || last_reserve_out_serial_id2 != last_reserve_out_serial_id
-          || last_deposit_serial_id2 != last_deposit_serial_id
-          || last_melt_serial_id2 != last_melt_serial_id
-          || last_refund_serial_id2 != last_refund_serial_id
-          || last_prewire_serial_id2 != last_prewire_serial_id);
+                                        &pp2));
+  FAILIF ( (pp.last_reserve_in_serial_id != pp2.last_reserve_in_serial_id) ||
+           (pp.last_reserve_out_serial_id != pp2.last_reserve_out_serial_id) ||
+           (pp.last_deposit_serial_id != pp2.last_deposit_serial_id) ||
+           (pp.last_melt_serial_id != pp2.last_melt_serial_id) ||
+           (pp.last_refund_serial_id != pp2.last_refund_serial_id) ||
+           (pp.last_wire_out_serial_id != pp2.last_wire_out_serial_id) );
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: insert_reserve_info\n");
@@ -289,16 +284,11 @@ run (void *cls)
                                        &master_pub,
                                        &reserve_balance,
                                        &withdraw_fee_balance,
-                                       past,
-                                       last_reserve_in_serial_id,
-                                       last_reserve_out_serial_id));
+                                       past));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: update_reserve_info\n");
 
-  last_reserve_in_serial_id++;
-  last_reserve_out_serial_id++;
-
   FAILIF (GNUNET_OK !=
           plugin->update_reserve_info (plugin->cls,
                                        session,
@@ -306,9 +296,7 @@ run (void *cls)
                                        &master_pub,
                                        &reserve_balance,
                                        &withdraw_fee_balance,
-                                       future,
-                                       last_reserve_in_serial_id,
-                                       last_reserve_out_serial_id));
+                                       future));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: get_reserve_info\n");
@@ -318,17 +306,14 @@ run (void *cls)
                                     session,
                                     &reserve_pub,
                                     &master_pub,
+                                    &rowid,
                                     &reserve_balance2,
                                     &withdraw_fee_balance2,
-                                    &date,
-                                    &last_reserve_in_serial_id2,
-                                    &last_reserve_out_serial_id2));
+                                    &date));
 
   FAILIF (0 != memcmp (&date, &future, sizeof (future))
           || 0 != memcmp (&reserve_balance2, &reserve_balance, sizeof (reserve_balance))
-          || 0 != memcmp (&withdraw_fee_balance2, &withdraw_fee_balance, sizeof (withdraw_fee_balance))
-          || last_reserve_in_serial_id2 != last_reserve_in_serial_id
-          || last_reserve_out_serial_id2 != last_reserve_out_serial_id);
+          || 0 != memcmp (&withdraw_fee_balance2, &withdraw_fee_balance, sizeof (withdraw_fee_balance)));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: insert_reserve_summary\n");
@@ -371,6 +356,7 @@ run (void *cls)
 
   struct TALER_Amount denom_balance, deposit_fee_balance, melt_fee_balance, refund_fee_balance;
   struct TALER_Amount denom_balance2, deposit_fee_balance2, melt_fee_balance2, refund_fee_balance2;
+  struct TALER_Amount rbalance, rbalance2;
 
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":12.345678",
@@ -384,41 +370,31 @@ run (void *cls)
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":45.678901",
                                          &refund_fee_balance));
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_string_to_amount (CURRENCY ":13.57986",
+                                         &rbalance));
 
   FAILIF (GNUNET_OK !=
           plugin->insert_denomination_balance (plugin->cls,
                                                session,
                                                &denom_pub_hash,
-                                               &refund_fee_balance,
-                                               &melt_fee_balance,
-                                               &deposit_fee_balance,
                                                &denom_balance,
-                                               last_reserve_out_serial_id,
-                                               last_deposit_serial_id,
-                                               last_melt_serial_id,
-                                               last_refund_serial_id));
+                                               &rbalance));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: update_denomination_balance\n");
 
-  last_reserve_out_serial_id++;
-  last_deposit_serial_id++;
-  last_melt_serial_id++;
-  last_refund_serial_id++;
+  pp.last_reserve_out_serial_id++;
+  pp.last_deposit_serial_id++;
+  pp.last_melt_serial_id++;
+  pp.last_refund_serial_id++;
 
   FAILIF (GNUNET_OK !=
           plugin->update_denomination_balance (plugin->cls,
                                                session,
                                                &denom_pub_hash,
                                                &denom_balance,
-                                               &deposit_fee_balance,
-                                               &melt_fee_balance,
-                                               &refund_fee_balance,
-                                               last_reserve_out_serial_id,
-                                               last_deposit_serial_id,
-                                               last_melt_serial_id,
-                                               last_refund_serial_id));
-
+                                               &rbalance));
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: get_denomination_balance\n");
 
@@ -427,107 +403,63 @@ run (void *cls)
                                             session,
                                             &denom_pub_hash,
                                             &denom_balance2,
-                                            &deposit_fee_balance2,
-                                            &melt_fee_balance2,
-                                            &refund_fee_balance2,
-                                            &last_reserve_out_serial_id2,
-                                            &last_deposit_serial_id2,
-                                            &last_melt_serial_id2,
-                                            &last_refund_serial_id2));
+                                            &rbalance2));
+
+  FAILIF (0 != memcmp (&denom_balance2, &denom_balance, sizeof (denom_balance)));
+  FAILIF (0 != memcmp (&rbalance2, &rbalance, sizeof (rbalance)));
 
-  FAILIF (0 != memcmp (&denom_balance2, &denom_balance, sizeof (denom_balance))
-          || 0 != memcmp (&deposit_fee_balance2, &deposit_fee_balance, sizeof (deposit_fee_balance))
-          || 0 != memcmp (&melt_fee_balance2, &melt_fee_balance, sizeof (melt_fee_balance))
-          || 0 != memcmp (&refund_fee_balance2, &refund_fee_balance, sizeof (refund_fee_balance))
-          || last_reserve_out_serial_id2 != last_reserve_out_serial_id
-          || last_deposit_serial_id2 != last_deposit_serial_id
-          || last_melt_serial_id2 != last_melt_serial_id
-          || last_refund_serial_id2 != last_refund_serial_id);
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Test: insert_denomination_summary\n");
+              "Test: insert_balance_summary\n");
 
   FAILIF (GNUNET_OK !=
-          plugin->insert_denomination_summary (plugin->cls,
+          plugin->insert_balance_summary (plugin->cls,
                                                session,
                                                &master_pub,
                                                &refund_fee_balance,
                                                &melt_fee_balance,
                                                &deposit_fee_balance,
-                                               &denom_balance));
+                                               &denom_balance,
+                                               &rbalance));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Test: update_denomination_summary\n");
+              "Test: update_balance_summary\n");
 
   FAILIF (GNUNET_OK !=
-          plugin->update_denomination_summary (plugin->cls,
+          plugin->update_balance_summary (plugin->cls,
                                                session,
                                                &master_pub,
                                                &denom_balance,
                                                &deposit_fee_balance,
                                                &melt_fee_balance,
-                                               &refund_fee_balance));
+                                               &refund_fee_balance,
+                                               &rbalance));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Test: get_denomination_summary\n");
+              "Test: get_balance_summary\n");
 
   ZR_BLK (&denom_balance2);
   ZR_BLK (&deposit_fee_balance2);
   ZR_BLK (&melt_fee_balance2);
   ZR_BLK (&refund_fee_balance2);
+  ZR_BLK (&rbalance2);
 
   FAILIF (GNUNET_OK !=
-          plugin->get_denomination_summary (plugin->cls,
+          plugin->get_balance_summary (plugin->cls,
                                             session,
                                             &master_pub,
                                             &denom_balance2,
                                             &deposit_fee_balance2,
                                             &melt_fee_balance2,
-                                            &refund_fee_balance2));
+                                            &refund_fee_balance2,
+                                            &rbalance2));
 
   FAILIF (0 != memcmp (&denom_balance2, &denom_balance, sizeof (denom_balance))
           || 0 != memcmp (&deposit_fee_balance2, &deposit_fee_balance, sizeof (deposit_fee_balance))
           || 0 != memcmp (&melt_fee_balance2, &melt_fee_balance, sizeof (melt_fee_balance))
           || 0 != memcmp (&refund_fee_balance2, &refund_fee_balance, sizeof (refund_fee_balance)));
+  FAILIF (0 != memcmp (&rbalance2, &rbalance, sizeof (rbalance)));
 
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Test: insert_risk_summary\n");
-
-  struct TALER_Amount balance, balance2;
-
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_string_to_amount (CURRENCY ":13.57986",
-                                         &balance));
-
-  FAILIF (GNUNET_OK !=
-          plugin->insert_risk_summary (plugin->cls,
-                                       session,
-                                       &master_pub,
-                                       &balance));
-
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Test: update_risk_summary\n");
-
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_string_to_amount (CURRENCY ":57.310986",
-                                         &balance));
-
-  FAILIF (GNUNET_OK !=
-          plugin->update_risk_summary (plugin->cls,
-                                       session,
-                                       &master_pub,
-                                       &balance));
-
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Test: get_risk_summary\n");
-
-  FAILIF (GNUNET_OK !=
-          plugin->get_risk_summary (plugin->cls,
-                                    session,
-                                    &master_pub,
-                                    &balance2));
-
-  FAILIF (0 != memcmp (&balance2, &balance, sizeof (balance)));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: insert_historic_denom_revenue\n");
@@ -538,10 +470,7 @@ run (void *cls)
                                                  &master_pub,
                                                  &denom_pub_hash,
                                                  past,
-                                                 &balance,
-                                                 &deposit_fee_balance,
-                                                 &melt_fee_balance,
-                                                 &refund_fee_balance));
+                                                 &rbalance));
 
   FAILIF (GNUNET_OK !=
           plugin->insert_historic_denom_revenue (plugin->cls,
@@ -549,10 +478,7 @@ run (void *cls)
                                                  &master_pub,
                                                  &rnd_hash,
                                                  now,
-                                                 &balance,
-                                                 &deposit_fee_balance,
-                                                 &melt_fee_balance,
-                                                 &refund_fee_balance));
+                                                 &rbalance));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: select_historic_denom_revenue\n");
@@ -561,10 +487,7 @@ run (void *cls)
   select_historic_denom_revenue_result (void *cls,
                                         const struct GNUNET_HashCode *denom_pub_hash2,
                                         struct GNUNET_TIME_Absolute revenue_timestamp2,
-                                        const struct TALER_Amount *revenue_balance2,
-                                        const struct TALER_Amount *deposit_fee_balance2,
-                                        const struct TALER_Amount *melt_fee_balance2,
-                                        const struct TALER_Amount *refund_fee_balance2)
+                                        const struct TALER_Amount *revenue_balance2)
   {
     static int n = 0;
 
@@ -577,10 +500,7 @@ run (void *cls)
             && 0 != memcmp (&revenue_timestamp2, &now, sizeof (now)))
         || (0 != memcmp (denom_pub_hash2, &denom_pub_hash, sizeof (denom_pub_hash))
             && 0 != memcmp (denom_pub_hash2, &rnd_hash, sizeof (rnd_hash)))
-        || 0 != memcmp (revenue_balance2, &balance, sizeof (balance))
-        || 0 != memcmp (deposit_fee_balance2, &deposit_fee_balance, sizeof (deposit_fee_balance))
-        || 0 != memcmp (melt_fee_balance2, &melt_fee_balance, sizeof (melt_fee_balance))
-        || 0 != memcmp (refund_fee_balance2, &refund_fee_balance, sizeof (refund_fee_balance)))
+        || 0 != memcmp (revenue_balance2, &rbalance, sizeof (rbalance)))
     {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "select_historic_denom_revenue_result: result does not match\n");
@@ -606,7 +526,7 @@ run (void *cls)
                                           &master_pub,
                                           &denom_pub_hash,
                                           past,
-                                          &balance));
+                                          &rbalance));
 
   FAILIF (GNUNET_OK !=
           plugin->insert_historic_losses (plugin->cls,
@@ -614,7 +534,8 @@ run (void *cls)
                                           &master_pub,
                                           &rnd_hash,
                                           past,
-                                          &balance));
+                                          &rbalance));
+
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: select_historic_losses\n");
@@ -636,7 +557,7 @@ run (void *cls)
             && 0 != memcmp (&loss_timestamp2, &now, sizeof (now)))
         || (0 != memcmp (denom_pub_hash2, &denom_pub_hash, sizeof (denom_pub_hash))
             && 0 != memcmp (denom_pub_hash2, &rnd_hash, sizeof (rnd_hash)))
-        || 0 != memcmp (loss_balance2, &balance, sizeof (balance)))
+        || 0 != memcmp (loss_balance2, &rbalance, sizeof (rbalance)))
     {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "select_historic_denom_revenue_result: result does not match\n");
@@ -720,20 +641,51 @@ run (void *cls)
           plugin->insert_predicted_result (plugin->cls,
                                            session,
                                            &master_pub,
-                                           &balance));
+                                           &rbalance));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: update_predicted_result\n");
 
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":78.901234",
-                                         &balance));
+                                         &rbalance));
 
   FAILIF (GNUNET_OK !=
           plugin->update_predicted_result (plugin->cls,
                                            session,
                                            &master_pub,
-                                           &balance));
+                                           &rbalance));
+
+  FAILIF (GNUNET_OK !=
+          plugin->insert_wire_fee_summary (plugin->cls,
+                                           session,
+                                           &master_pub,
+                                           &rbalance));
+  FAILIF (GNUNET_OK !=
+          plugin->update_wire_fee_summary (plugin->cls,
+                                           session,
+                                           &master_pub,
+                                           &reserve_profits));
+  {
+    struct TALER_Amount rprof;
+
+    FAILIF (GNUNET_OK !=
+            plugin->get_wire_fee_summary (plugin->cls,
+                                          session,
+                                          &master_pub,
+                                          &rprof));
+    FAILIF (0 !=
+            TALER_amount_cmp (&rprof,
+                              &reserve_profits));
+  }
+  FAILIF (GNUNET_OK !=
+          plugin->commit (plugin->cls,
+                          session));
+
+
+  FAILIF (GNUNET_OK !=
+          plugin->start (plugin->cls,
+                         session));
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Test: get_predicted_balance\n");
@@ -742,16 +694,35 @@ run (void *cls)
           plugin->get_predicted_balance (plugin->cls,
                                          session,
                                          &master_pub,
-                                         &balance2));
+                                         &rbalance2));
 
-  FAILIF (0 != memcmp (&balance2, &balance, sizeof (balance)));
+  FAILIF (GNUNET_OK !=
+          plugin->del_reserve_info (plugin->cls,
+                                    session,
+                                    &reserve_pub,
+                                    &master_pub));
+
+
+  FAILIF (0 != TALER_amount_cmp (&rbalance2,
+                                 &rbalance));
+
+  plugin->rollback (plugin->cls,
+                    session);
+
+#if GC_IMPLEMENTED
+  FAILIF (GNUNET_OK !=
+          plugin->gc (plugin->cls));
+#endif
 
   result = 0;
 
 drop:
-
+  if (NULL != session)
+    plugin->rollback (plugin->cls,
+                      session);
   GNUNET_break (GNUNET_OK ==
                 plugin->drop_tables (plugin->cls));
+ unload:
   TALER_AUDITORDB_plugin_unload (plugin);
   plugin = NULL;
 }
