@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015, 2016 GNUnet e.V.
+  Copyright (C) 2015, 2016, 2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -20,13 +20,8 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
-#include <jansson.h>
+#include "bank_api_common.h"
 #include <microhttpd.h> /* just for HTTP status codes */
-#include <gnunet/gnunet_util_lib.h>
-#include <gnunet/gnunet_json_lib.h>
-#include <gnunet/gnunet_curl_lib.h>
-#include "taler_bank_service.h"
-#include "taler_json_lib.h"
 #include "taler_signatures.h"
 
 
@@ -52,9 +47,9 @@ struct TALER_BANK_AdminAddIncomingHandle
   struct GNUNET_CURL_Job *job;
 
   /**
-   * HTTP headers for the request.
+   * HTTP authentication-related headers for the request.
    */
-  struct curl_slist *headers;
+  struct curl_slist *authh;
 
   /**
    * Function to call with the result.
@@ -69,34 +64,6 @@ struct TALER_BANK_AdminAddIncomingHandle
 };
 
 
-
-/**
- * Obtain the URL to use for an API request.
- *
- * @param u base URL of the bank
- * @param path Taler API path (i.e. "/reserve/withdraw")
- * @return the full URI to use with cURL
- */
-static char *
-path_to_url (const char *u,
-             const char *path)
-{
-  char *url;
-
-  if ( ('/' == path[0]) &&
-       (0 < strlen (u)) &&
-       ('/' == u[strlen (u) - 1]) )
-    path++; /* avoid generating URL with "//" from concat */
-  GNUNET_asprintf (&url,
-                   "%s%s",
-                   u,
-                   path);
-  return url;
-}
-
-
-
-
 /**
  * Function called when we're done processing the
  * HTTP /admin/add/incoming request.
@@ -111,6 +78,7 @@ handle_admin_add_incoming_finished (void *cls,
                                     const json_t *json)
 {
   struct TALER_BANK_AdminAddIncomingHandle *aai = cls;
+  uint64_t serial_id = UINT64_MAX;
 
   aai->job = NULL;
   switch (response_code)
@@ -118,6 +86,23 @@ handle_admin_add_incoming_finished (void *cls,
   case 0:
     break;
   case MHD_HTTP_OK:
+    {
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_uint64 ("serial_id",
+                                 &serial_id),
+        GNUNET_JSON_spec_end()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (json,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        response_code = 0;
+        break;
+      }
+    }
     break;
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the bank is buggy
@@ -150,6 +135,7 @@ handle_admin_add_incoming_finished (void *cls,
   }
   aai->cb (aai->cb_cls,
            response_code,
+           serial_id,
            json);
   TALER_BANK_admin_add_incoming_cancel (aai);
 }
@@ -162,10 +148,11 @@ handle_admin_add_incoming_finished (void *cls,
  * to the operators of the bank.
  *
  * @param ctx curl context for the event loop
- * @param bank_base_url URL of the bank
- * @param reserve_pub public key of the reserve
+ * @param bank_base_url URL of the bank (used to execute this request)
+ * @param auth authentication data to send to the bank
+ * @param exchange_base_url base URL of the exchange (for tracking)
+ * @param wtid wire transfer identifier for the transfer
  * @param amount amount that was deposited
- * @param execution_date when did we receive the amount
  * @param debit_account_no account number to withdraw from (53 bits at most)
  * @param credit_account_no account number to deposit into (53 bits at most)
  * @param res_cb the callback to call when the final result for this request is available
@@ -177,6 +164,8 @@ handle_admin_add_incoming_finished (void *cls,
 struct TALER_BANK_AdminAddIncomingHandle *
 TALER_BANK_admin_add_incoming (struct GNUNET_CURL_Context *ctx,
                                const char *bank_base_url,
+                               const struct TALER_BANK_AuthenticationData *auth,
+                               const char *exchange_base_url,
                                const struct TALER_WireTransferIdentifierRawP *wtid,
                                const struct TALER_Amount *amount,
                                uint64_t debit_account_no,
@@ -188,23 +177,50 @@ TALER_BANK_admin_add_incoming (struct GNUNET_CURL_Context *ctx,
   json_t *admin_obj;
   CURL *eh;
 
-  admin_obj = json_pack ("{s:o, s:o,"
-                         " s:I, s:I}",
-                         "wtid", GNUNET_JSON_from_data_auto (wtid), /* #4340 */
+  if (NULL == exchange_base_url)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  admin_obj = json_pack ("{s:{s:s}, s:s, s:o, s:o, s:I, s:I}",
+                         "auth", "type", "basic",
+                         "exchange_url", exchange_base_url,
+                         "wtid", GNUNET_JSON_from_data_auto (wtid),
                          "amount", TALER_JSON_from_amount (amount),
                          "debit_account", (json_int_t) debit_account_no,
                          "credit_account", (json_int_t) credit_account_no);
+  if (NULL == admin_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
   aai = GNUNET_new (struct TALER_BANK_AdminAddIncomingHandle);
   aai->cb = res_cb;
   aai->cb_cls = res_cb_cls;
-  aai->request_url = path_to_url (bank_base_url,
-                                  "/admin/add/incoming");
+  aai->request_url = TALER_BANK_path_to_url_ (bank_base_url,
+                                              "/admin/add/incoming");
+  aai->authh = TALER_BANK_make_auth_header_ (auth);
+  /* Append content type header here, can't do it in GNUNET_CURL_job_add
+     as that would override the CURLOPT_HTTPHEADER instead of appending. */
+  {
+    struct curl_slist *ext;
 
+    ext = curl_slist_append (aai->authh,
+                             "Content-Type: application/json");
+    if (NULL == ext)
+      GNUNET_break (0);
+    else
+      aai->authh = ext;
+  }
   eh = curl_easy_init ();
   GNUNET_assert (NULL != (aai->json_enc =
                           json_dumps (admin_obj,
                                       JSON_COMPACT)));
   json_decref (admin_obj);
+  GNUNET_assert (CURLE_OK ==
+                 curl_easy_setopt (eh,
+                                   CURLOPT_HTTPHEADER,
+                                   aai->authh));
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
                                    CURLOPT_URL,
@@ -219,7 +235,7 @@ TALER_BANK_admin_add_incoming (struct GNUNET_CURL_Context *ctx,
                                    strlen (aai->json_enc)));
   aai->job = GNUNET_CURL_job_add (ctx,
                                   eh,
-                                  GNUNET_YES,
+                                  GNUNET_NO,
                                   &handle_admin_add_incoming_finished,
                                   aai);
   return aai;
@@ -240,7 +256,7 @@ TALER_BANK_admin_add_incoming_cancel (struct TALER_BANK_AdminAddIncomingHandle *
     GNUNET_CURL_job_cancel (aai->job);
     aai->job = NULL;
   }
-  curl_slist_free_all (aai->headers);
+  curl_slist_free_all (aai->authh);
   GNUNET_free (aai->request_url);
   GNUNET_free (aai->json_enc);
   GNUNET_free (aai);
