@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016 GNUnet e.V.
+  Copyright (C) 2016, 2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -44,6 +44,11 @@ enum TBI_OpCode
    */
   TBI_OC_ADMIN_ADD_INCOMING,
 
+  /**
+   * Request wire transfer history.
+   */
+  TBI_OC_HISTORY,
+
   /**
    * Expect that we have received the specified transfer at fakebank.
    */
@@ -99,6 +104,11 @@ struct TBI_Command
        */
       uint64_t debit_account_no;
 
+      /**
+       * Exchange base URL to use.
+       */
+      const char *exchange_base_url;
+
       /**
        * Wire transfer identifier to use.  Initialized to
        * a random value.
@@ -115,8 +125,53 @@ struct TBI_Command
        */
       struct TALER_BANK_AdminAddIncomingHandle *aih;
 
+      /**
+       * The serial ID for this record, as returned by the bank.
+       */
+      uint64_t serial_id;
+
     } admin_add_incoming;
 
+    struct {
+
+      /**
+       * For which account do we query the history.
+       */
+      uint64_t account_number;
+
+      /**
+       * Which types of transactions should be listed?
+       */
+      enum TALER_BANK_Direction direction;
+
+      /**
+       * At which serial ID do we start? References the respective @e
+       * admin_add_incoming command.  Use NULL for the extremes.
+       */
+      const char *start_row_ref;
+
+      /**
+       * How many results should be returned (if available)?
+       */
+      int64_t num_results;
+
+      /**
+       * Set to the API's handle during the operation.
+       */
+      struct TALER_BANK_HistoryHandle *hh;
+
+      /**
+       * How many results did we actually get?
+       */
+      uint64_t results_obtained;
+
+      /**
+       * Set to #GNUNET_YES if we encountered a problem.
+       */
+      int failed;
+
+    } history;
+
     /**
      * If @e opcode is #TBI_OC_EXPECT_TRANSFER, this
      * specifies which transfer we expected.
