@@ -268,7 +268,7 @@ static int run_exchange;
  * every 50 iterations. Also includes how long the iteration took,
  * so we can see if it is stable.
  */
-static int be_verbose;
+static unsigned int be_verbose;
 
 /**
  * How many coins the benchmark should operate on
@@ -357,11 +357,6 @@ static struct Reserve *reserves;
  */
 static struct Coin *coins;
 
-/**
- * Transaction id counter, used in /deposit's
- */
-static unsigned int transaction_id;
-
 /**
  * Transfer UUID counter, used in /admin/add/incoming
  */
@@ -395,13 +390,13 @@ static char *currency;
 static struct GNUNET_TIME_Absolute start_time;
 
 /**
- * Number of times #bennchmark_run has executed. Used
+ * Number of times #benchmark_run has executed. Used
  * to indicate when we consider us warm.
  */
 static unsigned long long warm;
 
 /**
- * Number of times #bennchmark_run should execute
+ * Number of times #benchmark_run should execute
  * before we shut down.
  */
 static unsigned int num_iterations;
@@ -430,7 +425,7 @@ static unsigned long long num_admin;
 /**
  * Throw a weighted coin with @a probability.
  *
- * @reurn #GNUNET_OK with @a probability, #GNUNET_NO with 1 - @a probability
+ * @return #GNUNET_OK with @a probability, #GNUNET_NO with 1 - @a probability
  */
 static unsigned int
 eval_probability (float probability)
@@ -831,7 +826,7 @@ spend_coin (struct Coin *coin,
   struct GNUNET_TIME_Absolute wire_deadline;
   struct GNUNET_TIME_Absolute timestamp;
   struct GNUNET_TIME_Absolute refund_deadline;
-  struct GNUNET_HashCode h_contract;
+  struct GNUNET_HashCode h_contract_terms;
   struct TALER_CoinSpendPublicKeyP coin_pub;
   struct TALER_DepositRequestPS dr;
   struct TALER_MerchantPublicKeyP merchant_pub;
@@ -840,8 +835,8 @@ spend_coin (struct Coin *coin,
   GNUNET_CRYPTO_eddsa_key_get_public (&coin->coin_priv.eddsa_priv,
 				      &coin_pub.eddsa_pub);
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
-			      &h_contract,
-			      sizeof (h_contract));
+			      &h_contract_terms,
+			      sizeof (h_contract_terms));
   timestamp = GNUNET_TIME_absolute_get ();
   wire_deadline = GNUNET_TIME_absolute_add (timestamp,
 					    GNUNET_TIME_UNIT_WEEKS);
@@ -887,13 +882,12 @@ spend_coin (struct Coin *coin,
   memset (&dr, 0, sizeof (dr));
   dr.purpose.size = htonl (sizeof (struct TALER_DepositRequestPS));
   dr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
-  dr.h_contract = h_contract;
+  dr.h_contract_terms = h_contract_terms;
   TALER_JSON_hash (merchant_details,
 		   &dr.h_wire);
 
   dr.timestamp = GNUNET_TIME_absolute_hton (timestamp);
   dr.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
-  dr.transaction_id = GNUNET_htonll (transaction_id);
 
   TALER_amount_hton (&dr.amount_with_fee,
 		     &amount);
@@ -914,12 +908,11 @@ spend_coin (struct Coin *coin,
 				     &amount,
 				     wire_deadline,
 				     merchant_details,
-				     &h_contract,
+				     &h_contract_terms,
 				     &coin_pub,
 				     &coin->sig,
 				     &coin->pk->key,
 				     timestamp,
-				     transaction_id++,
 				     &merchant_pub,
 				     refund_deadline,
 				     &coin_sig,
@@ -1449,17 +1442,12 @@ run (void *cls)
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "gotten pool_size of %d\n",
               pool_size);
-  if (NULL == config_file)
-  {
-    fail ("-c option is mandatory");
-    return;
-  }
-
   cfg = GNUNET_CONFIGURATION_create ();
   GNUNET_SCHEDULER_add_shutdown (&do_shutdown,
 				 NULL);
-  if (GNUNET_SYSERR == GNUNET_CONFIGURATION_parse (cfg,
-						   config_file))
+  if (GNUNET_SYSERR ==
+      GNUNET_CONFIGURATION_parse (cfg,
+                                  config_file))
   {
     fail ("Failed to parse configuration file");
     return;
@@ -1574,24 +1562,34 @@ main (int argc,
   struct GNUNET_OS_Process *proc;
   unsigned int cnt;
   const struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'a', "automate", NULL,
-     "Initialize and start the bank and exchange", GNUNET_NO,
-     &GNUNET_GETOPT_set_one, &run_exchange},
-    GNUNET_GETOPT_OPTION_CFG_FILE (&config_file),
-    {'e', "exchange-uri", "URI",
-     "URI of the exchange", GNUNET_YES,
-     &GNUNET_GETOPT_set_string, &exchange_uri},
-    {'E', "exchange-admin-uri", "URI",
-     "URI of the administrative interface of the exchange", GNUNET_YES,
-     &GNUNET_GETOPT_set_string, &exchange_admin_uri},
-    GNUNET_GETOPT_OPTION_HELP ("tool to benchmark the Taler exchange"),
-    {'s', "pool-size", "SIZE",
-     "How many coins this benchmark should instantiate", GNUNET_YES,
-     &GNUNET_GETOPT_set_uint, &pool_size},
-    {'l', "limit", "LIMIT",
-     "Terminate the benchmark after LIMIT operations", GNUNET_YES,
-     &GNUNET_GETOPT_set_uint, &num_iterations},
-    GNUNET_GETOPT_OPTION_VERBOSE (&be_verbose),
+    GNUNET_GETOPT_option_flag ('a',
+                               "automate",
+                               "Initialize and start the bank and exchange",
+                               &run_exchange),
+    GNUNET_GETOPT_option_mandatory
+    (GNUNET_GETOPT_option_cfgfile (&config_file)),
+    GNUNET_GETOPT_option_string ('e',
+                                 "exchange-uri",
+                                 "URI",
+                                 "URI of the exchange",
+                                 &exchange_uri),
+    GNUNET_GETOPT_option_string ('E',
+                                 "exchange-admin-uri",
+                                 "URI",
+                                 "URI of the administrative interface of the exchange",
+                                 &exchange_admin_uri),
+    GNUNET_GETOPT_option_help ("tool to benchmark the Taler exchange"),
+    GNUNET_GETOPT_option_uint ('s',
+                               "pool-size",
+                               "SIZE",
+                               "How many coins this benchmark should instantiate",
+                               &pool_size),
+    GNUNET_GETOPT_option_uint ('l',
+                               "limit",
+                               "LIMIT",
+                               "Terminate the benchmark after LIMIT operations",
+                               &num_iterations),
+    GNUNET_GETOPT_option_verbose (&be_verbose),
     GNUNET_GETOPT_OPTION_END
   };
   int ret;
@@ -1694,7 +1692,8 @@ main (int argc,
     GNUNET_free (wget);
     fprintf (stderr, "\n");
   }
-  GNUNET_SCHEDULER_run (&run, NULL);
+  GNUNET_SCHEDULER_run (&run,
+                        NULL);
   if (run_exchange)
   {
     GNUNET_OS_process_kill (exchanged,
