@@ -274,12 +274,11 @@ handle_deposit_finished (void *cls,
  * @param dki public key information
  * @param amount the amount to be deposited
  * @param h_wire hash of the merchant’s account details
- * @param h_contract hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
+ * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
  * @param coin_pub coin’s public key
  * @param denom_pub denomination key with which the coin is signed
  * @param denom_sig exchange’s unblinded signature of the coin
- * @param timestamp timestamp when the contract was finalized, must match approximately the current time of the exchange
- * @param transaction_id transaction id for the transaction between merchant and customer
+ * @param timestamp timestamp when the deposit was finalized
  * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the exchange (can be zero if refunds are not allowed)
  * @param coin_sig the signature made with purpose #TALER_SIGNATURE_WALLET_COIN_DEPOSIT made by the customer with the coin’s private key.
@@ -289,12 +288,11 @@ static int
 verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
                    const struct TALER_Amount *amount,
                    const struct GNUNET_HashCode *h_wire,
-                   const struct GNUNET_HashCode *h_contract,
+                   const struct GNUNET_HashCode *h_contract_terms,
                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
                    const struct TALER_DenominationSignature *denom_sig,
                    const struct TALER_DenominationPublicKey *denom_pub,
                    struct GNUNET_TIME_Absolute timestamp,
-                   uint64_t transaction_id,
                    const struct TALER_MerchantPublicKeyP *merchant_pub,
                    struct GNUNET_TIME_Absolute refund_deadline,
                    const struct TALER_CoinSpendSignatureP *coin_sig)
@@ -304,11 +302,10 @@ verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
 
   dr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
   dr.purpose.size = htonl (sizeof (struct TALER_DepositRequestPS));
-  dr.h_contract = *h_contract;
+  dr.h_contract_terms = *h_contract_terms;
   dr.h_wire = *h_wire;
   dr.timestamp = GNUNET_TIME_absolute_hton (timestamp);
   dr.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
-  dr.transaction_id = GNUNET_htonll (transaction_id);
   TALER_amount_hton (&dr.amount_with_fee,
                      amount);
   TALER_amount_hton (&dr.deposit_fee,
@@ -321,15 +318,13 @@ verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
                                   &coin_sig->eddsa_signature,
                                   &coin_pub->eddsa_pub))
   {
-    TALER_LOG_WARNING ("Invalid coin signature on /deposit request\n");
+    GNUNET_break_op (0);
+    TALER_LOG_WARNING ("Invalid coin signature on /deposit request!\n");
     {
-      char *s;
-      s = TALER_amount_to_string (amount);
-      TALER_LOG_DEBUG ("... amount_with_fee was %s\n", s);
-      GNUNET_free (s);
-      s = TALER_amount_to_string (&dki->fee_deposit);
-      TALER_LOG_DEBUG ("... deposit_fee was %s\n", s);
-      GNUNET_free (s);
+      TALER_LOG_DEBUG ("... amount_with_fee was %s\n",
+                       TALER_amount2s (amount));
+      TALER_LOG_DEBUG ("... deposit_fee was %s\n",
+                       TALER_amount2s (&dki->fee_deposit));
     }
 
     return GNUNET_SYSERR;
@@ -342,12 +337,14 @@ verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
   if (GNUNET_YES !=
       TALER_test_coin_valid (&coin_info))
   {
+    GNUNET_break_op (0);
     TALER_LOG_WARNING ("Invalid coin passed for /deposit\n");
     return GNUNET_SYSERR;
   }
   if (0 < TALER_amount_cmp (&dki->fee_deposit,
                             amount))
   {
+    GNUNET_break_op (0);
     TALER_LOG_WARNING ("Deposit amount smaller than fee\n");
     return GNUNET_SYSERR;
   }
@@ -374,12 +371,11 @@ verify_signatures (const struct TALER_EXCHANGE_DenomPublicKey *dki,
  * @param wire_deadline date until which the merchant would like the exchange to settle the balance (advisory, the exchange cannot be
  *        forced to settle in the past or upon very short notice, but of course a well-behaved exchange will limit aggregation based on the advice received)
  * @param wire_details the merchant’s account details, in a format supported by the exchange
- * @param h_contract hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
+ * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
  * @param coin_pub coin’s public key
  * @param denom_pub denomination key with which the coin is signed
  * @param denom_sig exchange’s unblinded signature of the coin
- * @param timestamp timestamp when the contract was finalized, must match approximately the current time of the exchange
- * @param transaction_id transaction id for the transaction between merchant and customer
+ * @param timestamp timestamp when the contract was finalized, must not be too far in the future
  * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the exchange (can be zero if refunds are not allowed); must not be after the @a wire_deadline
  * @param coin_sig the signature made with purpose #TALER_SIGNATURE_WALLET_COIN_DEPOSIT made by the customer with the coin’s private key.
@@ -393,12 +389,11 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
                         const struct TALER_Amount *amount,
                         struct GNUNET_TIME_Absolute wire_deadline,
                         json_t *wire_details,
-                        const struct GNUNET_HashCode *h_contract,
+                        const struct GNUNET_HashCode *h_contract_terms,
                         const struct TALER_CoinSpendPublicKeyP *coin_pub,
                         const struct TALER_DenominationSignature *denom_sig,
                         const struct TALER_DenominationPublicKey *denom_pub,
                         struct GNUNET_TIME_Absolute timestamp,
-                        uint64_t transaction_id,
                         const struct TALER_MerchantPublicKeyP *merchant_pub,
                         struct GNUNET_TIME_Absolute refund_deadline,
                         const struct TALER_CoinSpendSignatureP *coin_sig,
@@ -435,12 +430,11 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
       verify_signatures (dki,
                          amount,
                          &h_wire,
-                         h_contract,
+                         h_contract_terms,
                          coin_pub,
                          denom_sig,
                          denom_pub,
                          timestamp,
-                         transaction_id,
                          merchant_pub,
                          refund_deadline,
                          coin_sig))
@@ -450,26 +444,30 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   }
 
   deposit_obj = json_pack ("{s:o, s:O," /* f/wire */
-                           " s:o, s:o," /* H_wire, H_contract */
+                           " s:o, s:o," /* H_wire, h_contract_terms */
                            " s:o, s:o," /* coin_pub, denom_pub */
                            " s:o, s:o," /* ub_sig, timestamp */
-                           " s:I, s:o," /* transaction id, merchant_pub */
+                           " s:o," /* merchant_pub */
                            " s:o, s:o," /* refund_deadline, wire_deadline */
                            " s:o}",     /* coin_sig */
                            "f", TALER_JSON_from_amount (amount),
                            "wire", wire_details,
                            "H_wire", GNUNET_JSON_from_data_auto (&h_wire),
-                           "H_contract", GNUNET_JSON_from_data_auto (h_contract),
+                           "h_contract_terms", GNUNET_JSON_from_data_auto (h_contract_terms),
                            "coin_pub", GNUNET_JSON_from_data_auto (coin_pub),
                            "denom_pub", GNUNET_JSON_from_rsa_public_key (denom_pub->rsa_public_key),
                            "ub_sig", GNUNET_JSON_from_rsa_signature (denom_sig->rsa_signature),
                            "timestamp", GNUNET_JSON_from_time_abs (timestamp),
-                           "transaction_id", (json_int_t) transaction_id,
                            "merchant_pub", GNUNET_JSON_from_data_auto (merchant_pub),
                            "refund_deadline", GNUNET_JSON_from_time_abs (refund_deadline),
                            "wire_transfer_deadline", GNUNET_JSON_from_time_abs (wire_deadline),
                            "coin_sig", GNUNET_JSON_from_data_auto (coin_sig)
                            );
+  if (NULL == deposit_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
 
   dh = GNUNET_new (struct TALER_EXCHANGE_DepositHandle);
   dh->exchange = exchange;
@@ -478,9 +476,8 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   dh->url = MAH_path_to_url (exchange, "/deposit");
   dh->depconf.purpose.size = htonl (sizeof (struct TALER_DepositConfirmationPS));
   dh->depconf.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT);
-  dh->depconf.h_contract = *h_contract;
+  dh->depconf.h_contract_terms = *h_contract_terms;
   dh->depconf.h_wire = h_wire;
-  dh->depconf.transaction_id = GNUNET_htonll (transaction_id);
   dh->depconf.timestamp = GNUNET_TIME_absolute_hton (timestamp);
   dh->depconf.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
   TALER_amount_hton (&dh->depconf.amount_without_fee,
