@@ -413,25 +413,24 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
     struct TALER_AuditorSignatureP auditor_sig;
     struct GNUNET_HashCode denom_h;
     const struct TALER_EXCHANGE_DenomPublicKey *dk;
-    unsigned int j;
-    struct GNUNET_JSON_Specification spec[] = {
+    struct GNUNET_JSON_Specification kspec[] = {
       GNUNET_JSON_spec_fixed_auto ("denom_pub_h",
-                           &denom_h),
+                                   &denom_h),
       GNUNET_JSON_spec_fixed_auto ("auditor_sig",
-                           &auditor_sig),
+                                   &auditor_sig),
       GNUNET_JSON_spec_end()
     };
 
     if (GNUNET_OK !=
         GNUNET_JSON_parse (key,
-                           spec,
+                           kspec,
                            NULL, NULL))
       {
       GNUNET_break_op (0);
       continue;
     }
     dk = NULL;
-    for (j=0;j<key_data->num_denom_keys;j++)
+    for (unsigned int j=0;j<key_data->num_denom_keys;j++)
     {
       if (0 == memcmp (&denom_h,
                        &key_data->denom_keys[j].h_key,
@@ -468,12 +467,14 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
                                     &auditor->auditor_pub.eddsa_pub))
     {
       GNUNET_break_op (0);
-      continue;
+      GNUNET_JSON_parse_free (spec);
+      return GNUNET_SYSERR;
     }
     auditor->denom_keys[off] = dk;
     off++;
   }
   auditor->num_denom_keys = off;
+  GNUNET_JSON_parse_free (spec);
   return GNUNET_OK;
 }
 
@@ -503,7 +504,10 @@ decode_keys_json (const json_t *resp_obj,
   hash_context = GNUNET_CRYPTO_hash_context_start ();
   /* parse the master public key and issue date of the response */
   {
+    const char *ver;
     struct GNUNET_JSON_Specification spec[] = {
+      GNUNET_JSON_spec_string ("version",
+                               &ver),
       GNUNET_JSON_spec_fixed_auto ("master_public_key",
                                    &key_data->master_pub),
       GNUNET_JSON_spec_fixed_auto ("eddsa_sig",
@@ -519,6 +523,7 @@ decode_keys_json (const json_t *resp_obj,
             GNUNET_JSON_parse (resp_obj,
                                spec,
                                NULL, NULL));
+    key_data->version = GNUNET_strdup (ver);
   }
 
   /* parse the signing keys */
@@ -594,6 +599,7 @@ decode_keys_json (const json_t *resp_obj,
                                     auditor_info,
                                     key_data));
       }
+      key_data->num_auditors = len;
     }
   }
 
@@ -629,19 +635,26 @@ decode_keys_json (const json_t *resp_obj,
 static void
 free_key_data (struct TALER_EXCHANGE_Keys *key_data)
 {
-  unsigned int i;
-
   GNUNET_array_grow (key_data->sign_keys,
                      key_data->num_sign_keys,
                      0);
-  for (i=0;i<key_data->num_denom_keys;i++)
+  for (unsigned int i=0;i<key_data->num_denom_keys;i++)
     GNUNET_CRYPTO_rsa_public_key_free (key_data->denom_keys[i].key.rsa_public_key);
   GNUNET_array_grow (key_data->denom_keys,
                      key_data->num_denom_keys,
                      0);
+  for (unsigned int i=0;i<key_data->num_auditors;i++)
+  {
+    GNUNET_array_grow (key_data->auditors[i].denom_keys,
+                       key_data->auditors[i].num_denom_keys,
+                       0);
+    GNUNET_free (key_data->auditors[i].auditor_url);
+  }
   GNUNET_array_grow (key_data->auditors,
                      key_data->num_auditors,
                      0);
+  GNUNET_free_non_null (key_data->version);
+  key_data->version = NULL;
 }
 
 
@@ -784,15 +797,15 @@ MAH_handle_is_ready (struct TALER_EXCHANGE_Handle *h)
 /**
  * Obtain the URL to use for an API request.
  *
- * @param base_url base URL of the exchange (i.e. "http://exchange/")
+ * @param h handle for the exchange
  * @param path Taler API path (i.e. "/reserve/withdraw")
  * @return the full URI to use with cURL
  */
 char *
-MAH_path_to_url (struct TALER_EXCHANGE_Handle *exchange,
+MAH_path_to_url (struct TALER_EXCHANGE_Handle *h,
                  const char *path)
 {
-  return MAH_path_to_url2 (exchange->url,
+  return MAH_path_to_url2 (h->url,
                            path);
 }
 
@@ -840,8 +853,8 @@ parse_date_string (const char *date,
       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"  };
   struct tm now;
   time_t t;
-  char day[3];
-  char mon[3];
+  char day[4];
+  char mon[4];
   unsigned int i;
   unsigned int mday;
   unsigned int year;
@@ -873,7 +886,7 @@ parse_date_string (const char *date,
   for (i=0;i<12;i++)
     if (0 == strcasecmp (mons[i], mon))
       now.tm_mon = i;
-  if ( (7 == now.tm_mday) ||
+  if ( (7 == now.tm_wday) ||
        (12 == now.tm_mon) )
     return GNUNET_SYSERR;
   t = mktime (&now);
