@@ -220,8 +220,7 @@ handle_refund_finished (void *cls,
  *        (as that fee is still being subtracted), and smaller than the amount
  *        (with deposit fee) of the original deposit contribution of this coin
  * @param refund_fee fee applicable to this coin for the refund
- * @param h_contract hash of the contact of the merchant with the customer that is being refunded
- * @param transaction_id transaction id for the transaction being refunded, must match @a h_contract
+ * @param h_contract_terms hash of the contact of the merchant with the customer that is being refunded
  * @param coin_pub coin’s public key of the coin from the original deposit operation
  * @param rtransaction_id transaction id for the transaction between merchant and customer (of refunding operation);
  *                        this is needed as we may first do a partial refund and later a full refund.  If both
@@ -237,8 +236,7 @@ struct TALER_EXCHANGE_RefundHandle *
 TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
 		       const struct TALER_Amount *amount,
 		       const struct TALER_Amount *refund_fee,
-		       const struct GNUNET_HashCode *h_contract,
-		       uint64_t transaction_id,
+		       const struct GNUNET_HashCode *h_contract_terms,
 		       const struct TALER_CoinSpendPublicKeyP *coin_pub,
 		       uint64_t rtransaction_id,
 		       const struct TALER_MerchantPrivateKeyP *merchant_priv,
@@ -256,8 +254,7 @@ TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
 		 MAH_handle_is_ready (exchange));
   rr.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND);
   rr.purpose.size = htonl (sizeof (struct TALER_RefundRequestPS));
-  rr.h_contract = *h_contract;
-  rr.transaction_id = GNUNET_htonll (transaction_id);
+  rr.h_contract_terms = *h_contract_terms;
   rr.coin_pub = *coin_pub;
   GNUNET_CRYPTO_eddsa_key_get_public (&merchant_priv->eddsa_priv,
                                       &rr.merchant.eddsa_pub);
@@ -271,20 +268,23 @@ TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
                                            &rr.purpose,
                                            &merchant_sig.eddsa_sig));
   refund_obj = json_pack ("{s:o, s:o," /* amount/fee */
-			  " s:o, s:o," /* H_contract, coin_pub */
-			  " s:I, s:I," /* transaction id, rtransaction id */
+			  " s:o, s:o," /* h_contract_terms, coin_pub */
+			  " s:I," /* rtransaction id */
 			  " s:o, s:o}", /* merchant_pub, merchant_sig */
 			  "refund_amount", TALER_JSON_from_amount (amount),
 			  "refund_fee", TALER_JSON_from_amount (refund_fee),
-			  "H_contract", GNUNET_JSON_from_data_auto (h_contract),
+			  "h_contract_terms", GNUNET_JSON_from_data_auto (h_contract_terms),
 			  "coin_pub", GNUNET_JSON_from_data_auto (coin_pub),
-			  "transaction_id", (json_int_t) transaction_id,
 			  "rtransaction_id", (json_int_t) rtransaction_id,
 			  "merchant_pub", GNUNET_JSON_from_data_auto (&rr.merchant),
 			  "merchant_sig", GNUNET_JSON_from_data_auto (&merchant_sig)
 			  );
-  GNUNET_assert (NULL != refund_obj);
-  
+  if (NULL == refund_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+
   rh = GNUNET_new (struct TALER_EXCHANGE_RefundHandle);
   rh->exchange = exchange;
   rh->cb = cb;
@@ -292,8 +292,7 @@ TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
   rh->url = MAH_path_to_url (exchange, "/refund");
   rh->depconf.purpose.size = htonl (sizeof (struct TALER_RefundConfirmationPS));
   rh->depconf.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND);
-  rh->depconf.h_contract = *h_contract;
-  rh->depconf.transaction_id = GNUNET_htonll (transaction_id);
+  rh->depconf.h_contract_terms = *h_contract_terms;
   rh->depconf.coin_pub = *coin_pub;
   rh->depconf.merchant = rr.merchant;
   rh->depconf.rtransaction_id = GNUNET_htonll (rtransaction_id);
