@@ -77,7 +77,8 @@ struct TALER_EXCHANGE_ReserveStatusHandle
  * Parse history given in JSON format and return it in binary
  * format.
  *
- * @param[in] history JSON array with the history
+ * @param exchange connection to the exchange we can use
+ * @param history JSON array with the history
  * @param reserve_pub public key of the reserve to inspect
  * @param currency currency we expect the balance to be in
  * @param[out] balance final balance
@@ -89,7 +90,8 @@ struct TALER_EXCHANGE_ReserveStatusHandle
  *         #GNUNET_SYSERR if there was a protocol violation in @a history
  */
 static int
-parse_reserve_history (const json_t *history,
+parse_reserve_history (struct TALER_EXCHANGE_Handle *exchange,
+                       const json_t *history,
                        const struct TALER_ReservePublicKeyP *reserve_pub,
                        const char *currency,
                        struct TALER_Amount *balance,
@@ -136,7 +138,17 @@ parse_reserve_history (const json_t *history,
                          "DEPOSIT"))
     {
       json_t *wire_account;
-      json_t *transfer;
+      void *wire_reference;
+      size_t wire_reference_size;
+
+      struct GNUNET_JSON_Specification withdraw_spec[] = {
+        GNUNET_JSON_spec_varsize ("wire_reference",
+                                  &wire_reference,
+                                  &wire_reference_size),
+        GNUNET_JSON_spec_json ("sender_account_details",
+                               &wire_account),
+        GNUNET_JSON_spec_end()
+      };
 
       rhistory[off].type = TALER_EXCHANGE_RTT_DEPOSIT;
       if (GNUNET_OK !=
@@ -148,33 +160,17 @@ parse_reserve_history (const json_t *history,
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
-      wire_account = json_object_get (transaction,
-                                      "sender_account_details");
-      /* check 'wire_account' is a JSON object (no need to check wireformat,
-         but we do at least expect "some" JSON object here) */
-      if ( (NULL == wire_account) ||
-           (! json_is_object (wire_account)) )
-      {
-        /* not even a JSON 'wire' specification, not acceptable */
-        GNUNET_break_op (0);
-        if (NULL != wire_account)
-          json_decref (wire_account);
-        return GNUNET_SYSERR;
-      }
-      transfer = json_object_get (transaction,
-                                  "transfer_details");
-      /* check 'transfer' is a JSON object */
-      if ( (NULL == transfer) ||
-           (! json_is_object (transfer)) )
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (transaction,
+                             withdraw_spec,
+                             NULL, NULL))
       {
         GNUNET_break_op (0);
-        json_decref (wire_account);
-        if (NULL != transfer)
-          json_decref (transfer);
         return GNUNET_SYSERR;
       }
       rhistory[off].details.in_details.sender_account_details = wire_account;
-      rhistory[off].details.in_details.transfer_details = transfer;
+      rhistory[off].details.in_details.wire_reference = wire_reference;
+      rhistory[off].details.in_details.wire_reference_size = wire_reference_size;
       /* end type==DEPOSIT */
     }
     else if (0 == strcasecmp (type,
@@ -182,12 +178,15 @@ parse_reserve_history (const json_t *history,
     {
       struct TALER_ReserveSignatureP sig;
       struct TALER_WithdrawRequestPS withdraw_purpose;
-      struct TALER_Amount amount_from_purpose;
       struct GNUNET_JSON_Specification withdraw_spec[] = {
-        GNUNET_JSON_spec_fixed_auto ("signature",
+        GNUNET_JSON_spec_fixed_auto ("reserve_sig",
                                      &sig),
-        GNUNET_JSON_spec_fixed_auto ("details",
-                                     &withdraw_purpose),
+	TALER_JSON_spec_amount_nbo ("withdraw_fee",
+				    &withdraw_purpose.withdraw_fee),
+        GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
+                                     &withdraw_purpose.h_denomination_pub),
+        GNUNET_JSON_spec_fixed_auto ("h_coin_envelope",
+                                     &withdraw_purpose.h_coin_envelope),
         GNUNET_JSON_spec_end()
       };
       unsigned int i;
@@ -201,6 +200,13 @@ parse_reserve_history (const json_t *history,
         GNUNET_break_op (0);
         return GNUNET_SYSERR;
       }
+      withdraw_purpose.purpose.size
+	= htonl (sizeof (withdraw_purpose));
+      withdraw_purpose.purpose.purpose
+	= htonl (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW);
+      withdraw_purpose.reserve_pub = *reserve_pub;
+      TALER_amount_hton (&withdraw_purpose.amount_with_fee,
+			 &amount);
       /* Check that the signature is a valid withdraw request */
       if (GNUNET_OK !=
           GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW,
@@ -212,17 +218,10 @@ parse_reserve_history (const json_t *history,
         GNUNET_JSON_parse_free (withdraw_spec);
         return GNUNET_SYSERR;
       }
-      TALER_amount_ntoh (&amount_from_purpose,
-                         &withdraw_purpose.amount_with_fee);
-      if (0 != TALER_amount_cmp (&amount,
-                                 &amount_from_purpose))
-      {
-        GNUNET_break_op (0);
-        GNUNET_JSON_parse_free (withdraw_spec);
-        return GNUNET_SYSERR;
-      }
-      rhistory[off].details.out_authorization_sig = json_object_get (transaction,
-                                                                     "signature");
+      /* TODO: check that withdraw fee matches expectations! */
+      rhistory[off].details.out_authorization_sig
+	= json_object_get (transaction,
+			   "signature");
       /* Check check that the same withdraw transaction
          isn't listed twice by the exchange. We use the
          "uuid" array to remember the hashes of all
@@ -256,6 +255,142 @@ parse_reserve_history (const json_t *history,
       }
       /* end type==WITHDRAW */
     }
+    else if (0 == strcasecmp (type,
+                              "PAYBACK"))
+    {
+      struct TALER_PaybackConfirmationPS pc;
+      struct GNUNET_TIME_Absolute timestamp;
+      const struct TALER_EXCHANGE_Keys *key_state;
+      struct GNUNET_JSON_Specification payback_spec[] = {
+        GNUNET_JSON_spec_fixed_auto ("coin_pub",
+                                     &pc.coin_pub),
+        GNUNET_JSON_spec_fixed_auto ("exchange_sig",
+                                     &rhistory[off].details.payback_details.exchange_sig),
+        GNUNET_JSON_spec_fixed_auto ("exchange_pub",
+                                     &rhistory[off].details.payback_details.exchange_pub),
+        GNUNET_JSON_spec_absolute_time_nbo ("timestamp",
+					    &pc.timestamp),
+        GNUNET_JSON_spec_end()
+      };
+
+      rhistory[off].type = TALER_EXCHANGE_RTT_PAYBACK;
+      rhistory[off].amount = amount;
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (transaction,
+                             payback_spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      rhistory[off].details.payback_details.coin_pub = pc.coin_pub;
+      TALER_amount_hton (&pc.payback_amount,
+			 &amount);
+      pc.purpose.size = htonl (sizeof (pc));
+      pc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK);
+      pc.reserve_pub = *reserve_pub;
+      timestamp = GNUNET_TIME_absolute_ntoh (pc.timestamp);
+      rhistory[off].details.payback_details.timestamp = timestamp;
+
+      key_state = TALER_EXCHANGE_get_keys (exchange);
+      if (GNUNET_OK !=
+          TALER_EXCHANGE_test_signing_key (key_state,
+                                           &rhistory[off].details.payback_details.exchange_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      if (GNUNET_OK !=
+          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK,
+                                      &pc.purpose,
+                                      &rhistory[off].details.payback_details.exchange_sig.eddsa_signature,
+                                      &rhistory[off].details.payback_details.exchange_pub.eddsa_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      if (GNUNET_OK !=
+          TALER_amount_add (&total_in,
+                            &total_in,
+                            &rhistory[off].amount))
+      {
+        /* overflow in history already!? inconceivable! Bad exchange! */
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      /* end type==PAYBACK */
+    }
+    else if (0 == strcasecmp (type,
+                              "CLOSING"))
+    {
+      const struct TALER_EXCHANGE_Keys *key_state;
+      struct TALER_ReserveCloseConfirmationPS rcc;
+      struct GNUNET_TIME_Absolute timestamp;
+      struct GNUNET_JSON_Specification closing_spec[] = {
+        GNUNET_JSON_spec_json ("receiver_account_details",
+			       &rhistory[off].details.close_details.receiver_account_details),
+        GNUNET_JSON_spec_fixed_auto ("wtid",
+				     &rhistory[off].details.close_details.wtid),
+        GNUNET_JSON_spec_fixed_auto ("exchange_sig",
+                                     &rhistory[off].details.close_details.exchange_sig),
+        GNUNET_JSON_spec_fixed_auto ("exchange_pub",
+                                     &rhistory[off].details.close_details.exchange_pub),
+	TALER_JSON_spec_amount_nbo ("closing_fee",
+				    &rcc.closing_fee),
+        GNUNET_JSON_spec_absolute_time_nbo ("timestamp",
+					    &rcc.timestamp),
+        GNUNET_JSON_spec_end()
+      };
+
+      rhistory[off].type = TALER_EXCHANGE_RTT_CLOSE;
+      rhistory[off].amount = amount;
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (transaction,
+                             closing_spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      TALER_amount_hton (&rcc.closing_amount,
+			 &amount);
+      TALER_JSON_hash (rhistory[off].details.close_details.receiver_account_details,
+		       &rcc.h_wire);
+      rcc.wtid = rhistory[off].details.close_details.wtid;
+      rcc.purpose.size = htonl (sizeof (rcc));
+      rcc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED);
+      rcc.reserve_pub = *reserve_pub;
+      timestamp = GNUNET_TIME_absolute_ntoh (rcc.timestamp);
+      rhistory[off].details.close_details.timestamp = timestamp;
+
+      key_state = TALER_EXCHANGE_get_keys (exchange);
+      if (GNUNET_OK !=
+          TALER_EXCHANGE_test_signing_key (key_state,
+                                           &rhistory[off].details.payback_details.exchange_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      if (GNUNET_OK !=
+          GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED,
+                                      &rcc.purpose,
+                                      &rhistory[off].details.payback_details.exchange_sig.eddsa_signature,
+                                      &rhistory[off].details.payback_details.exchange_pub.eddsa_pub))
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      if (GNUNET_OK !=
+          TALER_amount_add (&total_out,
+                            &total_out,
+                            &rhistory[off].amount))
+      {
+        /* overflow in history already!? inconceivable! Bad exchange! */
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      /* end type==CLOSING */
+    }
     else
     {
       /* unexpected 'type', protocol incompatibility, complain! */
@@ -274,11 +409,42 @@ parse_reserve_history (const json_t *history,
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-
   return GNUNET_OK;
 }
 
 
+/**
+ * Free memory (potentially) allocated by #parse_reserve_history().
+ *
+ * @param rhistory result to free
+ * @param len number of entries in @a rhistory
+ */
+static void
+free_rhistory (struct TALER_EXCHANGE_ReserveHistory *rhistory,
+               unsigned int len)
+{
+  for (unsigned int i=0;i<len;i++)
+  {
+    switch (rhistory[i].type)
+    {
+    case TALER_EXCHANGE_RTT_DEPOSIT:
+      GNUNET_free_non_null (rhistory[i].details.in_details.wire_reference);
+      if (NULL != rhistory[i].details.in_details.sender_account_details)
+        json_decref (rhistory[i].details.in_details.sender_account_details);
+      break;
+    case TALER_EXCHANGE_RTT_WITHDRAWAL:
+      break;
+    case TALER_EXCHANGE_RTT_PAYBACK:
+      break;
+    case TALER_EXCHANGE_RTT_CLOSE:
+      if (NULL != rhistory[i].details.close_details.receiver_account_details)
+        json_decref (rhistory[i].details.close_details.receiver_account_details);
+      break;
+    }
+  }
+}
+
+
 /**
  * Function called when we're done processing the
  * HTTP /reserve/status request.
@@ -292,9 +458,9 @@ handle_reserve_status_finished (void *cls,
                                 long response_code,
                                 const json_t *json)
 {
-  struct TALER_EXCHANGE_ReserveStatusHandle *wsh = cls;
+  struct TALER_EXCHANGE_ReserveStatusHandle *rsh = cls;
 
-  wsh->job = NULL;
+  rsh->job = NULL;
   switch (response_code)
   {
   case 0:
@@ -333,9 +499,11 @@ handle_reserve_status_finished (void *cls,
       {
         struct TALER_EXCHANGE_ReserveHistory rhistory[len];
 
+        memset (rhistory, 0, sizeof (rhistory));
         if (GNUNET_OK !=
-            parse_reserve_history (history,
-                                   &wsh->reserve_pub,
+            parse_reserve_history (rsh->exchange,
+                                   history,
+                                   &rsh->reserve_pub,
                                    balance.currency,
                                    &balance_from_history,
                                    len,
@@ -343,25 +511,29 @@ handle_reserve_status_finished (void *cls,
         {
           GNUNET_break_op (0);
           response_code = 0;
-          break;
         }
-        if (0 !=
-            TALER_amount_cmp (&balance_from_history,
-                              &balance))
+        if ( (0 != response_code) &&
+             (0 !=
+              TALER_amount_cmp (&balance_from_history,
+                                &balance)) )
         {
           /* exchange cannot add up balances!? */
           GNUNET_break_op (0);
           response_code = 0;
-          break;
         }
-        wsh->cb (wsh->cb_cls,
-                 response_code,
-		 TALER_EC_NONE,
-                 json,
-                 &balance,
-                 len,
-                 rhistory);
-        wsh->cb = NULL;
+        if (0 != response_code)
+        {
+          rsh->cb (rsh->cb_cls,
+                   response_code,
+                   TALER_EC_NONE,
+                   json,
+                   &balance,
+                   len,
+                   rhistory);
+          rsh->cb = NULL;
+        }
+        free_rhistory (rhistory,
+                       len);
       }
     }
     break;
@@ -386,14 +558,14 @@ handle_reserve_status_finished (void *cls,
     response_code = 0;
     break;
   }
-  if (NULL != wsh->cb)
-    wsh->cb (wsh->cb_cls,
+  if (NULL != rsh->cb)
+    rsh->cb (rsh->cb_cls,
              response_code,
 	     TALER_JSON_get_error_code (json),
              json,
              NULL,
              0, NULL);
-  TALER_EXCHANGE_reserve_status_cancel (wsh);
+  TALER_EXCHANGE_reserve_status_cancel (rsh);
 }
 
 
@@ -419,7 +591,7 @@ TALER_EXCHANGE_reserve_status (struct TALER_EXCHANGE_Handle *exchange,
                                TALER_EXCHANGE_ReserveStatusResultCallback cb,
                                void *cb_cls)
 {
-  struct TALER_EXCHANGE_ReserveStatusHandle *wsh;
+  struct TALER_EXCHANGE_ReserveStatusHandle *rsh;
   struct GNUNET_CURL_Context *ctx;
   CURL *eh;
   char *pub_str;
@@ -437,12 +609,12 @@ TALER_EXCHANGE_reserve_status (struct TALER_EXCHANGE_Handle *exchange,
                    "/reserve/status?reserve_pub=%s",
                    pub_str);
   GNUNET_free (pub_str);
-  wsh = GNUNET_new (struct TALER_EXCHANGE_ReserveStatusHandle);
-  wsh->exchange = exchange;
-  wsh->cb = cb;
-  wsh->cb_cls = cb_cls;
-  wsh->reserve_pub = *reserve_pub;
-  wsh->url = MAH_path_to_url (exchange,
+  rsh = GNUNET_new (struct TALER_EXCHANGE_ReserveStatusHandle);
+  rsh->exchange = exchange;
+  rsh->cb = cb;
+  rsh->cb_cls = cb_cls;
+  rsh->reserve_pub = *reserve_pub;
+  rsh->url = MAH_path_to_url (exchange,
                               arg_str);
   GNUNET_free (arg_str);
 
@@ -450,33 +622,33 @@ TALER_EXCHANGE_reserve_status (struct TALER_EXCHANGE_Handle *exchange,
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
                                    CURLOPT_URL,
-                                   wsh->url));
+                                   rsh->url));
   ctx = MAH_handle_to_context (exchange);
-  wsh->job = GNUNET_CURL_job_add (ctx,
+  rsh->job = GNUNET_CURL_job_add (ctx,
                           eh,
                           GNUNET_NO,
                           &handle_reserve_status_finished,
-                          wsh);
-  return wsh;
+                          rsh);
+  return rsh;
 }
 
 
 /**
- * Cancel a withdraw status request.  This function cannot be used
+ * Cancel a reserve status request.  This function cannot be used
  * on a request handle if a response is already served for it.
  *
- * @param wsh the withdraw status request handle
+ * @param rsh the reserve status request handle
  */
 void
-TALER_EXCHANGE_reserve_status_cancel (struct TALER_EXCHANGE_ReserveStatusHandle *wsh)
+TALER_EXCHANGE_reserve_status_cancel (struct TALER_EXCHANGE_ReserveStatusHandle *rsh)
 {
-  if (NULL != wsh->job)
+  if (NULL != rsh->job)
   {
-    GNUNET_CURL_job_cancel (wsh->job);
-    wsh->job = NULL;
+    GNUNET_CURL_job_cancel (rsh->job);
+    rsh->job = NULL;
   }
-  GNUNET_free (wsh->url);
-  GNUNET_free (wsh);
+  GNUNET_free (rsh->url);
+  GNUNET_free (rsh);
 }
 
 
@@ -651,7 +823,8 @@ reserve_withdraw_payment_required (struct TALER_EXCHANGE_ReserveWithdrawHandle *
     struct TALER_EXCHANGE_ReserveHistory rhistory[len];
 
     if (GNUNET_OK !=
-        parse_reserve_history (history,
+        parse_reserve_history (wsh->exchange,
+                               history,
                                &wsh->reserve_pub,
                                balance.currency,
                                &balance_from_history,
@@ -659,8 +832,12 @@ reserve_withdraw_payment_required (struct TALER_EXCHANGE_ReserveWithdrawHandle *
                                rhistory))
     {
       GNUNET_break_op (0);
+      free_rhistory (rhistory,
+                     len);
       return GNUNET_SYSERR;
     }
+    free_rhistory (rhistory,
+                   len);
   }
 
   if (0 !=
@@ -717,7 +894,7 @@ handle_reserve_withdraw_finished (void *cls,
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
         reserve_withdraw_ok (wsh,
-                          json))
+                             json))
     {
       GNUNET_break_op (0);
       response_code = 0;
@@ -872,6 +1049,12 @@ TALER_EXCHANGE_reserve_withdraw (struct TALER_EXCHANGE_Handle *exchange,
                             "reserve_pub", GNUNET_JSON_from_data_auto (&wsh->reserve_pub),
                             "reserve_sig", GNUNET_JSON_from_data_auto (&reserve_sig));
   GNUNET_free (coin_ev);
+  if (NULL == withdraw_obj)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+
 
   wsh->blinding_key = *blinding_key;
   wsh->url = MAH_path_to_url (exchange, "/reserve/withdraw");
