@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+  Copyright (C) 2014-2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -23,6 +23,7 @@
  */
 #include <platform.h>
 #include "taler_exchangedb_lib.h"
+#include "taler_wire_lib.h"
 
 /**
  * When generating filenames from a cryptographic hash, we do not use
@@ -184,9 +185,9 @@ static FILE *auditor_output_file;
 static char *exchange_directory;
 
 /**
- * Time to pretend when the key update is executed.
+ * Directory where we should write the wire transfer fee structure.
  */
-static char *pretend_time_str;
+static char *feedir;
 
 /**
  * Handle to the exchange's configuration
@@ -214,6 +215,16 @@ static struct TALER_MasterPublicKeyP master_public_key;
  */
 static struct GNUNET_TIME_Absolute lookahead_sign_stamp;
 
+/**
+ * Largest duration for spending of any key.
+ */
+static struct GNUNET_TIME_Relative max_duration_spend;
+
+/**
+ * Revoke denomination key identified by this hash (if non-zero).
+ */
+static struct GNUNET_HashCode revoke_dkh;
+
 /**
  * Return value from main().
  */
@@ -598,6 +609,8 @@ get_cointype_params (const char *ct,
     return GNUNET_SYSERR;
   }
   GNUNET_TIME_round_rel (&params->duration_spend);
+  max_duration_spend = GNUNET_TIME_relative_max (max_duration_spend,
+                                                 params->duration_spend);
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_time (kcfg,
                                            ct,
@@ -818,7 +831,7 @@ exchange_keys_update_cointype (void *cls,
       return;
     }
     if ( (NULL != auditor_output_file) &&
-         (sizeof (denomkey_issue.issue.properties) !=
+         (1 !=
           fwrite (&denomkey_issue.issue.properties,
                   sizeof (struct TALER_DenominationKeyValidityPS),
                   1,
@@ -861,6 +874,297 @@ exchange_keys_update_denomkeys ()
 }
 
 
+/**
+ * Sign @a af with @a priv
+ *
+ * @param[in,out] af fee structure to sign
+ * @param wireplugin name of the plugin for which we sign
+ * @param priv private key to use for signing
+ */
+static void
+sign_af (struct TALER_EXCHANGEDB_AggregateFees *af,
+         const char *wireplugin,
+         const struct GNUNET_CRYPTO_EddsaPrivateKey *priv)
+{
+  struct TALER_MasterWireFeePS wf;
+
+  TALER_EXCHANGEDB_fees_2_wf (wireplugin,
+                              af,
+                              &wf);
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_CRYPTO_eddsa_sign (priv,
+                                           &wf.purpose,
+                                           &af->master_sig.eddsa_signature));
+}
+
+
+/**
+ * Output the wire fee structure.  Must be run after #max_duration_spend
+ * was initialized.
+ *
+ * @param cls pointer to `int`, set to #GNUNET_SYSERR on error
+ * @param wiremethod method to write fees for
+ */
+static void
+create_wire_fee_for_method (void *cls,
+                            const char *wiremethod)
+{
+  int *ret = cls;
+  struct TALER_EXCHANGEDB_AggregateFees *af_head;
+  struct TALER_EXCHANGEDB_AggregateFees *af_tail;
+  unsigned int year;
+  struct GNUNET_TIME_Absolute last_date;
+  struct GNUNET_TIME_Absolute start_date;
+  struct GNUNET_TIME_Absolute end_date;
+  char yearstr[12];
+  char *fn;
+  char *section;
+
+  if (GNUNET_OK != *ret)
+    return;
+  last_date = GNUNET_TIME_absolute_add (lookahead_sign_stamp,
+                                        max_duration_spend);
+  GNUNET_asprintf (&section,
+                   "exchange-wire-%s",
+                   wiremethod);
+  GNUNET_asprintf (&fn,
+                   "%s%s.fee",
+                   feedir,
+                   wiremethod);
+  af_head = NULL;
+  af_tail = NULL;
+  year = GNUNET_TIME_get_current_year ();
+  start_date = GNUNET_TIME_year_to_time (year);
+  while (start_date.abs_value_us < last_date.abs_value_us)
+  {
+    struct TALER_EXCHANGEDB_AggregateFees *af;
+    char *opt;
+    char *amounts;
+
+    GNUNET_snprintf (yearstr,
+                     sizeof (yearstr),
+                     "%u",
+                     year);
+    end_date = GNUNET_TIME_year_to_time (year + 1);
+    af = GNUNET_new (struct TALER_EXCHANGEDB_AggregateFees);
+    af->start_date = start_date;
+    af->end_date = end_date;
+
+    /* handle wire fee */
+    GNUNET_asprintf (&opt,
+                     "wire-fee-%u",
+                     year);
+    if (GNUNET_OK !=
+        GNUNET_CONFIGURATION_get_value_string (kcfg,
+                                               section,
+                                               opt,
+                                               &amounts))
+    {
+      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                                 section,
+                                 opt);
+      *ret = GNUNET_SYSERR;
+      GNUNET_free (opt);
+      break;
+    }
+    if (GNUNET_OK !=
+        TALER_string_to_amount (amounts,
+                                &af->wire_fee))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Invalid amount `%s' specified in `%s' under `%s'\n",
+                  amounts,
+                  wiremethod,
+                  opt);
+      *ret = GNUNET_SYSERR;
+      GNUNET_free (amounts);
+      GNUNET_free (opt);
+      break;
+    }
+    GNUNET_free (amounts);
+    GNUNET_free (opt);
+
+    /* handle closing fee */
+    GNUNET_asprintf (&opt,
+                     "closing-fee-%u",
+                     year);
+    if (GNUNET_OK !=
+        GNUNET_CONFIGURATION_get_value_string (kcfg,
+                                               section,
+                                               opt,
+                                               &amounts))
+    {
+      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                                 section,
+                                 opt);
+      *ret = GNUNET_SYSERR;
+      GNUNET_free (opt);
+      break;
+    }
+    if (GNUNET_OK !=
+        TALER_string_to_amount (amounts,
+                                &af->closing_fee))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Invalid amount `%s' specified in `%s' under `%s'\n",
+                  amounts,
+                  wiremethod,
+                  opt);
+      *ret = GNUNET_SYSERR;
+      GNUNET_free (amounts);
+      GNUNET_free (opt);
+      break;
+    }
+    GNUNET_free (amounts);
+
+    GNUNET_free (opt);
+    sign_af (af,
+             wiremethod,
+             &master_priv.eddsa_priv);
+    if (NULL == af_tail)
+      af_head = af;
+    else
+      af_tail->next = af;
+    af_tail = af;
+    start_date = end_date;
+    year++;
+  }
+  if ( (GNUNET_OK == *ret) &&
+       (GNUNET_OK !=
+        TALER_EXCHANGEDB_fees_write (fn,
+                                     wiremethod,
+                                     af_head)) )
+    *ret = GNUNET_SYSERR;
+  GNUNET_free (section);
+  GNUNET_free (fn);
+  TALER_EXCHANGEDB_fees_free (af_head);
+}
+
+
+/**
+ * Output the wire fee structure.  Must be run after #max_duration_spend
+ * was initialized.
+ *
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
+ */
+static int
+create_wire_fees ()
+{
+  int ret;
+
+  ret = GNUNET_OK;
+  TALER_WIRE_find_enabled (kcfg,
+                           &create_wire_fee_for_method,
+                           &ret);
+  return ret;
+}
+
+
+/**
+ * Closure for functions processing a request to revoke a denomination
+ * public key and request the wallets to initiate /payback.
+ */
+struct RevokeClosure
+{
+  /**
+   * Hash of the denomination public key to revoke.
+   */
+  const struct GNUNET_HashCode *hc;
+
+  /**
+   * Base directory for keys.
+   */
+  char *basedir;
+
+  /**
+   * Set to #GNUNET_OK if we found a matching key,
+   * Set to #GNUNET_SYSERR on error.
+   */
+  int ok;
+};
+
+
+
+/**
+ * Revoke denomination keys matching the given hash.
+ *
+ * @param cls a `struct RevokeClosure` with information about what to revoke
+ * @param dki the denomination key
+ * @param alias coin alias
+ * @param revocation_master_sig non-NULL if @a dki was revoked
+ * @return #GNUNET_OK to continue to iterate,
+ *  #GNUNET_NO to stop iteration with no error,
+ *  #GNUNET_SYSERR to abort iteration with error!
+ */
+static int
+exchange_keys_revoke_by_dki (void *cls,
+                             const char *alias,
+                             const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki,
+                             const struct TALER_MasterSignatureP *revocation_master_sig)
+{
+  struct RevokeClosure *rc = cls;
+
+  if (NULL != revocation_master_sig)
+    return GNUNET_OK; /* refuse to do it twice */
+  if (0 != memcmp (rc->hc,
+                   &dki->issue.properties.denom_hash,
+                   sizeof (struct GNUNET_HashCode)))
+    return GNUNET_OK;
+  rc->ok = GNUNET_OK;
+  if (GNUNET_OK !=
+      TALER_EXCHANGEDB_denomination_key_revoke (rc->basedir,
+                                                alias,
+                                                dki,
+                                                &master_priv))
+  {
+    rc->ok = GNUNET_SYSERR;
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_NO;
+}
+
+
+/**
+ * Revoke the denomination key matching @a hc and request /payback to be
+ * initiated.
+ *
+ * @param hc denomination key hash to revoke
+ * @return #GNUNET_OK on success,
+ *         #GNUNET_NO if @a hc was not found
+ *         #GNUNET_SYSERR on error
+ */
+static int
+revoke_denomination (const struct GNUNET_HashCode *hc)
+{
+  struct RevokeClosure rc;
+
+  rc.hc = hc;
+  rc.ok = GNUNET_NO;
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_filename (kcfg,
+                                               "exchange",
+                                               "KEYDIR",
+                                               &rc.basedir))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "exchange",
+                               "KEYDIR");
+    return GNUNET_SYSERR;
+  }
+  if (-1 ==
+      TALER_EXCHANGEDB_denomination_keys_iterate (rc.basedir,
+                                                  &master_public_key,
+                                                  &exchange_keys_revoke_by_dki,
+                                                  &rc))
+  {
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+  GNUNET_free (rc.basedir);
+  return rc.ok;
+}
+
+
 /**
  * Main function that will be run.
  *
@@ -875,26 +1179,33 @@ run (void *cls,
      const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
+  static struct GNUNET_HashCode zero;
   struct GNUNET_TIME_Relative lookahead_sign;
   struct GNUNET_CRYPTO_EddsaPrivateKey *eddsa_priv;
 
   kcfg = cfg;
-  if (NULL != pretend_time_str)
+  if (NULL == feedir)
   {
     if (GNUNET_OK !=
-        GNUNET_STRINGS_fancy_time_to_absolute (pretend_time_str,
-                                               &now))
+        GNUNET_CONFIGURATION_get_value_filename (kcfg,
+                                                 "exchangedb",
+                                                 "WIREFEE_BASE_DIR",
+                                                 &feedir))
     {
       fprintf (stderr,
-               "timestamp `%s' invalid\n",
-               pretend_time_str);
+               "Wire fee directory not given in neither configuration nor command-line\n");
       global_ret = 1;
       return;
     }
   }
-  else
+  if (GNUNET_OK !=
+      GNUNET_DISK_directory_create (feedir))
   {
-    now = GNUNET_TIME_absolute_get ();
+    GNUNET_log_strerror_file (GNUNET_ERROR_TYPE_ERROR,
+                              "mkdir",
+                              feedir);
+    global_ret = 1;
+    return;
   }
   GNUNET_TIME_round_abs (&now);
   if ( (NULL == masterkeyfile) &&
@@ -956,7 +1267,7 @@ run (void *cls,
 
   /* check if key from file matches the one from the configuration */
   {
-    struct GNUNET_CRYPTO_EddsaPublicKey master_public_key_from_cfg;
+    struct TALER_MasterPublicKeyP master_public_key_from_cfg;
 
     if (GNUNET_OK !=
         GNUNET_CONFIGURATION_get_data (kcfg,
@@ -974,7 +1285,7 @@ run (void *cls,
     if (0 !=
         memcmp (&master_public_key,
                 &master_public_key_from_cfg,
-                sizeof (struct GNUNET_CRYPTO_EddsaPublicKey)))
+                sizeof (struct TALER_MasterPublicKeyP)))
     {
       GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange",
@@ -1022,10 +1333,19 @@ run (void *cls,
     global_ret = 1;
     return;
   }
-  if (NULL != auditor_output_file)
+  if (GNUNET_OK != create_wire_fees ())
   {
-    FCLOSE (auditor_output_file);
-    auditor_output_file = NULL;
+    global_ret = 1;
+    return;
+  }
+  if ( (0 != memcmp (&zero,
+                     &revoke_dkh,
+                     sizeof (zero))) &&
+       (GNUNET_OK !=
+        revoke_denomination (&revoke_dkh)) )
+  {
+    global_ret = 1;
+    return;
   }
 }
 
@@ -1047,24 +1367,40 @@ int
 main (int argc,
       char *const *argv)
 {
-  const struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'m', "master-key", "FILE",
-     "master key file (private key)", 1,
-     &GNUNET_GETOPT_set_filename, &masterkeyfile},
-    {'o', "output", "FILE",
-     "auditor denomination key signing request file to create", 1,
-     &GNUNET_GETOPT_set_filename, &auditorrequestfile},
-    {'t', "time", "TIMESTAMP",
-     "pretend it is a different time for the update", 0,
-     &GNUNET_GETOPT_set_string, &pretend_time_str},
-     GNUNET_GETOPT_OPTION_END
+  struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_filename ('m',
+                                   "master-key",
+                                   "FILENAME",
+                                   "master key file (private key)",
+                                   &masterkeyfile),
+    GNUNET_GETOPT_option_filename ('f',
+                                   "feedir",
+                                   "DIRNAME",
+                                   "directory where to write wire transfer fee structure",
+                                   &feedir),
+    GNUNET_GETOPT_option_filename ('o',
+                                   "output",
+                                   "FILENAME",
+                                   "auditor denomination key signing request file to create",
+                                   &auditorrequestfile),
+    GNUNET_GETOPT_option_base32_auto ('r',
+                                      "revoke",
+                                      "DKH",
+                                      "revoke denomination key hash (DKH) and request wallets to initiate /payback",
+                                      &revoke_dkh),
+    GNUNET_GETOPT_option_absolute_time ('t',
+                                        "time",
+                                        "TIMESTAMP",
+                                        "pretend it is a different time for the update",
+                                        &now),
+    GNUNET_GETOPT_OPTION_END
   };
 
   GNUNET_assert (GNUNET_OK ==
                  GNUNET_log_setup ("taler-exchange-keyup",
                                    "WARNING",
                                    NULL));
-
+  now = GNUNET_TIME_absolute_get ();
   if (GNUNET_OK !=
       GNUNET_PROGRAM_run (argc, argv,
 			 "taler-exchange-keyup",
@@ -1072,6 +1408,11 @@ main (int argc,
 			  options,
 			  &run, NULL))
     return 1;
+  if (NULL != auditor_output_file)
+  {
+    FCLOSE (auditor_output_file);
+    auditor_output_file = NULL;
+  }
   return global_ret;
 }
 
