@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016 GNUnet e.V.
+  Copyright (C) 2016, 2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -29,8 +29,6 @@
 #include "taler_wire_lib.h"
 
 
-
-
 /**
  * Information we keep for each loaded wire plugin.
  */
@@ -55,6 +53,12 @@ struct WirePlugin
    * Name of the plugin.
    */
   char *type;
+
+  /**
+   * Wire transfer fee structure.
+   */
+  struct TALER_EXCHANGEDB_AggregateFees *af;
+
 };
 
 
@@ -100,10 +104,20 @@ struct AggregationUnit
   struct TALER_MerchantPublicKeyP merchant_pub;
 
   /**
-   * Total amount to be transferred.
+   * Total amount to be transferred, before subtraction of @e wire_fee and rounding down.
    */
   struct TALER_Amount total_amount;
 
+  /**
+   * Final amount to be transferred (after fee and rounding down).
+   */
+  struct TALER_Amount final_amount;
+
+  /**
+   * Wire fee we charge for @e wp at @e execution_time.
+   */
+  struct TALER_Amount wire_fee;
+
   /**
    * Hash of @e wire.
    */
@@ -163,11 +177,44 @@ struct AggregationUnit
 };
 
 
+/**
+ * Context we use while closing a reserve.
+ */
+struct CloseTransferContext
+{
+  /**
+   * Handle for preparing the wire transfer.
+   */
+  struct TALER_WIRE_PrepareHandle *ph;
+
+  /**
+   * Our database session.
+   */
+  struct TALER_EXCHANGEDB_Session *session;
+
+  /**
+   * Wire transfer method.
+   */
+  char *type;
+};
+
+
+/**
+ * Active context while processing reserve closing,
+ * or NULL.
+ */
+static struct CloseTransferContext *ctc;
+
 /**
  * Which currency is used by this exchange?
  */
 static char *exchange_currency_string;
 
+/**
+ * What is the base URL of this exchange?
+ */
+static char *exchange_base_url;
+
 /**
  * The exchange's configuration (global)
  */
@@ -216,6 +263,11 @@ static int global_ret;
  */
 static int test_mode;
 
+/**
+ * Did #run_reserve_closures() have any work during its last run?
+ */
+static int reserves_idle;
+
 /**
  * Limit on the number of transactions we aggregate at once.  Note
  * that the limit must be big enough to ensure that when transactions
@@ -256,6 +308,83 @@ extract_type (const json_t *wire)
 }
 
 
+/**
+ * Advance the "af" pointer in @a wp to point to the
+ * currently valid record.
+ *
+ * @param wp wire transfer fee data structure to update
+ * @param now timestamp to update fees to
+ */
+static void
+advance_fees (struct WirePlugin *wp,
+              struct GNUNET_TIME_Absolute now)
+{
+  struct TALER_EXCHANGEDB_AggregateFees *af;
+
+  /* First, try to see if we have current fee information in memory */
+  af = wp->af;
+  while ( (NULL != af) &&
+          (af->end_date.abs_value_us < now.abs_value_us) )
+  {
+    struct TALER_EXCHANGEDB_AggregateFees *n = af->next;
+
+    GNUNET_free (af);
+    af = n;
+  }
+  wp->af = af;
+}
+
+
+/**
+ * Update wire transfer fee data structure in @a wp.
+ *
+ * @param wp wire transfer fee data structure to update
+ * @param now timestamp to update fees to
+ * @param session DB session to use
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR if we
+ *         lack current fee information (and need to exit)
+ */
+static int
+update_fees (struct WirePlugin *wp,
+             struct GNUNET_TIME_Absolute now,
+             struct TALER_EXCHANGEDB_Session *session)
+{
+  advance_fees (wp,
+                now);
+  if (NULL != wp->af)
+    return GNUNET_OK;
+  /* Let's try to load it from disk... */
+  wp->af = TALER_EXCHANGEDB_fees_read (cfg,
+                                       wp->type);
+  advance_fees (wp,
+                now);
+  for (struct TALER_EXCHANGEDB_AggregateFees *p = wp->af;
+       NULL != p;
+       p = p->next)
+  {
+    if (GNUNET_SYSERR ==
+        db_plugin->insert_wire_fee (db_plugin->cls,
+                                    session,
+                                    wp->type,
+                                    p->start_date,
+                                    p->end_date,
+                                    &p->wire_fee,
+                                    &p->master_sig))
+    {
+      TALER_EXCHANGEDB_fees_free (wp->af);
+      wp->af = NULL;
+      return GNUNET_SYSERR;
+    }
+  }
+  if (NULL != wp->af)
+    return GNUNET_OK;
+  GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+              "Failed to find current wire transfer fees for `%s'\n",
+              wp->type);
+  return GNUNET_SYSERR;
+}
+
+
 /**
  * Find the wire plugin for the given wire address.
  *
@@ -342,6 +471,7 @@ shutdown_task (void *cls)
                                  wp_tail,
                                  wp);
     TALER_WIRE_plugin_unload (wp->wire_plugin);
+    TALER_EXCHANGEDB_fees_free (wp->af);
     GNUNET_free (wp->type);
     GNUNET_free (wp);
   }
@@ -401,8 +531,7 @@ exchange_serve_process_config ()
  * @param coin_pub public key of the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param deposit_fee amount the exchange gets to keep as transaction fees
- * @param transaction_id unique transaction ID chosen by the merchant
- * @param h_contract hash of the contract between merchant and customer
+ * @param h_contract_terms hash of the proposal data known to merchant and customer
  * @param wire_deadline by which the merchant adviced that he would like the
  *        wire transfer to be executed
  * @param wire wire details for the merchant
@@ -415,8 +544,7 @@ deposit_cb (void *cls,
             const struct TALER_CoinSpendPublicKeyP *coin_pub,
             const struct TALER_Amount *amount_with_fee,
             const struct TALER_Amount *deposit_fee,
-            uint64_t transaction_id,
-            const struct GNUNET_HashCode *h_contract,
+            const struct GNUNET_HashCode *h_contract_terms,
             struct GNUNET_TIME_Absolute wire_deadline,
             const json_t *wire)
 {
@@ -432,9 +560,8 @@ deposit_cb (void *cls,
     return GNUNET_SYSERR;
   }
   au->row_id = row_id;
+  GNUNET_assert (NULL == au->wire);
   au->wire = json_incref ((json_t *) wire);
-  au->execution_time = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&au->execution_time);
   TALER_JSON_hash (au->wire,
                    &au->h_wire);
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
@@ -443,12 +570,26 @@ deposit_cb (void *cls,
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Starting aggregation under H(WTID)=%s\n",
               TALER_B2S (&au->wtid));
+
+  au->wp = find_plugin (extract_type (au->wire));
+  if (NULL == au->wp)
+    return GNUNET_SYSERR;
+
+  /* make sure we have current fees */
+  au->execution_time = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&au->execution_time);
+  if (GNUNET_OK !=
+      update_fees (au->wp,
+                   au->execution_time,
+                   au->session))
+    return GNUNET_SYSERR;
+  au->wire_fee = au->wp->af->wire_fee;
+
   if (GNUNET_OK !=
       db_plugin->insert_aggregation_tracking (db_plugin->cls,
                                               au->session,
                                               &au->wtid,
-                                              row_id,
-                                              au->execution_time))
+                                              row_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -476,8 +617,7 @@ deposit_cb (void *cls,
  * @param coin_pub public key of the coin
  * @param amount_with_fee amount that was deposited including fee
  * @param deposit_fee amount the exchange gets to keep as transaction fees
- * @param transaction_id unique transaction ID chosen by the merchant
- * @param h_contract hash of the contract between merchant and customer
+ * @param h_contract_terms hash of the proposal data known to merchant and customer
  * @param wire_deadline by which the merchant adviced that he would like the
  *        wire transfer to be executed
  * @param wire wire details for the merchant
@@ -490,8 +630,7 @@ aggregate_cb (void *cls,
               const struct TALER_CoinSpendPublicKeyP *coin_pub,
               const struct TALER_Amount *amount_with_fee,
               const struct TALER_Amount *deposit_fee,
-              uint64_t transaction_id,
-              const struct GNUNET_HashCode *h_contract,
+              const struct GNUNET_HashCode *h_contract_terms,
               struct GNUNET_TIME_Absolute wire_deadline,
               const json_t *wire)
 {
@@ -541,8 +680,7 @@ aggregate_cb (void *cls,
       db_plugin->insert_aggregation_tracking (db_plugin->cls,
                                               au->session,
                                               &au->wtid,
-                                              row_id,
-                                              au->execution_time))
+                                              row_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -561,7 +699,8 @@ aggregate_cb (void *cls,
 
 
 /**
- * Function to be called with the prepared transfer data.
+ * Function to be called with the prepared transfer data
+ * when running an aggregation on a merchant.
  *
  * @param cls closure with the `struct AggregationUnit`
  * @param buf transaction data to persist, NULL on error
@@ -573,6 +712,319 @@ prepare_cb (void *cls,
             size_t buf_size);
 
 
+/**
+ * Main work function that finds and triggers transfers for reserves
+ * closures.
+ *
+ * @param cls closure
+ */
+static void
+run_reserve_closures (void *cls);
+
+
+/**
+ * Main work function that queries the DB and aggregates transactions
+ * into larger wire transfers.
+ *
+ * @param cls NULL
+ */
+static void
+run_aggregation (void *cls);
+
+
+/**
+ * Function to be called with the prepared transfer data
+ * when closing a reserve.
+ *
+ * @param cls closure with a `struct CloseTransferContext`
+ * @param buf transaction data to persist, NULL on error
+ * @param buf_size number of bytes in @a buf, 0 on error
+ */
+static void
+prepare_close_cb (void *cls,
+		  const char *buf,
+		  size_t buf_size)
+{
+  GNUNET_assert (cls == ctc);
+  ctc->ph = NULL;
+  if (NULL == buf)
+  {
+    GNUNET_break (0); /* why? how to best recover? */
+    db_plugin->rollback (db_plugin->cls,
+                         ctc->session);
+    /* start again */
+    GNUNET_free (ctc->type);
+    GNUNET_free (ctc);
+    ctc = NULL;
+    task = GNUNET_SCHEDULER_add_now (&run_aggregation,
+                                     NULL);
+    return;
+  }
+
+  /* Commit our intention to execute the wire transfer! */
+  if (GNUNET_OK !=
+      db_plugin->wire_prepare_data_insert (db_plugin->cls,
+                                           ctc->session,
+                                           ctc->type,
+                                           buf,
+                                           buf_size))
+  {
+    GNUNET_break (0); /* why? how to best recover? */
+    db_plugin->rollback (db_plugin->cls,
+                         ctc->session);
+    /* start again */
+    task = GNUNET_SCHEDULER_add_now (&run_aggregation,
+                                     NULL);
+    GNUNET_free (ctc->type);
+    GNUNET_free (ctc);
+    ctc = NULL;
+    return;
+  }
+
+  /* finally commit */
+  if (GNUNET_OK !=
+      db_plugin->commit (db_plugin->cls,
+			 ctc->session))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+		"Failed to commit database transaction!\n");
+  }
+  GNUNET_free (ctc->type);
+  GNUNET_free (ctc);
+  ctc = NULL;
+  task = GNUNET_SCHEDULER_add_now (&run_aggregation,
+				   NULL);
+}
+
+
+/**
+ * Function called with details about expired reserves.
+ * We trigger the reserve closure by inserting the respective
+ * closing record and prewire instructions into the respective
+ * tables.
+ *
+ * @param cls a `struct TALER_EXCHANGEDB_Session *`
+ * @param reserve_pub public key of the reserve
+ * @param left amount left in the reserve
+ * @param account_details information about the reserve's bank account
+ * @param expiration_date when did the reserve expire
+ * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop
+ */
+static int
+expired_reserve_cb (void *cls,
+		    const struct TALER_ReservePublicKeyP *reserve_pub,
+		    const struct TALER_Amount *left,
+		    const json_t *account_details,
+		    struct GNUNET_TIME_Absolute expiration_date)
+{
+  struct TALER_EXCHANGEDB_Session *session = cls;
+  struct GNUNET_TIME_Absolute now;
+  struct TALER_WireTransferIdentifierRawP wtid;
+  struct TALER_Amount amount_without_fee;
+  const struct TALER_Amount *closing_fee;
+  int ret;
+  int iret;
+  const char *type;
+  struct WirePlugin *wp;
+
+  GNUNET_assert (NULL == ctc);
+  now = GNUNET_TIME_absolute_get ();
+
+  /* lookup wire plugin */
+  type = extract_type (account_details);
+  if (NULL == type)
+  {
+    GNUNET_break (0);
+    db_plugin->rollback (db_plugin->cls,
+			 session);
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return GNUNET_SYSERR;
+  }
+  wp = find_plugin (type);
+  if (NULL == wp)
+  {
+    GNUNET_break (0);
+    db_plugin->rollback (db_plugin->cls,
+			 session);
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return GNUNET_SYSERR;
+  }
+
+  /* lookup `closing_fee` */
+  if (GNUNET_OK !=
+      update_fees (wp,
+		   now,
+		   session))
+  {
+    GNUNET_break (0);
+    db_plugin->rollback (db_plugin->cls,
+			 session);
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return GNUNET_SYSERR;
+  }
+  closing_fee = &wp->af->closing_fee;
+
+  /* calculate transfer amount */
+  ret = TALER_amount_subtract (&amount_without_fee,
+			       left,
+			       closing_fee);
+  if ( (GNUNET_SYSERR == ret) ||
+       (GNUNET_NO == ret) )
+  {
+    /* Closing fee higher than remaining balance, close
+       without wire transfer. */
+    closing_fee = left;
+    TALER_amount_get_zero (left->currency,
+			   &amount_without_fee);
+  }
+
+  /* NOTE: sizeof (*reserve_pub) == sizeof (wtid) right now, but to
+     be future-compatible, we use the memset + min construction */
+  memset (&wtid,
+	  0,
+	  sizeof (wtid));
+  memcpy (&wtid,
+	  reserve_pub,
+	  GNUNET_MIN (sizeof (wtid),
+		      sizeof (*reserve_pub)));
+  iret = db_plugin->insert_reserve_closed (db_plugin->cls,
+					   session,
+					   reserve_pub,
+					   now,
+					   account_details,
+					   &wtid,
+					   left,
+					   closing_fee);
+  if ( (GNUNET_OK == ret) &&
+       (GNUNET_OK == iret) )
+  {
+    /* success, perform wire transfer */
+    if (GNUNET_SYSERR ==
+	wp->wire_plugin->amount_round (wp->wire_plugin->cls,
+				       &amount_without_fee))
+    {
+      GNUNET_break (0);
+      db_plugin->rollback (db_plugin->cls,
+			   session);
+      global_ret = GNUNET_SYSERR;
+      GNUNET_SCHEDULER_shutdown ();
+      return GNUNET_SYSERR;
+    }
+    ctc = GNUNET_new (struct CloseTransferContext);
+    ctc->session = session;
+    ctc->type = GNUNET_strdup (type);
+    ctc->ph
+      = wp->wire_plugin->prepare_wire_transfer (wp->wire_plugin->cls,
+						au->wire,
+						&amount_without_fee,
+						exchange_base_url,
+						&wtid,
+						&prepare_close_cb,
+						ctc);
+    if (NULL == ctc->ph)
+    {
+      GNUNET_break (0);
+      db_plugin->rollback (db_plugin->cls,
+			   session);
+      global_ret = GNUNET_SYSERR;
+      GNUNET_SCHEDULER_shutdown ();
+      GNUNET_free (ctc->type);
+      GNUNET_free (ctc);
+      ctc = NULL;
+    }
+    return GNUNET_SYSERR;
+  }
+  /* Check for hard failure */
+  if (GNUNET_SYSERR == iret)
+  {
+    GNUNET_break (0);
+    db_plugin->rollback (db_plugin->cls,
+                         session);
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return GNUNET_SYSERR;
+  }
+  /* Reserve balance was almost zero; just commit */
+  if (GNUNET_OK !=
+      db_plugin->commit (db_plugin->cls,
+			 session))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+		"Failed to commit database transaction!\n");
+  }
+  task = GNUNET_SCHEDULER_add_now (&run_reserve_closures,
+				   NULL);
+  return GNUNET_SYSERR;
+}
+
+
+/**
+ * Main work function that finds and triggers transfers for reserves
+ * closures.
+ *
+ * @param cls closure
+ */
+static void
+run_reserve_closures (void *cls)
+{
+  struct TALER_EXCHANGEDB_Session *session;
+  int ret;
+  const struct GNUNET_SCHEDULER_TaskContext *tc;
+
+  task = NULL;
+  reserves_idle = GNUNET_NO;
+  tc = GNUNET_SCHEDULER_get_task_context ();
+  if (0 != (tc->reason & GNUNET_SCHEDULER_REASON_SHUTDOWN))
+    return;
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Checking for reserves to close\n");
+  if (NULL == (session = db_plugin->get_session (db_plugin->cls)))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to obtain database session!\n");
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  if (GNUNET_OK !=
+      db_plugin->start (db_plugin->cls,
+			session))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to start database transaction!\n");
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  ret = db_plugin->get_expired_reserves (db_plugin->cls,
+					 session,
+					 GNUNET_TIME_absolute_get (),
+					 &expired_reserve_cb,
+					 session);
+  if (GNUNET_SYSERR == ret)
+  {
+    GNUNET_break (0);
+    db_plugin->rollback (db_plugin->cls,
+                         session);
+    global_ret = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  if (GNUNET_NO == ret)
+  {
+    reserves_idle = GNUNET_YES;
+    db_plugin->rollback (db_plugin->cls,
+                         session);
+    task = GNUNET_SCHEDULER_add_now (&run_aggregation,
+				     NULL);
+    return;
+  }
+}
+
+
 /**
  * Main work function that queries the DB and aggregates transactions
  * into larger wire transfers.
@@ -582,16 +1034,22 @@ prepare_cb (void *cls,
 static void
 run_aggregation (void *cls)
 {
+  static int swap;
   struct TALER_EXCHANGEDB_Session *session;
   unsigned int i;
   int ret;
   const struct GNUNET_SCHEDULER_TaskContext *tc;
-  struct WirePlugin *wp;
 
   task = NULL;
   tc = GNUNET_SCHEDULER_get_task_context ();
   if (0 != (tc->reason & GNUNET_SCHEDULER_REASON_SHUTDOWN))
     return;
+ if (0 == (++swap % 2))
+  {
+    task = GNUNET_SCHEDULER_add_now (&run_reserve_closures,
+				     NULL);
+    return;
+  }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Checking for ready deposits to aggregate\n");
   if (NULL == (session = db_plugin->get_session (db_plugin->cls)))
@@ -603,8 +1061,8 @@ run_aggregation (void *cls)
     return;
   }
   if (GNUNET_OK !=
-      db_plugin->start (db_plugin->cls,
-                        session))
+      db_plugin->start_deferred_wire_out (db_plugin->cls,
+                                          session))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to start database transaction!\n");
@@ -644,25 +1102,17 @@ run_aggregation (void *cls)
     else
     {
       /* nothing to do, sleep for a minute and try again */
-      task = GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_MINUTES,
-                                           &run_aggregation,
-                                           NULL);
+      if (GNUNET_NO == reserves_idle)
+	task = GNUNET_SCHEDULER_add_now (&run_reserve_closures,
+					 NULL);
+      else
+	task = GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_MINUTES,
+					     &run_aggregation,
+					     NULL);
     }
     return;
   }
 
-  wp = find_plugin (extract_type (au->wire));
-  if (NULL == wp)
-  {
-    json_decref (au->wire);
-    GNUNET_free (au);
-    au = NULL;
-    db_plugin->rollback (db_plugin->cls,
-                         session);
-    GNUNET_SCHEDULER_shutdown ();
-    return;
-  }
-
   /* Now try to find other deposits to aggregate */
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Found ready deposit for %s, aggregating\n",
@@ -690,13 +1140,18 @@ run_aggregation (void *cls)
     return;
   }
 
-  /* Round to the unit supported by the wire transfer method */
-  GNUNET_assert (GNUNET_SYSERR !=
-                 wp->wire_plugin->amount_round (wp->wire_plugin->cls,
-                                                &au->total_amount));
-  /* Check if after rounding down, we still have an amount to transfer */
-  if ( (0 == au->total_amount.value) &&
-       (0 == au->total_amount.fraction) )
+  /* Subtract wire transfer fee and round to the unit supported by the
+     wire transfer method; Check if after rounding down, we still have
+     an amount to transfer, and if not mark as 'tiny'. */
+  if ( (GNUNET_OK !=
+        TALER_amount_subtract (&au->final_amount,
+                               &au->total_amount,
+                               &au->wire_fee)) ||
+       (GNUNET_SYSERR ==
+        au->wp->wire_plugin->amount_round (au->wp->wire_plugin->cls,
+                                           &au->final_amount)) ||
+       ( (0 == au->final_amount.value) &&
+         (0 == au->final_amount.fraction) ) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Aggregate value too low for transfer\n");
@@ -756,20 +1211,20 @@ run_aggregation (void *cls)
   {
     char *amount_s;
 
-    amount_s = TALER_amount_to_string (&au->total_amount);
+    amount_s = TALER_amount_to_string (&au->final_amount);
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Preparing wire transfer of %s to %s\n",
                 amount_s,
                 TALER_B2S (&au->merchant_pub));
     GNUNET_free (amount_s);
   }
-  au->wp = wp;
-  au->ph = wp->wire_plugin->prepare_wire_transfer (wp->wire_plugin->cls,
-                                                   au->wire,
-                                                   &au->total_amount,
-                                                   &au->wtid,
-                                                   &prepare_cb,
-                                                   au);
+  au->ph = au->wp->wire_plugin->prepare_wire_transfer (au->wp->wire_plugin->cls,
+                                                       au->wire,
+                                                       &au->final_amount,
+                                                       exchange_base_url,
+                                                       &au->wtid,
+                                                       &prepare_cb,
+                                                       au);
   if (NULL == au->ph)
   {
     GNUNET_break (0); /* why? how to best recover? */
@@ -813,11 +1268,6 @@ prepare_cb (void *cls,
 {
   struct TALER_EXCHANGEDB_Session *session = au->session;
 
-  if (NULL != au->wire)
-  {
-    json_decref (au->wire);
-    au->wire = NULL;
-  }
   GNUNET_free_non_null (au->additional_rows);
   if (NULL == buf)
   {
@@ -827,6 +1277,11 @@ prepare_cb (void *cls,
     /* start again */
     task = GNUNET_SCHEDULER_add_now (&run_aggregation,
                                      NULL);
+    if (NULL != au->wire)
+    {
+      json_decref (au->wire);
+      au->wire = NULL;
+    }
     GNUNET_free (au);
     au = NULL;
     return;
@@ -846,10 +1301,46 @@ prepare_cb (void *cls,
     /* start again */
     task = GNUNET_SCHEDULER_add_now (&run_aggregation,
                                      NULL);
+    if (NULL != au->wire)
+    {
+      json_decref (au->wire);
+      au->wire = NULL;
+    }
     GNUNET_free (au);
     au = NULL;
     return;
   }
+
+  /* Commit the WTID data to 'wire_out' to finally satisfy aggregation
+     table constraints */
+  if (GNUNET_OK !=
+      db_plugin->store_wire_transfer_out (db_plugin->cls,
+                                          session,
+                                          au->execution_time,
+                                          &au->wtid,
+                                          au->wire,
+                                          &au->final_amount))
+  {
+    GNUNET_break (0); /* why? how to best recover? */
+    db_plugin->rollback (db_plugin->cls,
+                         session);
+    /* start again */
+    task = GNUNET_SCHEDULER_add_now (&run_aggregation,
+                                     NULL);
+    if (NULL != au->wire)
+    {
+      json_decref (au->wire);
+      au->wire = NULL;
+    }
+    GNUNET_free (au);
+    au = NULL;
+    return;
+  }
+  if (NULL != au->wire)
+  {
+    json_decref (au->wire);
+    au->wire = NULL;
+  }
   GNUNET_free (au);
   au = NULL;
 
@@ -879,11 +1370,13 @@ prepare_cb (void *cls,
  *
  * @param cls NULL
  * @param success #GNUNET_OK on success, #GNUNET_SYSERR on failure
+ * @param serial_id unique ID of the wire transfer in the bank's records; UINT64_MAX on error
  * @param emsg NULL on success, otherwise an error message
  */
 static void
 wire_confirm_cb (void *cls,
                  int success,
+                 uint64_t serial_id,
                  const char *emsg)
 {
   struct TALER_EXCHANGEDB_Session *session = wpd->session;
@@ -984,7 +1477,6 @@ wire_prepare_cb (void *cls,
  * do.
  *
  * @param cls NULL
- * @param tc scheduler context
  */
 static void
 run_transfers (void *cls)
@@ -1065,6 +1557,18 @@ run (void *cls,
      const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *c)
 {
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_string (c,
+                                             "exchange",
+                                             "BASE_URL",
+                                             &exchange_base_url))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "exchange",
+                               "BASE_URL");
+    global_ret = 1;
+    return;
+  }
   cfg = GNUNET_CONFIGURATION_dup (c);
   if (GNUNET_OK != exchange_serve_process_config ())
   {
@@ -1081,7 +1585,7 @@ run (void *cls,
 
 
 /**
- * The main function of the taler-exchange-httpd server ("the exchange").
+ * The main function of the taler-exchange-aggregator.
  *
  * @param argc number of arguments from the command line
  * @param argv command line arguments
@@ -1091,11 +1595,12 @@ int
 main (int argc,
       char *const *argv)
 {
-  static const struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'t', "test", NULL,
-     "run in test mode and exit when idle", 0,
-     &GNUNET_GETOPT_set_one, &test_mode},
-    GNUNET_GETOPT_OPTION_VERSION (VERSION "-" VCS_VERSION),
+  struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_flag ('t',
+			       "test",
+			       "run in test mode and exit when idle",
+			       &test_mode),
+    GNUNET_GETOPT_option_version (VERSION "-" VCS_VERSION),
     GNUNET_GETOPT_OPTION_END
   };
 
