@@ -32,6 +32,7 @@
 #include "taler-exchange-httpd_deposit.h"
 #include "taler-exchange-httpd_refund.h"
 #include "taler-exchange-httpd_reserve.h"
+#include "taler-exchange-httpd_payback.h"
 #include "taler-exchange-httpd_wire.h"
 #include "taler-exchange-httpd_refresh.h"
 #include "taler-exchange-httpd_tracking.h"
@@ -100,6 +101,11 @@ static struct MHD_Daemon *mhd_admin;
  */
 static int no_admin;
 
+/**
+ * Initialize the database by creating tables and indices.
+ */
+static int init_db;
+
 /**
  * Port to run the daemon on.
  */
@@ -290,6 +296,13 @@ handle_mhd_request (void *cls,
         "Only POST is allowed", 0,
         &TEH_MHD_handler_send_json_pack_error, MHD_HTTP_METHOD_NOT_ALLOWED },
 
+      { "/payback", MHD_HTTP_METHOD_POST, "application/json",
+        NULL, 0,
+        &TEH_PAYBACK_handler_payback, MHD_HTTP_OK },
+      { "/refresh/link", NULL, "text/plain",
+        "Only GET is allowed", 0,
+        &TEH_MHD_handler_send_json_pack_error, MHD_HTTP_METHOD_NOT_ALLOWED },
+
       { NULL, NULL, NULL, NULL, 0, 0 }
     };
   static struct TEH_RequestHandler h404 =
@@ -464,8 +477,8 @@ handle_mhd_admin_request (void *cls,
  * Parse the configuration to determine on which port
  * or UNIX domain path we should run an HTTP service.
  *
- * @param type section of the configuration to parse ("exchange" or "exchange-admin")
- * @param[out] port set to the port number, or 0 for none
+ * @param section section of the configuration to parse ("exchange" or "exchange-admin")
+ * @param[out] rport set to the port number, or 0 for none
  * @param[out] unix_path set to the UNIX path, or NULL for none
  * @param[out] unix_mode set to the mode to be used for @a unix_path
  * @return #GNUNET_OK on success
@@ -581,7 +594,6 @@ parse_port_config (const char *section,
  * Load configuration parameters for the exchange
  * server into the corresponding global variables.
  *
- * @param exchange_directory the exchange's directory
  * @return #GNUNET_OK on success
  */
 static int
@@ -619,10 +631,6 @@ exchange_serve_process_config ()
              (unsigned int) TALER_CURRENCY_LEN);
     return GNUNET_SYSERR;
   }
-  if (GNUNET_OK !=
-      TEH_VALIDATION_init (cfg))
-    return GNUNET_SYSERR;
-
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              "exchange",
@@ -632,7 +640,6 @@ exchange_serve_process_config ()
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                "exchange",
                                "master_public_key");
-    TEH_VALIDATION_done ();
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -643,11 +650,17 @@ exchange_serve_process_config ()
     fprintf (stderr,
              "Invalid master public key given in exchange configuration.");
     GNUNET_free (TEH_master_public_key_str);
-    TEH_VALIDATION_done ();
     return GNUNET_SYSERR;
   }
   GNUNET_free (TEH_master_public_key_str);
 
+  if ( (GNUNET_OK !=
+        TEH_VALIDATION_init (cfg)) ||
+       (GNUNET_OK !=
+        TEH_WIRE_init ()) )
+    return GNUNET_SYSERR;
+
+
   if (NULL ==
       (TEH_plugin = TALER_EXCHANGEDB_plugin_load (cfg)))
   {
@@ -656,13 +669,10 @@ exchange_serve_process_config ()
     TEH_VALIDATION_done ();
     return GNUNET_SYSERR;
   }
-  if (GNUNET_YES ==
-      GNUNET_CONFIGURATION_get_value_yesno (cfg,
-                                            "exchange",
-                                            "TESTRUN"))
+  if (0 != init_db)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "Running in TEST mode! Database contents will not persist!\n");
+                "Ensuring that tables and indices are created!\n");
     TEH_plugin->create_tables (TEH_plugin->cls);
   }
 
@@ -931,25 +941,35 @@ main (int argc,
   char *loglev = NULL;
   char *logfile = NULL;
   const struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'C', "connection-close", NULL,
-     "force HTTP connections to be closed after each request", 0,
-     &GNUNET_GETOPT_set_one, &TEH_exchange_connection_close},
-    GNUNET_GETOPT_OPTION_CFG_FILE (&cfgfile),
-    {'D', "disable-admin", NULL,
-     "do not run the /admin-HTTP server", 0,
-     &GNUNET_GETOPT_set_one, &no_admin},
-    {'t', "timeout", "SECONDS",
-     "after how long do connections timeout by default (in seconds)", 1,
-     &GNUNET_GETOPT_set_uint, &connection_timeout},
+    GNUNET_GETOPT_option_flag ('C',
+                                  "connection-close",
+                                  "force HTTP connections to be closed after each request",
+                                  &TEH_exchange_connection_close),
+    GNUNET_GETOPT_option_cfgfile (&cfgfile),
+    GNUNET_GETOPT_option_flag ('D',
+                                  "disable-admin",
+                                  "do not run the /admin-HTTP server",
+                                  &no_admin),
+    GNUNET_GETOPT_option_flag ('i',
+                                  "init-db",
+                                  "create database tables and indicies if necessary",
+                                  &init_db),
+   GNUNET_GETOPT_option_uint ('t',
+                                  "timeout",
+                                  "SECONDS",
+                                  "after how long do connections timeout by default (in seconds)",
+                                  &connection_timeout),
 #if HAVE_DEVELOPER
-    {'f', "file-input", "FILENAME",
-     "run in test-mode using FILENAME as the HTTP request to process", 1,
-     &GNUNET_GETOPT_set_filename, &input_filename},
+   GNUNET_GETOPT_option_filename ('f',
+                                  "file-input",
+                                  "FILENAME",
+                                  "run in test-mode using FILENAME as the HTTP request to process",
+                                  &input_filename),
 #endif
-    GNUNET_GETOPT_OPTION_HELP ("HTTP server providing a RESTful API to access a Taler exchange"),
-    GNUNET_GETOPT_OPTION_LOGLEVEL (&loglev),
-    GNUNET_GETOPT_OPTION_LOGFILE (&logfile),
-    GNUNET_GETOPT_OPTION_VERSION (VERSION "-" VCS_VERSION),
+    GNUNET_GETOPT_option_help ("HTTP server providing a RESTful API to access a Taler exchange"),
+    GNUNET_GETOPT_option_loglevel (&loglev),
+    GNUNET_GETOPT_option_logfile (&logfile),
+    GNUNET_GETOPT_option_version (VERSION "-" VCS_VERSION),
     GNUNET_GETOPT_OPTION_END
   };
   int ret;
@@ -1120,12 +1140,16 @@ main (int argc,
     {
       MHD_socket sock = MHD_quiesce_daemon (mhd);
       MHD_socket admin_sock;
+      int admin_sock_opened = GNUNET_NO;
       pid_t chld;
       int flags;
 
       /* Set flags to make 'sock' inherited by child */
       if (NULL != mhd_admin)
+      {
         admin_sock = MHD_quiesce_daemon (mhd_admin);
+        admin_sock_opened = GNUNET_YES;
+      }
       flags = fcntl (sock, F_GETFD);
       GNUNET_assert (-1 != flags);
       flags &= ~FD_CLOEXEC;
@@ -1149,7 +1173,7 @@ main (int argc,
                                "dup2");
           _exit (1);
         }
-        if ( (NULL != mhd_admin) &&
+        if ( (GNUNET_YES == admin_sock_opened) &&
              (4 != dup2 (admin_sock, 4)) )
         {
           GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
@@ -1174,7 +1198,7 @@ main (int argc,
          before exiting; as the listen socket is no longer used,
          close it here */
       GNUNET_break (0 == close (sock));
-      if (NULL != mhd_admin)
+      if (GNUNET_YES == admin_sock_opened)
         GNUNET_break (0 == close (admin_sock));
       while ( (0 != MHD_get_daemon_info (mhd,
                                          MHD_DAEMON_INFO_CURRENT_CONNECTIONS)->num_connections) ||
