@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+  Copyright (C) 2014-2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -178,10 +178,21 @@ calculate_transaction_list_totals (struct TALER_EXCHANGEDB_TransactionList *tl,
         return GNUNET_SYSERR;
       }
       break;
+    case TALER_EXCHANGEDB_TT_PAYBACK:
+      /* spent += pos->value */
+      if (GNUNET_OK !=
+          TALER_amount_add (&spent,
+                            &spent,
+                            &pos->details.payback->value))
+      {
+        GNUNET_break (0);
+        return GNUNET_SYSERR;
+      }
+      break;
     }
   }
   /* spent = spent - refunded */
-  if (GNUNET_OK !=
+  if (GNUNET_SYSERR ==
       TALER_amount_subtract (&spent,
                              &spent,
                              &refunded))
@@ -236,13 +247,14 @@ TEH_DB_execute_deposit (struct MHD_Connection *connection,
     return TEH_RESPONSE_reply_deposit_success (connection,
                                                &deposit->coin.coin_pub,
                                                &deposit->h_wire,
-                                               &deposit->h_contract,
-                                               deposit->transaction_id,
+                                               &deposit->h_contract_terms,
                                                deposit->timestamp,
                                                deposit->refund_deadline,
                                                &deposit->merchant_pub,
                                                &amount_without_fee);
   }
+
+  /* FIXME: move the 'mks'-logic outside of _db.c? */
   mks = TEH_KS_acquire ();
   dki = TEH_KS_denomination_key_lookup (mks,
                                         &deposit->coin.denom_pub,
@@ -251,7 +263,7 @@ TEH_DB_execute_deposit (struct MHD_Connection *connection,
   {
     TEH_KS_release (mks);
     return TEH_RESPONSE_reply_internal_db_error (connection,
-					   TALER_EC_DEPOSIT_DB_DENOMINATION_KEY_UNKNOWN);
+                                                 TALER_EC_DEPOSIT_DB_DENOMINATION_KEY_UNKNOWN);
   }
   TALER_amount_ntoh (&value,
                      &dki->issue.properties.value);
@@ -284,8 +296,9 @@ TEH_DB_execute_deposit (struct MHD_Connection *connection,
   {
     TEH_plugin->rollback (TEH_plugin->cls,
                           session);
-    ret = TEH_RESPONSE_reply_deposit_insufficient_funds (connection,
-                                                         tl);
+    ret = TEH_RESPONSE_reply_coin_insufficient_funds (connection,
+                                                      TALER_EC_DEPOSIT_INSUFFICIENT_FUNDS,
+                                                      tl);
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
                                             tl);
     return ret;
@@ -312,8 +325,7 @@ TEH_DB_execute_deposit (struct MHD_Connection *connection,
   return TEH_RESPONSE_reply_deposit_success (connection,
                                              &deposit->coin.coin_pub,
                                              &deposit->h_wire,
-                                             &deposit->h_contract,
-                                             deposit->transaction_id,
+                                             &deposit->h_contract_terms,
                                              deposit->timestamp,
                                              deposit->refund_deadline,
                                              &deposit->merchant_pub,
@@ -377,15 +389,14 @@ TEH_DB_execute_refund (struct MHD_Connection *connection,
     case TALER_EXCHANGEDB_TT_DEPOSIT:
       if (GNUNET_NO == deposit_found)
       {
-        dep = tlp->details.deposit;
-        if ( (0 == memcmp (&dep->merchant_pub,
+        if ( (0 == memcmp (&tlp->details.deposit->merchant_pub,
                            &refund->merchant_pub,
                            sizeof (struct TALER_MerchantPublicKeyP))) &&
-             (0 == memcmp (&dep->h_contract,
-                           &refund->h_contract,
-                           sizeof (struct GNUNET_HashCode))) &&
-             (dep->transaction_id == refund->transaction_id) )
+             (0 == memcmp (&tlp->details.deposit->h_contract_terms,
+                           &refund->h_contract_terms,
+                           sizeof (struct GNUNET_HashCode))) )
         {
+          dep = tlp->details.deposit;
           deposit_found = GNUNET_YES;
           break;
         }
@@ -397,31 +408,29 @@ TEH_DB_execute_refund (struct MHD_Connection *connection,
     case TALER_EXCHANGEDB_TT_REFUND:
       if (GNUNET_NO == refund_found)
       {
-        ref = tlp->details.refund;
         /* First, check if existing refund request is identical */
-        if ( (0 == memcmp (&ref->merchant_pub,
+        if ( (0 == memcmp (&tlp->details.refund->merchant_pub,
                            &refund->merchant_pub,
                            sizeof (struct TALER_MerchantPublicKeyP))) &&
-             (0 == memcmp (&ref->h_contract,
-                           &refund->h_contract,
+             (0 == memcmp (&tlp->details.refund->h_contract_terms,
+                           &refund->h_contract_terms,
                            sizeof (struct GNUNET_HashCode))) &&
-             (ref->transaction_id == refund->transaction_id) &&
-             (ref->rtransaction_id == refund->rtransaction_id) )
+             (tlp->details.refund->rtransaction_id == refund->rtransaction_id) )
         {
+          ref = tlp->details.refund;
           refund_found = GNUNET_YES;
           break;
         }
         /* Second, check if existing refund request conflicts */
-        if ( (0 == memcmp (&ref->merchant_pub,
+        if ( (0 == memcmp (&tlp->details.refund->merchant_pub,
                            &refund->merchant_pub,
                            sizeof (struct TALER_MerchantPublicKeyP))) &&
-             (0 == memcmp (&ref->h_contract,
-                           &refund->h_contract,
+             (0 == memcmp (&tlp->details.refund->h_contract_terms,
+                           &refund->h_contract_terms,
                            sizeof (struct GNUNET_HashCode))) &&
-             (ref->transaction_id == refund->transaction_id) &&
-             (ref->rtransaction_id != refund->rtransaction_id) )
+             (tlp->details.refund->rtransaction_id != refund->rtransaction_id) )
         {
-          GNUNET_break_op (0); /* conflicting refound found */
+          GNUNET_break_op (0); /* conflicting refund found */
           refund_found = GNUNET_SYSERR;
           /* NOTE: Alternatively we could total up all existing
              refunds and check if the sum still permits the
@@ -431,6 +440,9 @@ TEH_DB_execute_refund (struct MHD_Connection *connection,
         }
       }
       break;
+    case TALER_EXCHANGEDB_TT_PAYBACK:
+      /* Paybacks cannot be refunded, ignore here */
+      break;
     }
   }
   /* handle if deposit was NOT found */
@@ -581,7 +593,7 @@ TEH_DB_execute_refund (struct MHD_Connection *connection,
     return TEH_RESPONSE_reply_internal_db_error (connection,
 						 TALER_EC_REFUND_STORE_DB_ERROR);
   }
-  COMMIT_TRANSACTION(session, connection);
+  COMMIT_TRANSACTION (session, connection);
 
   return TEH_RESPONSE_reply_refund_success (connection,
                                             refund);
@@ -610,9 +622,11 @@ TEH_DB_execute_reserve_status (struct MHD_Connection *connection,
     return TEH_RESPONSE_reply_internal_db_error (connection,
 						 TALER_EC_DB_SETUP_FAILED);
   }
+  START_TRANSACTION (session, connection);
   rh = TEH_plugin->get_reserve_history (TEH_plugin->cls,
                                         session,
                                         reserve_pub);
+  COMMIT_TRANSACTION (session, connection);
   if (NULL == rh)
     return TEH_RESPONSE_reply_json_pack (connection,
                                          MHD_HTTP_NOT_FOUND,
@@ -620,7 +634,7 @@ TEH_DB_execute_reserve_status (struct MHD_Connection *connection,
                                          "error", "Reserve not found",
                                          "parameter", "withdraw_pub");
   res = TEH_RESPONSE_reply_reserve_status_success (connection,
-                                                    rh);
+                                                   rh);
   TEH_plugin->free_reserve_history (TEH_plugin->cls,
                                     rh);
   return res;
@@ -640,7 +654,7 @@ TEH_DB_execute_reserve_status (struct MHD_Connection *connection,
  * @param blinded_msg_len number of bytes in @a blinded_msg
  * @param h_blind hash of @a blinded_msg
  * @param signature signature over the withdraw request, to be stored in DB
- * @param denom_sig[out] where to write the resulting signature
+ * @param[out] denom_sig where to write the resulting signature
  *        (used to release memory in case of transaction failure
  * @return MHD result code
  */
@@ -752,6 +766,40 @@ execute_reserve_withdraw_transaction (struct MHD_Connection *connection,
         }
       res |= 2;
       break;
+
+    case TALER_EXCHANGEDB_RO_PAYBACK_COIN:
+      if (0 == (res & 1))
+        deposit_total = pos->details.payback->value;
+      else
+        if (GNUNET_OK !=
+            TALER_amount_add (&deposit_total,
+                              &deposit_total,
+                              &pos->details.payback->value))
+        {
+          TEH_plugin->rollback (TEH_plugin->cls,
+                                session);
+          return TEH_RESPONSE_reply_internal_db_error (connection,
+						       TALER_EC_WITHDRAW_AMOUNT_DEPOSITS_OVERFLOW);
+        }
+      res |= 1;
+      break;
+
+    case TALER_EXCHANGEDB_RO_EXCHANGE_TO_BANK:
+      if (0 == (res & 2))
+        withdraw_total = pos->details.bank->amount;
+      else
+        if (GNUNET_OK !=
+            TALER_amount_add (&withdraw_total,
+                              &withdraw_total,
+                              &pos->details.bank->amount))
+        {
+          TEH_plugin->rollback (TEH_plugin->cls,
+                                session);
+          return TEH_RESPONSE_reply_internal_db_error (connection,
+						       TALER_EC_WITHDRAW_AMOUNT_WITHDRAWALS_OVERFLOW);
+        }
+      res |= 2;
+      break;
     }
   }
   if (0 == (res & 1))
@@ -1819,6 +1867,7 @@ TEH_DB_execute_admin_add_incoming (struct MHD_Connection *connection,
 {
   struct TALER_EXCHANGEDB_Session *session;
   int ret;
+  void *json_str;
 
   if (NULL == (session = TEH_plugin->get_session (TEH_plugin->cls)))
   {
@@ -1826,13 +1875,23 @@ TEH_DB_execute_admin_add_incoming (struct MHD_Connection *connection,
     return TEH_RESPONSE_reply_internal_db_error (connection,
 						 TALER_EC_DB_SETUP_FAILED);
   }
+  json_str = json_dumps (transfer_details,
+                         JSON_INDENT(2));
+  if (NULL == json_str)
+  {
+    GNUNET_break (0);
+    return TEH_RESPONSE_reply_internal_db_error (connection,
+						 TALER_EC_PARSER_OUT_OF_MEMORY);
+  }
   ret = TEH_plugin->reserves_in_insert (TEH_plugin->cls,
                                         session,
                                         reserve_pub,
                                         amount,
                                         execution_time,
                                         sender_account_details,
-                                        transfer_details);
+                                        json_str,
+                                        strlen (json_str));
+  free (json_str);
   if (GNUNET_SYSERR == ret)
   {
     GNUNET_break (0);
@@ -1869,6 +1928,11 @@ struct WtidTransactionContext
    */
   struct TALER_MerchantPublicKeyP merchant_pub;
 
+  /**
+   * Which method was used to wire the funds?
+   */
+  char *wire_method;
+
   /**
    * Hash of the wire details of the merchant (identical for all
    * deposits), only valid if @e is_valid is #GNUNET_YES.
@@ -1912,22 +1976,24 @@ struct WtidTransactionContext
  * transaction data for the given wire transfer identifier.
  *
  * @param cls our context for transmission
+ * @param rowid which row in the DB is the information from (for diagnostics)
  * @param merchant_pub public key of the merchant (should be same for all callbacks with the same @e cls)
+ * @param wire_method which wire plugin was used
  * @param h_wire hash of wire transfer details of the merchant (should be same for all callbacks with the same @e cls)
  * @param exec_time execution time of the wire transfer (should be same for all callbacks with the same @e cls)
- * @param h_contract which contract was this payment about
- * @param transaction_id merchant's transaction ID for the payment
+ * @param h_contract_terms which proposal was this payment about
  * @param coin_pub which public key was this payment about
  * @param deposit_value amount contributed by this coin in total
  * @param deposit_fee deposit fee charged by exchange for this coin
  */
 static void
 handle_transaction_data (void *cls,
+                         uint64_t rowid,
                          const struct TALER_MerchantPublicKeyP *merchant_pub,
+                         const char *wire_method,
                          const struct GNUNET_HashCode *h_wire,
                          struct GNUNET_TIME_Absolute exec_time,
-                         const struct GNUNET_HashCode *h_contract,
-                         uint64_t transaction_id,
+                         const struct GNUNET_HashCode *h_contract_terms,
                          const struct TALER_CoinSpendPublicKeyP *coin_pub,
                          const struct TALER_Amount *deposit_value,
                          const struct TALER_Amount *deposit_fee)
@@ -1943,6 +2009,7 @@ handle_transaction_data (void *cls,
     ctx->merchant_pub = *merchant_pub;
     ctx->h_wire = *h_wire;
     ctx->exec_time = exec_time;
+    ctx->wire_method = GNUNET_strdup (wire_method);
     ctx->is_valid = GNUNET_YES;
     if (GNUNET_OK !=
         TALER_amount_subtract (&ctx->total,
@@ -1959,6 +2026,8 @@ handle_transaction_data (void *cls,
     if ( (0 != memcmp (&ctx->merchant_pub,
                        merchant_pub,
                        sizeof (struct TALER_MerchantPublicKeyP))) ||
+         (0 != strcmp (wire_method,
+                       ctx->wire_method)) ||
          (0 != memcmp (&ctx->h_wire,
                        h_wire,
                        sizeof (struct GNUNET_HashCode))) )
@@ -1989,8 +2058,7 @@ handle_transaction_data (void *cls,
   wdd = GNUNET_new (struct TEH_TrackTransferDetail);
   wdd->deposit_value = *deposit_value;
   wdd->deposit_fee = *deposit_fee;
-  wdd->h_contract = *h_contract;
-  wdd->transaction_id = transaction_id;
+  wdd->h_contract_terms = *h_contract_terms;
   wdd->coin_pub = *coin_pub;
   GNUNET_CONTAINER_DLL_insert (ctx->wdd_head,
                                ctx->wdd_tail,
@@ -2014,6 +2082,10 @@ TEH_DB_execute_track_transfer (struct MHD_Connection *connection,
   struct WtidTransactionContext ctx;
   struct TALER_EXCHANGEDB_Session *session;
   struct TEH_TrackTransferDetail *wdd;
+  struct GNUNET_TIME_Absolute wire_fee_start_date;
+  struct GNUNET_TIME_Absolute wire_fee_end_date;
+  struct TALER_Amount wire_fee;
+  struct TALER_MasterSignatureP wire_fee_master_sig;
 
   if (NULL == (session = TEH_plugin->get_session (TEH_plugin->cls)))
   {
@@ -2024,6 +2096,7 @@ TEH_DB_execute_track_transfer (struct MHD_Connection *connection,
   ctx.is_valid = GNUNET_NO;
   ctx.wdd_head = NULL;
   ctx.wdd_tail = NULL;
+  ctx.wire_method = NULL;
   ret = TEH_plugin->lookup_wire_transfer (TEH_plugin->cls,
                                           session,
                                           wtid,
@@ -2050,10 +2123,36 @@ TEH_DB_execute_track_transfer (struct MHD_Connection *connection,
                                           "wtid");
     goto cleanup;
   }
+  if (GNUNET_OK !=
+      TEH_plugin->get_wire_fee (TEH_plugin->cls,
+                                session,
+                                ctx.wire_method,
+                                ctx.exec_time,
+                                &wire_fee_start_date,
+                                &wire_fee_end_date,
+                                &wire_fee,
+                                &wire_fee_master_sig))
+  {
+    GNUNET_break (0);
+    ret = TEH_RESPONSE_reply_internal_db_error (connection,
+						TALER_EC_TRACK_TRANSFER_WIRE_FEE_NOT_FOUND);
+    goto cleanup;
+  }
+  if (GNUNET_OK !=
+      TALER_amount_subtract (&ctx.total,
+                             &ctx.total,
+                             &wire_fee))
+  {
+    GNUNET_break (0);
+    ret = TEH_RESPONSE_reply_internal_db_error (connection,
+						TALER_EC_TRACK_TRANSFER_WIRE_FEE_INCONSISTENT);
+    goto cleanup;
+  }
   ret = TEH_RESPONSE_reply_track_transfer_details (connection,
                                                    &ctx.total,
                                                    &ctx.merchant_pub,
                                                    &ctx.h_wire,
+                                                   &wire_fee,
                                                    ctx.exec_time,
                                                    ctx.wdd_head);
  cleanup:
@@ -2064,6 +2163,7 @@ TEH_DB_execute_track_transfer (struct MHD_Connection *connection,
                                  wdd);
     GNUNET_free (wdd);
   }
+  GNUNET_free_non_null (ctx.wire_method);
   return ret;
 }
 
@@ -2080,9 +2180,9 @@ struct DepositWtidContext
   struct MHD_Connection *connection;
 
   /**
-   * Hash of the contract we are looking up.
+   * Hash of the proposal data we are looking up.
    */
-  struct GNUNET_HashCode h_contract;
+  struct GNUNET_HashCode h_contract_terms;
 
   /**
    * Hash of the wire transfer details we are looking up.
@@ -2094,11 +2194,6 @@ struct DepositWtidContext
    */
   struct TALER_CoinSpendPublicKeyP coin_pub;
 
-  /**
-   * Transaction ID we are looking up.
-   */
-  uint64_t transaction_id;
-
   /**
    * MHD result code to return.
    */
@@ -2150,11 +2245,10 @@ handle_wtid_data (void *cls,
     else
     {
       ctx->res = TEH_RESPONSE_reply_track_transaction (ctx->connection,
-                                                       &ctx->h_contract,
+                                                       &ctx->h_contract_terms,
                                                        &ctx->h_wire,
                                                        &ctx->coin_pub,
                                                        &coin_delta,
-                                                       ctx->transaction_id,
                                                        wtid,
                                                        execution_time);
     }
@@ -2167,20 +2261,18 @@ handle_wtid_data (void *cls,
  * associated with the given deposit.
  *
  * @param connection the MHD connection to handle
- * @param h_contract hash of the contract
+ * @param h_contract_terms hash of the proposal data
  * @param h_wire hash of the wire details
  * @param coin_pub public key of the coin to link
  * @param merchant_pub public key of the merchant
- * @param transaction_id transaction ID of the merchant
  * @return MHD result code
  */
 int
 TEH_DB_execute_track_transaction (struct MHD_Connection *connection,
-                                  const struct GNUNET_HashCode *h_contract,
+                                  const struct GNUNET_HashCode *h_contract_terms,
                                   const struct GNUNET_HashCode *h_wire,
                                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                                  const struct TALER_MerchantPublicKeyP *merchant_pub,
-                                  uint64_t transaction_id)
+                                  const struct TALER_MerchantPublicKeyP *merchant_pub)
 {
   int ret;
   struct DepositWtidContext ctx;
@@ -2193,18 +2285,16 @@ TEH_DB_execute_track_transaction (struct MHD_Connection *connection,
 						 TALER_EC_DB_SETUP_FAILED);
   }
   ctx.connection = connection;
-  ctx.h_contract = *h_contract;
+  ctx.h_contract_terms = *h_contract_terms;
   ctx.h_wire = *h_wire;
   ctx.coin_pub = *coin_pub;
-  ctx.transaction_id = transaction_id;
   ctx.res = GNUNET_SYSERR;
   ret = TEH_plugin->wire_lookup_deposit_wtid (TEH_plugin->cls,
                                               session,
-					      h_contract,
+					      h_contract_terms,
 					      h_wire,
 					      coin_pub,
 					      merchant_pub,
-					      transaction_id,
 					      &handle_wtid_data,
 					      &ctx);
   if (GNUNET_SYSERR == ret)
@@ -2231,4 +2321,144 @@ TEH_DB_execute_track_transaction (struct MHD_Connection *connection,
 }
 
 
+/**
+ * Execute a "/payback".  The validity of the coin and signature have
+ * already been checked.  The database must now check that the coin is
+ * not (double) spent, and execute the transaction (record details,
+ * generate success or failure response).
+ *
+ * @param connection the MHD connection to handle
+ * @param coin information about the coin
+ * @param value how much are coins of the @a coin's denomination worth?
+ * @param h_blind blinded coin to use for the lookup
+ * @param coin_blind blinding factor used (for later verification by the auditor)
+ * @param coin_sig signature of the coin (to be stored)
+ * @return MHD result code
+ */
+int
+TEH_DB_execute_payback (struct MHD_Connection *connection,
+                        const struct TALER_CoinPublicInfo *coin,
+                        const struct TALER_Amount *value,
+                        const struct GNUNET_HashCode *h_blind,
+                        const struct TALER_DenominationBlindingKeyP *coin_blind,
+                        const struct TALER_CoinSpendSignatureP *coin_sig)
+{
+  int ret;
+  struct TALER_EXCHANGEDB_Session *session;
+  struct TALER_EXCHANGEDB_TransactionList *tl;
+  struct TALER_ReservePublicKeyP reserve_pub;
+  struct TALER_Amount amount;
+  struct TALER_Amount spent;
+  struct GNUNET_TIME_Absolute now;
+
+  if (NULL == (session = TEH_plugin->get_session (TEH_plugin->cls)))
+  {
+    GNUNET_break (0);
+    return TEH_RESPONSE_reply_internal_db_error (connection,
+						 TALER_EC_DB_SETUP_FAILED);
+  }
+
+  START_TRANSACTION (session, connection);
+
+  /* Check whether a payback is allowed, and if so, to which
+     reserve / account the money should go */
+  ret = TEH_plugin->get_reserve_by_h_blind (TEH_plugin->cls,
+                                            session,
+                                            h_blind,
+                                            &reserve_pub);
+  if (GNUNET_SYSERR == ret)
+  {
+    GNUNET_break (0);
+    TEH_plugin->rollback (TEH_plugin->cls,
+                          session);
+    return TEH_RESPONSE_reply_internal_db_error (connection,
+						 TALER_EC_PAYBACK_DB_FETCH_FAILED);
+  }
+  if (GNUNET_NO == ret)
+  {
+    GNUNET_break_op (0);
+    TEH_plugin->rollback (TEH_plugin->cls,
+                          session);
+    return TEH_RESPONSE_reply_payback_unknown (connection,
+                                               TALER_EC_PAYBACK_WITHDRAW_NOT_FOUND);
+  }
+
+  /* Calculate remaining balance. */
+  tl = TEH_plugin->get_coin_transactions (TEH_plugin->cls,
+                                          session,
+                                          &coin->coin_pub);
+  TALER_amount_get_zero (value->currency,
+                         &spent);
+  if (GNUNET_OK !=
+      calculate_transaction_list_totals (tl,
+                                         &spent,
+                                         &spent))
+  {
+    GNUNET_break (0);
+    TEH_plugin->rollback (TEH_plugin->cls,
+                          session);
+    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                            tl);
+    return TEH_RESPONSE_reply_internal_db_error (connection,
+						 TALER_EC_PAYBACK_HISTORY_DB_ERROR);
+  }
+  if (GNUNET_SYSERR ==
+      TALER_amount_subtract (&amount,
+                             value,
+                             &spent))
+  {
+    GNUNET_break (0);
+    TEH_plugin->rollback (TEH_plugin->cls,
+                          session);
+    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                            tl);
+    return TEH_RESPONSE_reply_internal_db_error (connection,
+						 TALER_EC_PAYBACK_COIN_BALANCE_NEGATIVE);
+  }
+  if ( (0 == amount.fraction) &&
+       (0 == amount.value) )
+  {
+    TEH_plugin->rollback (TEH_plugin->cls,
+                          session);
+    ret = TEH_RESPONSE_reply_coin_insufficient_funds (connection,
+                                                      TALER_EC_PAYBACK_COIN_BALANCE_ZERO,
+                                                      tl);
+    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                            tl);
+    return ret;
+  }
+  TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                          tl);
+  now = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&now);
+
+  /* add coin to list of wire transfers for payback */
+  ret = TEH_plugin->insert_payback_request (TEH_plugin->cls,
+                                            session,
+                                            &reserve_pub,
+                                            coin,
+                                            coin_sig,
+                                            coin_blind,
+                                            &amount,
+                                            h_blind,
+                                            now);
+  if (GNUNET_SYSERR == ret)
+  {
+    TALER_LOG_WARNING ("Failed to store /payback information in database\n");
+    TEH_plugin->rollback (TEH_plugin->cls,
+                          session);
+    return TEH_RESPONSE_reply_internal_db_error (connection,
+						 TALER_EC_PAYBACK_DB_PUT_FAILED);
+  }
+
+  COMMIT_TRANSACTION(session, connection);
+
+  return TEH_RESPONSE_reply_payback_success (connection,
+                                             &coin->coin_pub,
+                                             &reserve_pub,
+                                             &amount,
+                                             now);
+}
+
+
 /* end of taler-exchange-httpd_db.c */
