@@ -133,7 +133,7 @@ struct TEH_DB_MeltDetails
  * @param session_hash hash code of the session the coins are melted into
  * @param num_new_denoms number of entries in @a denom_pubs, size of y-dimension of @a commit_coin array
  * @param denom_pubs array of public denomination keys for the refresh (?)
- * @param coin_melt_details signatures and (residual) value of and information about the respective coin to be melted
+ * @param coin_melt_detail signatures and (residual) value of and information about the respective coin to be melted
  * @param commit_coin 2d array of coin commitments (what the exchange is to sign
  *                    once the "/refres/reveal" of cut and choose is done)
  * @param transfer_pubs array of transfer public keys (what the exchange is
@@ -146,7 +146,7 @@ TEH_DB_execute_refresh_melt (struct MHD_Connection *connection,
                              const struct GNUNET_HashCode *session_hash,
                              unsigned int num_new_denoms,
                              const struct TALER_DenominationPublicKey *denom_pubs,
-                             const struct TEH_DB_MeltDetails *coin_melt_details,
+                             const struct TEH_DB_MeltDetails *coin_melt_detail,
                              struct TALER_EXCHANGEDB_RefreshCommitCoin *const* commit_coin,
                              const struct TALER_TransferPublicKeyP *transfer_pubs);
 
@@ -214,7 +214,7 @@ TEH_DB_execute_admin_add_incoming (struct MHD_Connection *connection,
  */
 int
 TEH_DB_execute_track_transfer (struct MHD_Connection *connection,
-                              const struct TALER_WireTransferIdentifierRawP *wtid);
+                               const struct TALER_WireTransferIdentifierRawP *wtid);
 
 
 /**
@@ -222,20 +222,41 @@ TEH_DB_execute_track_transfer (struct MHD_Connection *connection,
  * associated with the given deposit.
  *
  * @param connection the MHD connection to handle
- * @param h_contract hash of the contract
+ * @param h_contract_terms hash of the contract
  * @param h_wire hash of the wire details
  * @param coin_pub public key of the coin to link
  * @param merchant_pub public key of the merchant
- * @param transaction_id transaction ID of the merchant
  * @return MHD result code
  */
 int
 TEH_DB_execute_track_transaction (struct MHD_Connection *connection,
-                             const struct GNUNET_HashCode *h_contract,
-			     const struct GNUNET_HashCode *h_wire,
-			     const struct TALER_CoinSpendPublicKeyP *coin_pub,
-			     const struct TALER_MerchantPublicKeyP *merchant_pub,
-			     uint64_t transaction_id);
+                                  const struct GNUNET_HashCode *h_contract_terms,
+                                  const struct GNUNET_HashCode *h_wire,
+                                  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                                  const struct TALER_MerchantPublicKeyP *merchant_pub);
+
+
+/**
+ * Execute a "/payback".  The validity of the coin and signature have
+ * already been checked.  The database must now check that the coin is
+ * not (double) spent, and execute the transaction (record details,
+ * generate success or failure response).
+ *
+ * @param connection the MHD connection to handle
+ * @param coin information about the coin
+ * @param value how much are coins of the @a coin's denomination worth?
+ * @param h_blind blinded coin to use for the lookup
+ * @param coin_blind blinding factor used (for later verification by the auditor)
+ * @param coin_sig signature of the coin
+ * @return MHD result code
+ */
+int
+TEH_DB_execute_payback (struct MHD_Connection *connection,
+                        const struct TALER_CoinPublicInfo *coin,
+                        const struct TALER_Amount *value,
+                        const struct GNUNET_HashCode *h_blind,
+                        const struct TALER_DenominationBlindingKeyP *coin_blind,
+                        const struct TALER_CoinSpendSignatureP *coin_sig);
 
 
 #endif
