@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015 GNUnet e.V.
+  Copyright (C) 2014-2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -28,6 +28,20 @@
 #include "taler_exchangedb_plugin.h"
 
 
+/**
+ * Taler protocol version in the format CURRENT:REVISION:AGE
+ * as used by GNU libtool.  See
+ * https://www.gnu.org/software/libtool/manual/html_node/Libtool-versioning.html
+ *
+ * Please be very careful when updating and follow
+ * https://www.gnu.org/software/libtool/manual/html_node/Updating-version-info.html#Updating-version-info
+ * precisely.  Note that this version has NOTHING to do with the
+ * release version, and the format is NOT the same that semantic
+ * versioning uses either.
+ */
+#define TALER_PROTOCOL_VERSION "0:0:0"
+
+
 /**
  * Snapshot of the (coin and signing) keys (including private keys) of
  * the exchange.  There can be multiple instances of this struct, as it is
@@ -58,6 +72,12 @@ struct TEH_KS_StateHandle
    */
   json_t *auditors_array;
 
+  /**
+   * JSON array with revoked denomination keys. (Currently not really used
+   * after initialization).
+   */
+  json_t *payback_array;
+
   /**
    * Cached JSON text that the exchange will send for a "/keys" request.
    * Includes our @e TEH_master_public_key public key, the signing and
@@ -65,12 +85,28 @@ struct TEH_KS_StateHandle
    */
   char *keys_json;
 
+  /**
+   * deflate-compressed version of @e keys_json, or NULL if not available.
+   */
+  void *keys_jsonz;
+
+  /**
+   * Number of bytes in @e keys_jsonz.
+   */
+  size_t keys_jsonz_size;
+
   /**
    * Mapping from denomination keys to denomination key issue struct.
    * Used to lookup the key by hash.
    */
   struct GNUNET_CONTAINER_MultiHashMap *denomkey_map;
 
+  /**
+   * Mapping from revoked denomination keys to denomination key issue struct.
+   * Used to lookup the key by hash.
+   */
+  struct GNUNET_CONTAINER_MultiHashMap *revoked_map;
+
   /**
    * Hash context we used to combine the hashes of all denomination
    * keys into one big hash.
@@ -204,12 +240,80 @@ TALER_EXCHANGE_conf_duration_provide ()
 }
 
 
+/**
+ * Store a copy of @a dki in @a map.
+ *
+ * @param map hash map to store @a dki in
+ * @param dki information to store in @a map
+ * @return #GNUNET_OK on success,
+ *         #GNUNET_NO if such an entry already exists
+ */
+static int
+store_in_map (struct GNUNET_CONTAINER_MultiHashMap *map,
+              const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki)
+{
+  struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *d2;
+  int res;
+
+  d2 = GNUNET_new (struct TALER_EXCHANGEDB_DenominationKeyIssueInformation);
+  d2->issue = dki->issue;
+  d2->denom_priv.rsa_private_key
+    = GNUNET_CRYPTO_rsa_private_key_dup (dki->denom_priv.rsa_private_key);
+  d2->denom_pub.rsa_public_key
+    = GNUNET_CRYPTO_rsa_public_key_dup (dki->denom_pub.rsa_public_key);
+  res = GNUNET_CONTAINER_multihashmap_put (map,
+                                           &d2->issue.properties.denom_hash,
+                                           d2,
+                                           GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY);
+  if (GNUNET_OK != res)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Duplicate denomination key `%s'\n",
+                GNUNET_h2s (&d2->issue.properties.denom_hash));
+    GNUNET_CRYPTO_rsa_private_key_free (d2->denom_priv.rsa_private_key);
+    GNUNET_CRYPTO_rsa_public_key_free (d2->denom_pub.rsa_public_key);
+    GNUNET_free (d2);
+    return GNUNET_NO;
+  }
+  return GNUNET_OK;
+}
+
+
+/**
+ * Handle a signal, writing relevant signal numbers to the pipe.
+ *
+ * @param signal_number the signal number
+ */
+static void
+handle_signal (int signal_number)
+{
+  ssize_t res;
+  char c = signal_number;
+
+  res = write (reload_pipe[1],
+               &c,
+               1);
+  if ( (res < 0) &&
+       (EINTR != errno) )
+  {
+    GNUNET_break (0);
+    return;
+  }
+  if (0 == res)
+  {
+    GNUNET_break (0);
+    return;
+  }
+}
+
+
 /**
  * Iterator for (re)loading/initializing denomination keys.
  *
  * @param cls closure
  * @param dki the denomination key issue
  * @param alias coin alias
+ * @param revocation_master_sig non-NULL if @a dki was revoked
  * @return #GNUNET_OK to continue to iterate,
  *  #GNUNET_NO to stop iteration with no error,
  *  #GNUNET_SYSERR to abort iteration with error!
@@ -217,29 +321,21 @@ TALER_EXCHANGE_conf_duration_provide ()
 static int
 reload_keys_denom_iter (void *cls,
                         const char *alias,
-                        const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki)
+                        const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki,
+                        const struct TALER_MasterSignatureP *revocation_master_sig)
 {
   struct TEH_KS_StateHandle *ctx = cls;
   struct GNUNET_TIME_Absolute now;
+  struct GNUNET_TIME_Absolute start;
   struct GNUNET_TIME_Absolute horizon;
   struct GNUNET_TIME_Absolute expire_deposit;
   struct GNUNET_HashCode denom_key_hash;
-  struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *d2;
   struct TALER_EXCHANGEDB_Session *session;
   int res;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Loading denomination key `%s'\n",
               alias);
-  horizon = GNUNET_TIME_relative_to_absolute (TALER_EXCHANGE_conf_duration_provide ());
-  if (GNUNET_TIME_absolute_ntoh (dki->issue.properties.start).abs_value_us >
-      horizon.abs_value_us)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "Skipping future denomination key `%s'\n",
-                alias);
-    return GNUNET_OK;
-  }
   now = GNUNET_TIME_absolute_get ();
   expire_deposit = GNUNET_TIME_absolute_ntoh (dki->issue.properties.expire_deposit);
   if (expire_deposit.abs_value_us < now.abs_value_us)
@@ -249,13 +345,6 @@ reload_keys_denom_iter (void *cls,
                 alias);
     return GNUNET_OK;
   }
-
-  GNUNET_CRYPTO_rsa_public_key_hash (dki->denom_pub.rsa_public_key,
-                                     &denom_key_hash);
-  GNUNET_CRYPTO_hash_context_read (ctx->hash_context,
-                                   &denom_key_hash,
-                                   sizeof (struct GNUNET_HashCode));
-
   if (0 != memcmp (&dki->issue.properties.master,
                    &TEH_master_public_key,
                    sizeof (struct TALER_MasterPublicKeyP)))
@@ -266,6 +355,86 @@ reload_keys_denom_iter (void *cls,
     return GNUNET_OK;
   }
 
+  session = TEH_plugin->get_session (TEH_plugin->cls);
+  if (NULL == session)
+    return GNUNET_SYSERR;
+
+  if (NULL != revocation_master_sig)
+  {
+    unsigned int thresh = 0;
+
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Adding denomination key `%s' to revokation set\n",
+                alias);
+    res = store_in_map (ctx->revoked_map,
+                        dki);
+    if (GNUNET_NO == res)
+      return GNUNET_OK;
+    /* Try to insert DKI into DB until we succeed; note that if the DB
+       failure is persistent, we need to die, as we cannot continue
+       without the DKI being in the DB). */
+    res = GNUNET_SYSERR;
+    while (GNUNET_OK != res)
+    {
+      thresh++;
+      if (thresh > 16)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                    "Giving up, this is fatal. Committing suicide via SIGTERM.\n");
+        handle_signal (SIGTERM);
+        return GNUNET_SYSERR;
+      }
+      res = TEH_plugin->start (TEH_plugin->cls,
+                               session);
+      if (GNUNET_OK != res)
+      {
+        /* Transaction start failed!? Very bad error, log and retry */
+        GNUNET_break (0);
+        continue;
+      }
+      res = TEH_plugin->insert_denomination_revocation (TEH_plugin->cls,
+                                                        session,
+                                                        &dki->issue.properties.denom_hash,
+                                                        revocation_master_sig);
+      if (GNUNET_SYSERR == res)
+      {
+        GNUNET_break (0);
+        TEH_plugin->rollback (TEH_plugin->cls,
+                              session);
+        continue;
+      }
+      if (GNUNET_NO == res)
+      {
+        TEH_plugin->rollback (TEH_plugin->cls,
+                              session);
+        break; /* already in is also OK! */
+      }
+      res = TEH_plugin->commit (TEH_plugin->cls,
+                                session);
+    }
+    GNUNET_assert (0 ==
+                   json_array_append_new (ctx->payback_array,
+                                          GNUNET_JSON_from_data_auto (&dki->issue.properties.denom_hash)));
+    return GNUNET_OK;
+  }
+  horizon = GNUNET_TIME_relative_to_absolute (TALER_EXCHANGE_conf_duration_provide ());
+  start = GNUNET_TIME_absolute_ntoh (dki->issue.properties.start);
+  if (start.abs_value_us > horizon.abs_value_us)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Skipping future denomination key `%s' (starts at %s)\n",
+                alias,
+                GNUNET_STRINGS_absolute_time_to_string (start));
+    return GNUNET_OK;
+  }
+
+  GNUNET_CRYPTO_rsa_public_key_hash (dki->denom_pub.rsa_public_key,
+                                     &denom_key_hash);
+  GNUNET_CRYPTO_hash_context_read (ctx->hash_context,
+                                   &denom_key_hash,
+                                   sizeof (struct GNUNET_HashCode));
+
+
 
   session = TEH_plugin->get_session (TEH_plugin->cls);
   if (NULL == session)
@@ -325,26 +494,10 @@ reload_keys_denom_iter (void *cls,
        as it is possible we just retry until we succeed. */
   }
 
-  d2 = GNUNET_new (struct TALER_EXCHANGEDB_DenominationKeyIssueInformation);
-  d2->issue = dki->issue;
-  d2->denom_priv.rsa_private_key
-    = GNUNET_CRYPTO_rsa_private_key_dup (dki->denom_priv.rsa_private_key);
-  d2->denom_pub.rsa_public_key
-    = GNUNET_CRYPTO_rsa_public_key_dup (dki->denom_pub.rsa_public_key);
-  res = GNUNET_CONTAINER_multihashmap_put (ctx->denomkey_map,
-                                           &denom_key_hash,
-                                           d2,
-                                           GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY);
-  if (GNUNET_OK != res)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Duplicate denomination key `%s'\n",
-                alias);
-    GNUNET_CRYPTO_rsa_private_key_free (d2->denom_priv.rsa_private_key);
-    GNUNET_CRYPTO_rsa_public_key_free (d2->denom_pub.rsa_public_key);
-    GNUNET_free (d2);
+  res = store_in_map (ctx->denomkey_map,
+                      dki);
+  if (GNUNET_NO == res)
     return GNUNET_OK;
-  }
   ctx->min_dk_expire = GNUNET_TIME_absolute_min (ctx->min_dk_expire,
                                                  expire_deposit);
   GNUNET_assert (0 ==
@@ -592,6 +745,11 @@ ks_release_ (struct TEH_KS_StateHandle *key_state)
       json_decref (key_state->denom_keys_array);
       key_state->denom_keys_array = NULL;
     }
+    if (NULL != key_state->payback_array)
+    {
+      json_decref (key_state->payback_array);
+      key_state->payback_array = NULL;
+    }
     if (NULL != key_state->sign_keys_array)
     {
       json_decref (key_state->sign_keys_array);
@@ -605,7 +763,16 @@ ks_release_ (struct TEH_KS_StateHandle *key_state)
       GNUNET_CONTAINER_multihashmap_destroy (key_state->denomkey_map);
       key_state->denomkey_map = NULL;
     }
+    if (NULL != key_state->revoked_map)
+    {
+      GNUNET_CONTAINER_multihashmap_iterate (key_state->revoked_map,
+                                             &free_denom_key,
+                                             key_state);
+      GNUNET_CONTAINER_multihashmap_destroy (key_state->revoked_map);
+      key_state->revoked_map = NULL;
+    }
     GNUNET_free_non_null (key_state->keys_json);
+    GNUNET_free_non_null (key_state->keys_jsonz);
     GNUNET_free (key_state);
   }
 }
@@ -660,6 +827,9 @@ TEH_KS_acquire_ (const char *location)
     key_state->denom_keys_array = json_array ();
     GNUNET_assert (NULL != key_state->denom_keys_array);
 
+    key_state->payback_array = json_array ();
+    GNUNET_assert (NULL != key_state->payback_array);
+
     key_state->sign_keys_array = json_array ();
     GNUNET_assert (NULL != key_state->sign_keys_array);
 
@@ -668,14 +838,22 @@ TEH_KS_acquire_ (const char *location)
 
     key_state->denomkey_map = GNUNET_CONTAINER_multihashmap_create (32,
                                                                     GNUNET_NO);
+    key_state->revoked_map = GNUNET_CONTAINER_multihashmap_create (4,
+                                                                   GNUNET_NO);
     key_state->reload_time = GNUNET_TIME_absolute_get ();
     GNUNET_TIME_round_abs (&key_state->reload_time);
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Loading keys from `%s'\n",
                 TEH_exchange_directory);
-    TALER_EXCHANGEDB_denomination_keys_iterate (TEH_exchange_directory,
-                                                &reload_keys_denom_iter,
-                                                key_state);
+    if (-1 == TALER_EXCHANGEDB_denomination_keys_iterate (TEH_exchange_directory,
+                                                          &TEH_master_public_key,
+                                                          &reload_keys_denom_iter,
+                                                          key_state))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR, "Can't load denomination keys.\n");
+      GNUNET_assert (0 == pthread_mutex_unlock (&internal_key_state_mutex));
+      return NULL;
+    }
     TALER_EXCHANGEDB_signing_keys_iterate (TEH_exchange_directory,
                                            &reload_keys_sign_iter,
                                            key_state);
@@ -693,6 +871,14 @@ TEH_KS_acquire_ (const char *location)
       return NULL;
     }
 
+    if (0 == GNUNET_CONTAINER_multihashmap_size (key_state->denomkey_map))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Have no denomination keys. Bad configuration.\n");
+      GNUNET_assert (0 == pthread_mutex_unlock (&internal_key_state_mutex));
+      return NULL;
+    }
+
     ks.purpose.size = htonl (sizeof (ks));
     ks.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_KEY_SET);
     ks.list_issue_date = GNUNET_TIME_absolute_hton (key_state->reload_time);
@@ -710,11 +896,13 @@ TEH_KS_acquire_ (const char *location)
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "No valid signing key found!\n");
 
-    keys = json_pack ("{s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+    keys = json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+                      "version", TALER_PROTOCOL_VERSION,
                       "master_public_key",
                       GNUNET_JSON_from_data_auto (&TEH_master_public_key),
                       "signkeys", key_state->sign_keys_array,
                       "denoms", key_state->denom_keys_array,
+                      "payback", key_state->payback_array,
                       "auditors", key_state->auditors_array,
                       "list_issue_date", GNUNET_JSON_from_time_abs (key_state->reload_time),
                       "eddsa_pub", GNUNET_JSON_from_data_auto (&key_state->current_sign_key_issue.issue.signkey_pub),
@@ -723,10 +911,22 @@ TEH_KS_acquire_ (const char *location)
     key_state->auditors_array = NULL;
     key_state->sign_keys_array = NULL;
     key_state->denom_keys_array = NULL;
+    key_state->payback_array = NULL;
     key_state->keys_json = json_dumps (keys,
                                        JSON_INDENT (2));
     GNUNET_assert (NULL != key_state->keys_json);
     json_decref (keys);
+    /* also compute compressed version of /keys */
+    key_state->keys_jsonz = GNUNET_strdup (key_state->keys_json);
+    key_state->keys_jsonz_size = strlen (key_state->keys_json);
+    if (MHD_YES !=
+	TEH_RESPONSE_body_compress (&key_state->keys_jsonz,
+				    &key_state->keys_jsonz_size))
+    {
+      GNUNET_free (key_state->keys_jsonz);
+      key_state->keys_jsonz = NULL;
+      key_state->keys_jsonz_size = 0;
+    }
     internal_key_state = key_state;
   }
   key_state = internal_key_state;
@@ -754,10 +954,12 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
   struct GNUNET_HashCode hc;
   struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki;
   struct GNUNET_TIME_Absolute now;
+  const struct GNUNET_CONTAINER_MultiHashMap *map;
 
   GNUNET_CRYPTO_rsa_public_key_hash (denom_pub->rsa_public_key,
                                      &hc);
-  dki = GNUNET_CONTAINER_multihashmap_get (key_state->denomkey_map,
+  map = (TEH_KS_DKU_PAYBACK == use) ? key_state->revoked_map : key_state->denomkey_map;
+  dki = GNUNET_CONTAINER_multihashmap_get (map,
 					   &hc);
   if (NULL == dki)
     return NULL;
@@ -793,39 +995,21 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
       return NULL;
     }
     break;
+  case TEH_KS_DKU_PAYBACK:
+    if (now.abs_value_us >
+	GNUNET_TIME_absolute_ntoh (dki->issue.properties.expire_deposit).abs_value_us)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+		  "Not returning DKI for %s, as time to payback coin has passed\n",
+		  GNUNET_h2s (&hc));
+      return NULL;
+    }
+    break;
   }
   return dki;
 }
 
 
-/**
- * Handle a signal, writing relevant signal numbers to the pipe.
- *
- * @param signal_number the signal number
- */
-static void
-handle_signal (int signal_number)
-{
-  ssize_t res;
-  char c = signal_number;
-
-  res = write (reload_pipe[1],
-               &c,
-               1);
-  if ( (res < 0) &&
-       (EINTR != errno) )
-  {
-    GNUNET_break (0);
-    return;
-  }
-  if (0 == res)
-  {
-    GNUNET_break (0);
-    return;
-  }
-}
-
-
 /**
  * Call #handle_signal() to pass the received signal via
  * the control pipe.
@@ -1089,10 +1273,26 @@ TEH_KS_handler_keys (struct TEH_RequestHandler *rh,
   struct MHD_Response *response;
   int ret;
   char dat[128];
+  char *json;
+  size_t json_len;
+  int comp;
 
   key_state = TEH_KS_acquire ();
-  response = MHD_create_response_from_buffer (strlen (key_state->keys_json),
-                                              key_state->keys_json,
+  comp = MHD_NO;
+  if (NULL != key_state->keys_jsonz)
+    comp = TEH_RESPONSE_can_compress (connection);
+  if (MHD_YES == comp)
+  {
+    json = key_state->keys_jsonz;
+    json_len = key_state->keys_jsonz_size;
+  }
+  else
+  {
+    json = key_state->keys_json;
+    json_len = strlen (key_state->keys_json);
+  }
+  response = MHD_create_response_from_buffer (json_len,
+                                              json,
                                               MHD_RESPMEM_MUST_COPY);
   TEH_KS_release (key_state);
   if (NULL == response)
@@ -1105,6 +1305,15 @@ TEH_KS_handler_keys (struct TEH_RequestHandler *rh,
                 MHD_add_response_header (response,
                                          MHD_HTTP_HEADER_CONTENT_TYPE,
                                          rh->mime_type));
+  if (MHD_YES !=
+      MHD_add_response_header (response,
+			       MHD_HTTP_HEADER_CONTENT_ENCODING,
+			       "deflate"))
+  {
+    GNUNET_break (0);
+    MHD_destroy_response (response);
+    return MHD_NO;
+  }
   get_date_string (key_state->reload_time,
                    dat);
   GNUNET_break (MHD_YES ==
