@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 Inria & GNUnet e.V.
+  Copyright (C) 2014-2017 Inria & GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -23,6 +23,7 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
+#include <zlib.h>
 #include "taler-exchange-httpd_responses.h"
 #include "taler_util.h"
 #include "taler_json_lib.h"
@@ -47,6 +48,74 @@ TEH_RESPONSE_add_global_headers (struct MHD_Response *response)
 }
 
 
+/**
+ * Is HTTP body deflate compression supported by the client?
+ *
+ * @param connection connection to check
+ * @return #MHD_YES if 'deflate' compression is allowed
+ */
+int
+TEH_RESPONSE_can_compress (struct MHD_Connection *connection)
+{
+  const char *ae;
+  const char *de;
+
+  ae = MHD_lookup_connection_value (connection,
+				    MHD_HEADER_KIND,
+				    MHD_HTTP_HEADER_ACCEPT_ENCODING);
+  if (NULL == ae)
+    return MHD_NO;
+  de = strstr (ae,
+	       "deflate");
+  if (NULL == de)
+    return MHD_NO;
+  if ( ( (de == ae) ||
+	 ( de[-1] == ',') ||
+	 (de[-1] == ' ') ) &&
+       ( (de[strlen ("deflate")] == '\0') ||
+	 (de[strlen ("deflate")] == ',') ) )
+    return MHD_YES;
+  return MHD_NO;
+}
+
+
+/**
+ * Try to compress a response body.  Updates @a buf and @a buf_size.
+ *
+ * @param[in,out] buf pointer to body to compress
+ * @param[in,out] buf_size pointer to initial size of @a buf
+ * @return #MHD_YES if @a buf was compressed
+ */
+int
+TEH_RESPONSE_body_compress (void **buf,
+			    size_t *buf_size)
+{
+  Bytef *cbuf;
+  uLongf cbuf_size;
+  int ret;
+
+  cbuf_size = compressBound (*buf_size);
+  cbuf = malloc (cbuf_size);
+  if (NULL == cbuf)
+    return MHD_NO;
+  ret = compress (cbuf,
+		  &cbuf_size,
+		  (const Bytef *) *buf,
+		  *buf_size);
+  if ( (Z_OK != ret) ||
+       (cbuf_size >= *buf_size) )
+  {
+    /* compression failed */
+    free (cbuf);
+    return MHD_NO;
+  }
+  free (*buf);
+  *buf = (void *) cbuf;
+  *buf_size = (size_t) cbuf_size;
+  return MHD_YES;
+}
+
+
 /**
  * Send JSON object as response.
  *
@@ -61,12 +130,26 @@ TEH_RESPONSE_reply_json (struct MHD_Connection *connection,
                          unsigned int response_code)
 {
   struct MHD_Response *resp;
-  char *json_str;
+  void *json_str;
+  size_t json_len;
   int ret;
+  int comp;
 
-  json_str = json_dumps (json, JSON_INDENT(2));
-  GNUNET_assert (NULL != json_str);
-  resp = MHD_create_response_from_buffer (strlen (json_str),
+  json_str = json_dumps (json,
+			 JSON_INDENT(2));
+  if (NULL == json_str)
+  {
+    GNUNET_break (0);
+    return MHD_NO;
+  }
+  json_len = strlen (json_str);
+  /* try to compress the body */
+  comp = MHD_NO;
+  if (MHD_YES ==
+      TEH_RESPONSE_can_compress (connection))
+    comp = TEH_RESPONSE_body_compress (&json_str,
+				       &json_len);
+  resp = MHD_create_response_from_buffer (json_len,
                                           json_str,
                                           MHD_RESPMEM_MUST_FREE);
   if (NULL == resp)
@@ -79,6 +162,19 @@ TEH_RESPONSE_reply_json (struct MHD_Connection *connection,
   (void) MHD_add_response_header (resp,
                                   MHD_HTTP_HEADER_CONTENT_TYPE,
                                   "application/json");
+  if (MHD_YES == comp)
+  {
+    /* Need to indicate to client that body is compressed */
+    if (MHD_NO ==
+	MHD_add_response_header (resp,
+				 MHD_HTTP_HEADER_CONTENT_ENCODING,
+				 "deflate"))
+    {
+      GNUNET_break (0);
+      MHD_destroy_response (resp);
+      return MHD_NO;
+    }
+  }
   ret = MHD_queue_response (connection,
                             response_code,
                             resp);
@@ -211,7 +307,7 @@ TEH_RESPONSE_reply_arg_missing (struct MHD_Connection *connection,
 {
   return TEH_RESPONSE_reply_json_pack (connection,
                                        MHD_HTTP_BAD_REQUEST,
-                                       "{ s:s, s:I, s:s}",
+                                       "{s:s, s:I, s:s}",
                                        "error", "missing parameter",
 				       "code", (json_int_t) ec,
                                        "parameter", param_name);
@@ -376,8 +472,7 @@ TEH_RESPONSE_reply_invalid_json (struct MHD_Connection *connection)
  * @param connection connection to the client
  * @param coin_pub public key of the coin
  * @param h_wire hash of wire details
- * @param h_contract hash of contract details
- * @param transaction_id transaction ID
+ * @param h_contract_terms hash of contract details
  * @param timestamp client's timestamp
  * @param refund_deadline until when this deposit be refunded
  * @param merchant merchant public key
@@ -388,8 +483,7 @@ int
 TEH_RESPONSE_reply_deposit_success (struct MHD_Connection *connection,
                                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                                     const struct GNUNET_HashCode *h_wire,
-                                    const struct GNUNET_HashCode *h_contract,
-                                    uint64_t transaction_id,
+                                    const struct GNUNET_HashCode *h_contract_terms,
                                     struct GNUNET_TIME_Absolute timestamp,
                                     struct GNUNET_TIME_Absolute refund_deadline,
                                     const struct TALER_MerchantPublicKeyP *merchant,
@@ -401,9 +495,8 @@ TEH_RESPONSE_reply_deposit_success (struct MHD_Connection *connection,
 
   dc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT);
   dc.purpose.size = htonl (sizeof (struct TALER_DepositConfirmationPS));
-  dc.h_contract = *h_contract;
+  dc.h_contract_terms = *h_contract_terms;
   dc.h_wire = *h_wire;
-  dc.transaction_id = GNUNET_htonll (transaction_id);
   dc.timestamp = GNUNET_TIME_absolute_hton (timestamp);
   dc.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
   TALER_amount_hton (&dc.amount_without_fee,
@@ -431,15 +524,15 @@ TEH_RESPONSE_reply_deposit_success (struct MHD_Connection *connection,
 static json_t *
 compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
 {
-  json_t *details;
-  const char *type;
-  struct TALER_Amount value;
   json_t *history;
-  const struct GNUNET_CRYPTO_EddsaSignature *sig;
-  const struct TALER_EXCHANGEDB_TransactionList *pos;
 
   history = json_array ();
-  for (pos = tl; NULL != pos; pos = pos->next)
+  if (NULL == history)
+  {
+    GNUNET_break (0); /* out of memory!? */
+    return NULL;
+  }
+  for (const struct TALER_EXCHANGEDB_TransactionList *pos = tl; NULL != pos; pos = pos->next)
   {
     switch (pos->type)
     {
@@ -448,27 +541,23 @@ compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
         struct TALER_DepositRequestPS dr;
         const struct TALER_EXCHANGEDB_Deposit *deposit = pos->details.deposit;
 
-        type = "DEPOSIT";
-        value = deposit->amount_with_fee;
         dr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_DEPOSIT);
         dr.purpose.size = htonl (sizeof (struct TALER_DepositRequestPS));
-        dr.h_contract = deposit->h_contract;
+        dr.h_contract_terms = deposit->h_contract_terms;
         dr.h_wire = deposit->h_wire;
         dr.timestamp = GNUNET_TIME_absolute_hton (deposit->timestamp);
         dr.refund_deadline = GNUNET_TIME_absolute_hton (deposit->refund_deadline);
-        dr.transaction_id = GNUNET_htonll (deposit->transaction_id);
         TALER_amount_hton (&dr.amount_with_fee,
                            &deposit->amount_with_fee);
         TALER_amount_hton (&dr.deposit_fee,
                            &deposit->deposit_fee);
         dr.merchant = deposit->merchant_pub;
         dr.coin_pub = deposit->coin.coin_pub;
-        sig = &deposit->csig.eddsa_signature;
 	/* internal sanity check before we hand out a bogus sig... */
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_DEPOSIT,
                                         &dr.purpose,
-                                        sig,
+                                        &deposit->csig.eddsa_signature,
                                         &deposit->coin.coin_pub.eddsa_pub))
 	{
 	  GNUNET_break (0);
@@ -476,16 +565,25 @@ compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
 	  return NULL;
 	}
 
-        details = GNUNET_JSON_from_data_auto (&dr);
-        break;
+	GNUNET_assert (0 ==
+		       json_array_append_new (history,
+					      json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+							 "type", "DEPOSIT",
+							 "amount", TALER_JSON_from_amount (&deposit->amount_with_fee),
+							 "deposit_fee", TALER_JSON_from_amount (&deposit->deposit_fee),
+							 "timestamp", GNUNET_JSON_from_time_abs (deposit->timestamp),
+							 "refund_deadline", GNUNET_JSON_from_time_abs (deposit->refund_deadline),
+							 "merchant_pub", GNUNET_JSON_from_data_auto (&deposit->merchant_pub),
+							 "h_contract_terms", GNUNET_JSON_from_data_auto (&deposit->h_contract_terms),
+							 "h_wire", GNUNET_JSON_from_data_auto (&deposit->h_wire),
+							 "coin_sig", GNUNET_JSON_from_data_auto (&deposit->csig))));
+	break;
       }
     case TALER_EXCHANGEDB_TT_REFRESH_MELT:
       {
         struct TALER_RefreshMeltCoinAffirmationPS ms;
         const struct TALER_EXCHANGEDB_RefreshMelt *melt = pos->details.melt;
 
-        type = "MELT";
-        value = melt->amount_with_fee;
         ms.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT);
         ms.purpose.size = htonl (sizeof (struct TALER_RefreshMeltCoinAffirmationPS));
         ms.session_hash = melt->session_hash;
@@ -494,12 +592,11 @@ compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
         TALER_amount_hton (&ms.melt_fee,
                            &melt->melt_fee);
         ms.coin_pub = melt->coin.coin_pub;
-        sig = &melt->coin_sig.eddsa_signature;
 	/* internal sanity check before we hand out a bogus sig... */
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_MELT,
                                         &ms.purpose,
-                                        sig,
+                                        &melt->coin_sig.eddsa_signature,
                                         &melt->coin.coin_pub.eddsa_pub))
 	{
 	  GNUNET_break (0);
@@ -507,15 +604,22 @@ compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
 	  return NULL;
 	}
 
-        details = GNUNET_JSON_from_data_auto (&ms);
+	GNUNET_assert (0 ==
+		       json_array_append_new (history,
+					      json_pack ("{s:s, s:o, s:o, s:o, s:o}",
+							 "type", "MELT",
+							 "amount", TALER_JSON_from_amount (&melt->amount_with_fee),
+							 "melt_fee", TALER_JSON_from_amount (&melt->melt_fee),
+							 "session_hash", GNUNET_JSON_from_data_auto (&melt->session_hash),
+							 "coin_sig", GNUNET_JSON_from_data_auto (&melt->coin_sig))));
       }
       break;
     case TALER_EXCHANGEDB_TT_REFUND:
       {
         struct TALER_RefundRequestPS rr;
         const struct TALER_EXCHANGEDB_Refund *refund = pos->details.refund;
+	struct TALER_Amount value;
 
-        type = "REFUND";
         if (GNUNET_OK !=
             TALER_amount_subtract (&value,
                                    &refund->refund_amount,
@@ -527,8 +631,7 @@ compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
         }
         rr.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND);
         rr.purpose.size = htonl (sizeof (struct TALER_RefundRequestPS));
-        rr.h_contract = refund->h_contract;
-        rr.transaction_id = GNUNET_htonll (refund->transaction_id);
+        rr.h_contract_terms = refund->h_contract_terms;
         rr.coin_pub = refund->coin.coin_pub;
         rr.merchant = refund->merchant_pub;
         rr.rtransaction_id = GNUNET_htonll (refund->rtransaction_id);
@@ -537,59 +640,93 @@ compile_transaction_history (const struct TALER_EXCHANGEDB_TransactionList *tl)
         TALER_amount_hton (&rr.refund_fee,
                            &refund->refund_fee);
 	/* internal sanity check before we hand out a bogus sig... */
-        sig = &refund->merchant_sig.eddsa_sig;
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
                                         &rr.purpose,
-                                        sig,
+                                        &refund->merchant_sig.eddsa_sig,
                                         &refund->merchant_pub.eddsa_pub))
 	{
 	  GNUNET_break (0);
 	  json_decref (history);
 	  return NULL;
 	}
-        details = GNUNET_JSON_from_data_auto (&rr);
+
+	GNUNET_assert (0 ==
+		       json_array_append_new (history,
+					      json_pack ("{s:s, s:o, s:o, s:o, s:o, s:I, s:o}",
+							 "type", "REFUND",
+							 "amount", TALER_JSON_from_amount (&value),
+							 "refund_fee", TALER_JSON_from_amount (&refund->refund_fee),
+							 "h_contract_terms", GNUNET_JSON_from_data_auto (&refund->h_contract_terms),
+							 "merchant_pub", GNUNET_JSON_from_data_auto (&refund->merchant_pub),
+							 "rtransaction_id", (json_int_t) refund->rtransaction_id,
+							 "merchant_sig", GNUNET_JSON_from_data_auto (&refund->merchant_sig))));
+      }
+      break;
+    case TALER_EXCHANGEDB_TT_PAYBACK:
+      {
+        const struct TALER_EXCHANGEDB_Payback *payback = pos->details.payback;
+        struct TALER_PaybackConfirmationPS pc;
+        struct TALER_ExchangePublicKeyP epub;
+        struct TALER_ExchangeSignatureP esig;
+
+        pc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK);
+        pc.purpose.size = htonl (sizeof (pc));
+        pc.timestamp = GNUNET_TIME_absolute_hton (payback->timestamp);
+        TALER_amount_hton (&pc.payback_amount,
+                           &payback->value);
+        pc.coin_pub = payback->coin.coin_pub;
+        pc.reserve_pub = payback->reserve_pub;
+        TEH_KS_sign (&pc.purpose,
+                     &epub,
+                     &esig);
+        GNUNET_assert (0 ==
+                       json_array_append_new (history,
+                                              json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
+                                                         "type", "PAYBACK",
+                                                         "amount", TALER_JSON_from_amount (&payback->value),
+                                                         "exchange_sig", GNUNET_JSON_from_data_auto (&esig),
+                                                         "exchange_pub", GNUNET_JSON_from_data_auto (&epub),
+							 "reserve_pub", GNUNET_JSON_from_data_auto (&payback->reserve_pub),
+							 "timestamp", GNUNET_JSON_from_time_abs (payback->timestamp))));
       }
       break;
     default:
       GNUNET_assert (0);
     }
-    GNUNET_assert (0 ==
-                   json_array_append_new (history,
-                                          json_pack ("{s:s, s:o, s:o, s:o}",
-                                                     "type", type,
-                                                     "amount", TALER_JSON_from_amount (&value),
-                                                     "signature", GNUNET_JSON_from_data_auto (sig),
-                                                     "details", details)));
   }
   return history;
 }
 
 
 /**
- * Send proof that a /deposit request is invalid to client.  This
- * function will create a message with all of the operations affecting
- * the coin that demonstrate that the coin has insufficient value.
+ * Send proof that a request is invalid to client because of
+ * insufficient funds.  This function will create a message with all
+ * of the operations affecting the coin that demonstrate that the coin
+ * has insufficient value.
  *
  * @param connection connection to the client
+ * @param ec error code to return
  * @param tl transaction list to use to build reply
  * @return MHD result code
  */
 int
-TEH_RESPONSE_reply_deposit_insufficient_funds (struct MHD_Connection *connection,
-                                               const struct TALER_EXCHANGEDB_TransactionList *tl)
+TEH_RESPONSE_reply_coin_insufficient_funds (struct MHD_Connection *connection,
+                                            enum TALER_ErrorCode ec,
+                                            const struct TALER_EXCHANGEDB_TransactionList *tl)
 {
   json_t *history;
 
   history = compile_transaction_history (tl);
   if (NULL == history)
-    return TEH_RESPONSE_reply_internal_db_error (connection,
-						 TALER_EC_DEPOSIT_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS);
+    return TEH_RESPONSE_reply_internal_error (connection,
+                                              TALER_EC_COIN_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS,
+                                              "failed to convert transaction history to JSON");
   return TEH_RESPONSE_reply_json_pack (connection,
                                        MHD_HTTP_FORBIDDEN,
                                        "{s:s, s:I, s:o}",
                                        "error", "insufficient funds",
-				       "code", (json_int_t) TALER_EC_DEPOSIT_INSUFFICIENT_FUNDS,
+				       "code", (json_int_t) ec,
                                        "history", history);
 }
 
@@ -608,20 +745,17 @@ compile_reserve_history (const struct TALER_EXCHANGEDB_ReserveHistory *rh,
 {
   struct TALER_Amount deposit_total;
   struct TALER_Amount withdraw_total;
-  struct TALER_Amount value;
   json_t *json_history;
   int ret;
-  const struct TALER_EXCHANGEDB_ReserveHistory *pos;
-  struct TALER_WithdrawRequestPS wr;
 
   json_history = json_array ();
   ret = 0;
-  for (pos = rh; NULL != pos; pos = pos->next)
+  for (const struct TALER_EXCHANGEDB_ReserveHistory *pos = rh; NULL != pos; pos = pos->next)
   {
     switch (pos->type)
     {
     case TALER_EXCHANGEDB_RO_BANK_TO_EXCHANGE:
-      if (0 == ret)
+      if (0 == (1 & ret))
         deposit_total = pos->details.bank->amount;
       else
         if (GNUNET_OK !=
@@ -632,66 +766,153 @@ compile_reserve_history (const struct TALER_EXCHANGEDB_ReserveHistory *rh,
           json_decref (json_history);
           return NULL;
         }
-      ret = 1;
+      ret |= 1;
       GNUNET_assert (0 ==
                      json_array_append_new (json_history,
-                                            json_pack ("{s:s, s:O, s:O, s:o}",
+                                            json_pack ("{s:s, s:O, s:o, s:o}",
                                                        "type", "DEPOSIT",
                                                        "sender_account_details", pos->details.bank->sender_account_details,
-                                                       "transfer_details", pos->details.bank->transfer_details,
+                                                       "wire_reference", GNUNET_JSON_from_data (pos->details.bank->wire_reference,
+                                                                                                pos->details.bank->wire_reference_size),
                                                        "amount", TALER_JSON_from_amount (&pos->details.bank->amount))));
       break;
     case TALER_EXCHANGEDB_RO_WITHDRAW_COIN:
-      break;
-    }
-  }
+      {
+	struct GNUNET_HashCode h_denom_pub;
+	struct TALER_Amount value;
 
-  ret = 0;
-  for (pos = rh; NULL != pos; pos = pos->next)
-  {
-    switch (pos->type)
-    {
-    case TALER_EXCHANGEDB_RO_BANK_TO_EXCHANGE:
+	value = pos->details.withdraw->amount_with_fee;
+	if (0 == (2 & ret))
+	{
+	  withdraw_total = value;
+	}
+	else
+	{
+	  if (GNUNET_OK !=
+	      TALER_amount_add (&withdraw_total,
+				&withdraw_total,
+				&value))
+	  {
+	    json_decref (json_history);
+	    return NULL;
+	  }
+	}
+	ret |= 2;
+	GNUNET_CRYPTO_rsa_public_key_hash (pos->details.withdraw->denom_pub.rsa_public_key,
+					   &h_denom_pub);
+	GNUNET_assert (0 ==
+		       json_array_append_new (json_history,
+					      json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
+							 "type", "WITHDRAW",
+							 "reserve_sig", GNUNET_JSON_from_data_auto (&pos->details.withdraw->reserve_sig),
+							 "h_coin_envelope", GNUNET_JSON_from_data_auto (&pos->details.withdraw->h_coin_envelope),
+							 "h_denom_pub", GNUNET_JSON_from_data_auto (&h_denom_pub),
+							 "withdraw_fee", TALER_JSON_from_amount (&pos->details.withdraw->withdraw_fee),
+							 "amount", TALER_JSON_from_amount (&value))));
+      }
       break;
-    case TALER_EXCHANGEDB_RO_WITHDRAW_COIN:
-      value = pos->details.withdraw->amount_with_fee;
-      if (0 == ret)
+    case TALER_EXCHANGEDB_RO_PAYBACK_COIN:
       {
-        withdraw_total = value;
+	const struct TALER_EXCHANGEDB_Payback *payback;
+	struct TALER_PaybackConfirmationPS pc;
+	struct TALER_ExchangePublicKeyP pub;
+	struct TALER_ExchangeSignatureP sig;
+
+	payback = pos->details.payback;
+	if (0 == (1 & ret))
+	  deposit_total = payback->value;
+	else
+	  if (GNUNET_OK !=
+	      TALER_amount_add (&deposit_total,
+				&deposit_total,
+				&payback->value))
+	  {
+	    json_decref (json_history);
+	    return NULL;
+	  }
+	ret |= 1;
+	pc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK);
+	pc.purpose.size = htonl (sizeof (struct TALER_PaybackConfirmationPS));
+	pc.timestamp = GNUNET_TIME_absolute_hton (payback->timestamp);
+	TALER_amount_hton (&pc.payback_amount,
+			   &payback->value);
+	pc.coin_pub = payback->coin.coin_pub;
+	pc.reserve_pub = payback->reserve_pub;
+	TEH_KS_sign (&pc.purpose,
+		     &pub,
+		     &sig);
+
+	GNUNET_assert (0 ==
+		       json_array_append_new (json_history,
+					      json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
+							 "type", "PAYBACK",
+							 "exchange_pub", GNUNET_JSON_from_data_auto (&pub),
+							 "exchange_sig", GNUNET_JSON_from_data_auto (&sig),
+							 "timestamp", GNUNET_JSON_from_time_abs (payback->timestamp),
+							 "amount", TALER_JSON_from_amount (&payback->value),
+							 "coin_pub", GNUNET_JSON_from_data_auto (&payback->coin.coin_pub))));
       }
-      else
+      break;
+    case TALER_EXCHANGEDB_RO_EXCHANGE_TO_BANK:
       {
-        if (GNUNET_OK !=
-            TALER_amount_add (&withdraw_total,
-                              &withdraw_total,
-                              &value))
-        {
-          json_decref (json_history);
-          return NULL;
-        }
+	struct TALER_ReserveCloseConfirmationPS rcc;
+	struct TALER_ExchangePublicKeyP pub;
+	struct TALER_ExchangeSignatureP sig;
+	struct TALER_Amount value;
+
+	value = pos->details.closing->amount;
+	if (0 == (2 & ret))
+	{
+	  withdraw_total = value;
+	}
+	else
+	{
+	  if (GNUNET_OK !=
+	      TALER_amount_add (&withdraw_total,
+				&withdraw_total,
+				&value))
+	  {
+	    json_decref (json_history);
+	    return NULL;
+	  }
+	}
+	ret |= 2;
+	rcc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED);
+	rcc.purpose.size = htonl (sizeof (struct TALER_ReserveCloseConfirmationPS));
+	rcc.timestamp = GNUNET_TIME_absolute_hton (pos->details.closing->execution_date);
+	TALER_amount_hton (&rcc.closing_amount,
+			   &value);
+	TALER_amount_hton (&rcc.closing_fee,
+			   &pos->details.closing->closing_fee);
+	rcc.reserve_pub = pos->details.closing->reserve_pub;
+	TALER_JSON_hash (pos->details.closing->receiver_account_details,
+			 &rcc.h_wire);
+	rcc.wtid = pos->details.closing->wtid;
+	TEH_KS_sign (&rcc.purpose,
+		     &pub,
+		     &sig);
+	GNUNET_assert (0 ==
+		       json_array_append_new (json_history,
+					      json_pack ("{s:s, s:O, s:o, s:o, s:o, s:o, s:o, s:o}",
+							 "type", "CLOSING",
+							 "receiver_account_details", pos->details.closing->receiver_account_details,
+							 "wtid", GNUNET_JSON_from_data_auto (&pos->details.closing->wtid),
+							 "exchange_pub", GNUNET_JSON_from_data_auto (&pub),
+							 "exchange_sig", GNUNET_JSON_from_data_auto (&sig),
+							 "timestamp", GNUNET_JSON_from_time_abs (pos->details.closing->execution_date),
+							 "amount", TALER_JSON_from_amount (&value),
+							 "closing_fee", TALER_JSON_from_amount (&pos->details.closing->closing_fee))));
       }
-      ret = 1;
-      wr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW);
-      wr.purpose.size = htonl (sizeof (struct TALER_WithdrawRequestPS));
-      wr.reserve_pub = pos->details.withdraw->reserve_pub;
-      TALER_amount_hton (&wr.amount_with_fee,
-                         &value);
-      TALER_amount_hton (&wr.withdraw_fee,
-                         &pos->details.withdraw->withdraw_fee);
-      GNUNET_CRYPTO_rsa_public_key_hash (pos->details.withdraw->denom_pub.rsa_public_key,
-                                         &wr.h_denomination_pub);
-      wr.h_coin_envelope = pos->details.withdraw->h_coin_envelope;
-      GNUNET_assert (0 ==
-                     json_array_append_new (json_history,
-                                            json_pack ("{s:s, s:o, s:o, s:o}",
-                                                       "type", "WITHDRAW",
-                                                       "signature", GNUNET_JSON_from_data_auto (&pos->details.withdraw->reserve_sig),
-                                                       "details", GNUNET_JSON_from_data_auto (&wr),
-                                                       "amount", TALER_JSON_from_amount (&value))));
       break;
     }
   }
-  if (0 == ret)
+  if (0 == (1 & ret))
+  {
+    GNUNET_break (0);
+    json_decref (json_history);
+    return NULL;
+  }
+  if (0 == (2 & ret))
   {
     /* did not encounter any withdraw operations, set to zero */
     TALER_amount_get_zero (deposit_total.currency,
@@ -771,8 +992,7 @@ TEH_RESPONSE_reply_refund_success (struct MHD_Connection *connection,
 
   rc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND);
   rc.purpose.size = htonl (sizeof (struct TALER_RefundConfirmationPS));
-  rc.h_contract = refund->h_contract;
-  rc.transaction_id = GNUNET_htonll (refund->transaction_id);
+  rc.h_contract_terms = refund->h_contract_terms;
   rc.coin_pub = refund->coin.coin_pub;
   rc.merchant = refund->merchant_pub;
   rc.rtransaction_id = GNUNET_htonll (refund->rtransaction_id);
@@ -1167,23 +1387,21 @@ TEH_RESPONSE_reply_transfer_pending (struct MHD_Connection *connection,
  * them. Generates the 200 reply.
  *
  * @param connection connection to the client
- * @param h_contract hash of the contract
+ * @param h_contract_terms hash of the contract
  * @param h_wire hash of wire account details
  * @param coin_pub public key of the coin
  * @param coin_contribution how much did the coin we asked about
  *        contribute to the total transfer value? (deposit value minus fee)
- * @param transaction_id merchant transaction identifier
  * @param wtid raw wire transfer identifier
  * @param exec_time execution time of the wire transfer
  * @return MHD result code
  */
 int
 TEH_RESPONSE_reply_track_transaction (struct MHD_Connection *connection,
-                                      const struct GNUNET_HashCode *h_contract,
+                                      const struct GNUNET_HashCode *h_contract_terms,
                                       const struct GNUNET_HashCode *h_wire,
                                       const struct TALER_CoinSpendPublicKeyP *coin_pub,
                                       const struct TALER_Amount *coin_contribution,
-                                      uint64_t transaction_id,
                                       const struct TALER_WireTransferIdentifierRawP *wtid,
                                       struct GNUNET_TIME_Absolute exec_time)
 {
@@ -1194,10 +1412,9 @@ TEH_RESPONSE_reply_track_transaction (struct MHD_Connection *connection,
   cw.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_WIRE);
   cw.purpose.size = htonl (sizeof (struct TALER_ConfirmWirePS));
   cw.h_wire = *h_wire;
-  cw.h_contract = *h_contract;
+  cw.h_contract_terms = *h_contract_terms;
   cw.wtid = *wtid;
   cw.coin_pub = *coin_pub;
-  cw.transaction_id = GNUNET_htonll (transaction_id);
   cw.execution_time = GNUNET_TIME_absolute_hton (exec_time);
   TALER_amount_hton (&cw.coin_contribution,
                      coin_contribution);
@@ -1223,6 +1440,7 @@ TEH_RESPONSE_reply_track_transaction (struct MHD_Connection *connection,
  * @param total total amount that was transferred
  * @param merchant_pub public key of the merchant
  * @param h_wire destination account
+ * @param wire_fee wire fee that was charged
  * @param exec_time execution time of the wire transfer
  * @param wdd_head linked list with details about the combined deposits
  * @return MHD result code
@@ -1232,6 +1450,7 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
                                            const struct TALER_Amount *total,
                                            const struct TALER_MerchantPublicKeyP *merchant_pub,
                                            const struct GNUNET_HashCode *h_wire,
+                                           const struct TALER_Amount *wire_fee,
                                            struct GNUNET_TIME_Absolute exec_time,
                                            const struct TEH_TrackTransferDetail *wdd_head)
 {
@@ -1248,9 +1467,8 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
   hash_context = GNUNET_CRYPTO_hash_context_start ();
   for (wdd_pos = wdd_head; NULL != wdd_pos; wdd_pos = wdd_pos->next)
   {
-    dd.h_contract = wdd_pos->h_contract;
+    dd.h_contract_terms = wdd_pos->h_contract_terms;
     dd.execution_time = GNUNET_TIME_absolute_hton (exec_time);
-    dd.transaction_id = GNUNET_htonll (wdd_pos->transaction_id);
     dd.coin_pub = wdd_pos->coin_pub;
     TALER_amount_hton (&dd.deposit_value,
                        &wdd_pos->deposit_value);
@@ -1261,9 +1479,8 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
                                      sizeof (struct TALER_WireDepositDetailP));
     GNUNET_assert (0 ==
                    json_array_append_new (deposits,
-                                          json_pack ("{s:o, s:I, s:o, s:o, s:o}",
-                                                     "H_contract", GNUNET_JSON_from_data_auto (&wdd_pos->h_contract),
-                                                     "transaction_id", (json_int_t) wdd_pos->transaction_id,
+                                          json_pack ("{s:o, s:o, s:o, s:o}",
+                                                     "h_contract_terms", GNUNET_JSON_from_data_auto (&wdd_pos->h_contract_terms),
                                                      "coin_pub", GNUNET_JSON_from_data_auto (&wdd_pos->coin_pub),
                                                      "deposit_value", TALER_JSON_from_amount (&wdd_pos->deposit_value),
                                                      "deposit_fee", TALER_JSON_from_amount (&wdd_pos->deposit_fee))));
@@ -1272,6 +1489,8 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
   wdp.purpose.size = htonl (sizeof (struct TALER_WireDepositDataPS));
   TALER_amount_hton (&wdp.total,
                      total);
+  TALER_amount_hton (&wdp.wire_fee,
+                     wire_fee);
   wdp.merchant_pub = *merchant_pub;
   wdp.h_wire = *h_wire;
   GNUNET_CRYPTO_hash_context_finish (hash_context,
@@ -1281,8 +1500,9 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
                &sig);
   return TEH_RESPONSE_reply_json_pack (connection,
                                        MHD_HTTP_OK,
-                                       "{s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
+                                       "{s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o}",
                                        "total", TALER_JSON_from_amount (total),
+                                       "wire_fee", TALER_JSON_from_amount (wire_fee),
                                        "merchant_pub", GNUNET_JSON_from_data_auto (merchant_pub),
                                        "H_wire", GNUNET_JSON_from_data_auto (h_wire),
                                        "execution_time", GNUNET_JSON_from_time_abs (exec_time),
@@ -1292,4 +1512,68 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
 }
 
 
+
+/**
+ * A wallet asked for /payback, but we do not know anything about the
+ * original withdraw operation specified. Generates a 404 reply.
+ *
+ * @param connection connection to the client
+ * @param ec Taler error code
+ * @return MHD result code
+ */
+int
+TEH_RESPONSE_reply_payback_unknown (struct MHD_Connection *connection,
+                                    enum TALER_ErrorCode ec)
+{
+  return TEH_RESPONSE_reply_json_pack (connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       "{s:s, s:I}",
+                                       "error", "blinded coin unknown",
+				       "code", (json_int_t) ec);
+}
+
+
+/**
+ * A wallet asked for /payback, return the successful response.
+ *
+ * @param connection connection to the client
+ * @param coin_pub coin for which we are processing the payback request
+ * @param reserve_pub public key of the reserve that will receive the payback
+ * @param amount the amount we will wire back
+ * @param timestamp when did the exchange receive the /payback request
+ * @return MHD result code
+ */
+int
+TEH_RESPONSE_reply_payback_success (struct MHD_Connection *connection,
+                                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                                    const struct TALER_ReservePublicKeyP *reserve_pub,
+                                    const struct TALER_Amount *amount,
+                                    struct GNUNET_TIME_Absolute timestamp)
+{
+  struct TALER_PaybackConfirmationPS pc;
+  struct TALER_ExchangePublicKeyP pub;
+  struct TALER_ExchangeSignatureP sig;
+
+  pc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK);
+  pc.purpose.size = htonl (sizeof (struct TALER_PaybackConfirmationPS));
+  pc.timestamp = GNUNET_TIME_absolute_hton (timestamp);
+  TALER_amount_hton (&pc.payback_amount,
+                     amount);
+  pc.coin_pub = *coin_pub;
+  pc.reserve_pub = *reserve_pub;
+  TEH_KS_sign (&pc.purpose,
+               &pub,
+               &sig);
+  return TEH_RESPONSE_reply_json_pack (connection,
+                                       MHD_HTTP_OK,
+                                       "{s:o, s:o, s:o, s:o, s:o}",
+                                       "reserve_pub", GNUNET_JSON_from_data_auto (reserve_pub),
+                                       "timestamp", GNUNET_JSON_from_time_abs (timestamp),
+                                       "amount", TALER_JSON_from_amount (amount),
+                                       "exchange_sig", GNUNET_JSON_from_data_auto (&sig),
+                                       "exchange_pub", GNUNET_JSON_from_data_auto (&pub));
+}
+
+
+
 /* end of taler-exchange-httpd_responses.c */
