@@ -33,6 +33,7 @@
 #include "taler-exchange-httpd.h"
 #include "taler-exchange-httpd_db.h"
 
+
 /**
  * Add headers we want to return in every response.
  * Useful for testing, like if we want to always close
@@ -44,6 +45,28 @@ void
 TEH_RESPONSE_add_global_headers (struct MHD_Response *response);
 
 
+/**
+ * Try to compress a response body.  Updates @a buf and @a buf_size.
+ *
+ * @param[in,out] buf pointer to body to compress
+ * @param[in,out] buf_size pointer to initial size of @a buf
+ * @return #MHD_YES if @a buf was compressed
+ */
+int
+TEH_RESPONSE_body_compress (void **buf,
+			    size_t *buf_size);
+
+
+/**
+ * Is HTTP body deflate compression supported by the client?
+ *
+ * @param connection connection to check
+ * @return #MHD_YES if 'deflate' compression is allowed
+ */
+int
+TEH_RESPONSE_can_compress (struct MHD_Connection *connection);
+
+
 /**
  * Send JSON object as response.
  *
@@ -231,8 +254,7 @@ TEH_RESPONSE_reply_invalid_json (struct MHD_Connection *connectionx);
  * @param connection connection to the client
  * @param coin_pub public key of the coin
  * @param h_wire hash of wire details
- * @param h_contract hash of contract details
- * @param transaction_id transaction ID
+ * @param h_contract_terms hash of proposal data
  * @param timestamp client's timestamp
  * @param refund_deadline until when this deposit be refunded
  * @param merchant merchant public key
@@ -243,8 +265,7 @@ int
 TEH_RESPONSE_reply_deposit_success (struct MHD_Connection *connection,
                                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                                     const struct GNUNET_HashCode *h_wire,
-                                    const struct GNUNET_HashCode *h_contract,
-                                    uint64_t transaction_id,
+                                    const struct GNUNET_HashCode *h_contract_terms,
                                     struct GNUNET_TIME_Absolute timestamp,
                                     struct GNUNET_TIME_Absolute refund_deadline,
                                     const struct TALER_MerchantPublicKeyP *merchant,
@@ -252,17 +273,20 @@ TEH_RESPONSE_reply_deposit_success (struct MHD_Connection *connection,
 
 
 /**
- * Send proof that a /deposit request is invalid to client.  This
- * function will create a message with all of the operations affecting
- * the coin that demonstrate that the coin has insufficient value.
+ * Send proof that a request is invalid to client because of
+ * insufficient funds.  This function will create a message with all
+ * of the operations affecting the coin that demonstrate that the coin
+ * has insufficient value.
  *
  * @param connection connection to the client
+ * @param ec error code to return
  * @param tl transaction list to use to build reply
  * @return MHD result code
  */
 int
-TEH_RESPONSE_reply_deposit_insufficient_funds (struct MHD_Connection *connection,
-                                               const struct TALER_EXCHANGEDB_TransactionList *tl);
+TEH_RESPONSE_reply_coin_insufficient_funds (struct MHD_Connection *connection,
+                                            enum TALER_ErrorCode ec,
+                                            const struct TALER_EXCHANGEDB_TransactionList *tl);
 
 
 /**
@@ -337,22 +361,20 @@ TEH_RESPONSE_reply_transfer_pending (struct MHD_Connection *connection,
  * them. Generates the 200 reply.
  *
  * @param connection connection to the client
- * @param h_contract hash of the contract
+ * @param h_contract_terms hash of the proposal data
  * @param h_wire hash of wire account details
  * @param coin_pub public key of the coin
  * @param coin_contribution contribution of this coin to the total amount transferred
- * @param transaction_id merchant transaction identifier
  * @param wtid raw wire transfer identifier
  * @param exec_time execution time of the wire transfer
  * @return MHD result code
  */
 int
 TEH_RESPONSE_reply_track_transaction (struct MHD_Connection *connection,
-                                      const struct GNUNET_HashCode *h_contract,
+                                      const struct GNUNET_HashCode *h_contract_terms,
                                       const struct GNUNET_HashCode *h_wire,
                                       const struct TALER_CoinSpendPublicKeyP *coin_pub,
                                       const struct TALER_Amount *coin_contribution,
-                                      uint64_t transaction_id,
                                       const struct TALER_WireTransferIdentifierRawP *wtid,
                                       struct GNUNET_TIME_Absolute exec_time);
 
@@ -374,14 +396,9 @@ struct TEH_TrackTransferDetail
   struct TEH_TrackTransferDetail *prev;
 
   /**
-   * Hash of the contract
+   * Hash of the proposal data.
    */
-  struct GNUNET_HashCode h_contract;
-
-  /**
-   * Merchant's transaction ID.
-   */
-  uint64_t transaction_id;
+  struct GNUNET_HashCode h_contract_terms;
 
   /**
    * Coin's public key.
@@ -408,6 +425,7 @@ struct TEH_TrackTransferDetail
  * @param total total amount that was transferred
  * @param merchant_pub public key of the merchant
  * @param h_wire destination account
+ * @param wire_fee wire fee that was charged
  * @param exec_time execution time of the wire transfer
  * @param wdd_head linked list with details about the combined deposits
  * @return MHD result code
@@ -417,6 +435,7 @@ TEH_RESPONSE_reply_track_transfer_details (struct MHD_Connection *connection,
                                            const struct TALER_Amount *total,
                                            const struct TALER_MerchantPublicKeyP *merchant_pub,
                                            const struct GNUNET_HashCode *h_wire,
+                                           const struct TALER_Amount *wire_fee,
                                            struct GNUNET_TIME_Absolute exec_time,
                                            const struct TEH_TrackTransferDetail *wdd_head);
 
@@ -563,4 +582,35 @@ TEH_RESPONSE_reply_refresh_link_success (struct MHD_Connection *connection,
                                          const struct TEH_RESPONSE_LinkSessionInfo *sessions);
 
 
+/**
+ * A wallet asked for /payback, but we do not know anything about the
+ * original withdraw operation specified. Generates a 404 reply.
+ *
+ * @param connection connection to the client
+ * @param ec Taler error code
+ * @return MHD result code
+ */
+int
+TEH_RESPONSE_reply_payback_unknown (struct MHD_Connection *connection,
+                                    enum TALER_ErrorCode ec);
+
+
+/**
+ * A wallet asked for /payback, return the successful response.
+ *
+ * @param connection connection to the client
+ * @param coin_pub coin for which we are processing the payback request
+ * @param reserve_pub public key of the reserve that will receive the payback
+ * @param amount the amount we will wire back
+ * @param timestamp when did the exchange receive the /payback request
+ * @return MHD result code
+ */
+int
+TEH_RESPONSE_reply_payback_success (struct MHD_Connection *connection,
+                                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                                    const struct TALER_ReservePublicKeyP *reserve_pub,
+                                    const struct TALER_Amount *amount,
+                                    struct GNUNET_TIME_Absolute timestamp);
+
+
 #endif
