@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2016 Inria and GNUnet e.V.
+  (C) 2016, 2017 Inria and GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -110,6 +110,11 @@ struct Command
        */
       uint64_t credit_account;
 
+      /**
+       * Base URL of the exchange.
+       */
+      const char *exchange_base_url;
+
       /**
        * Subject of the transfer, set by the command.
        */
@@ -134,11 +139,6 @@ struct Command
        */
       uint64_t merchant_account;
 
-      /**
-       * Merchant's transaction ID.
-       */
-      uint64_t transaction_id;
-
       /**
        * By when does the merchant request the funds to be wired.
        */
@@ -419,7 +419,7 @@ do_deposit (struct Command *cmd)
   /* contract is just picked at random;
      note: we may want to write this back to 'cmd' in the future. */
   GNUNET_CRYPTO_hash_create_random (GNUNET_CRYPTO_QUALITY_WEAK,
-                                    &deposit.h_contract);
+                                    &deposit.h_contract_terms);
   if ( (GNUNET_OK !=
         TALER_string_to_amount (cmd->details.deposit.amount_with_fee,
                                 &deposit.amount_with_fee)) ||
@@ -439,7 +439,6 @@ do_deposit (struct Command *cmd)
   GNUNET_assert (GNUNET_OK ==
                  TALER_JSON_hash (deposit.receiver_wire_account,
                                   &deposit.h_wire));
-  deposit.transaction_id = cmd->details.deposit.transaction_id;
   deposit.timestamp = GNUNET_TIME_absolute_get ();
   deposit.wire_deadline = GNUNET_TIME_relative_to_absolute (cmd->details.deposit.wire_deadline);
 
@@ -518,7 +517,7 @@ interpreter (void *cls)
                                    NULL, NULL, NULL,
                                    "taler-exchange-aggregator",
                                    "taler-exchange-aggregator",
-                                   "-c", "test_taler_exchange_httpd.conf",
+                                   "-c", config_filename,
                                    "-t", /* enable temporary tables */
                                    NULL);
       if (NULL == aggregator_proc)
@@ -562,10 +561,11 @@ interpreter (void *cls)
         }
         if (GNUNET_OK !=
             TALER_FAKEBANK_check (fb,
-                            &want_amount,
-                            cmd->details.expect_transaction.debit_account,
-                            cmd->details.expect_transaction.credit_account,
-                            &cmd->details.expect_transaction.wtid))
+                                  &want_amount,
+                                  cmd->details.expect_transaction.debit_account,
+                                  cmd->details.expect_transaction.credit_account,
+                                  cmd->details.expect_transaction.exchange_base_url,
+                                  &cmd->details.expect_transaction.wtid))
         {
           fail (cmd);
           return;
@@ -606,7 +606,6 @@ run_test ()
       .label = "do-deposit-1",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 1,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:1",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -621,7 +620,8 @@ run_test ()
       .label = "expect-deposit-1",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.9"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.89"
     },
 
     {
@@ -645,7 +645,6 @@ run_test ()
       .label = "do-deposit-2a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 2,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:1",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -655,7 +654,6 @@ run_test ()
       .label = "do-deposit-2b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 3,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:1",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -669,7 +667,8 @@ run_test ()
       .label = "expect-deposit-2",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:1.8"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:1.79"
     },
 
     {
@@ -683,7 +682,6 @@ run_test ()
       .label = "do-deposit-3a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 4,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:1",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -693,7 +691,6 @@ run_test ()
       .label = "do-deposit-3b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 5,
-      .details.deposit.transaction_id = 5,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:1",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -703,7 +700,6 @@ run_test ()
       .label = "do-deposit-3c",
       .details.deposit.merchant_name = "alice",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 1,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:1",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -717,21 +713,24 @@ run_test ()
       .label = "expect-deposit-3a",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.9"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.89"
     },
     {
       .opcode = OPCODE_EXPECT_TRANSACTION,
       .label = "expect-deposit-3b",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.9"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.89"
     },
     {
       .opcode = OPCODE_EXPECT_TRANSACTION,
       .label = "expect-deposit-3c",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 5,
-      .details.expect_transaction.amount = "EUR:0.9"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.89"
     },
     {
       .opcode = OPCODE_EXPECT_TRANSACTIONS_EMPTY,
@@ -744,7 +743,6 @@ run_test ()
       .label = "do-deposit-4a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 6,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.2",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -754,7 +752,6 @@ run_test ()
       .label = "do-deposit-4b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 7,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.2",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -781,7 +778,8 @@ run_test ()
       .label = "expect-deposit-4",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.2"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.19"
     },
 
     /* test picking all deposits at earliest deadline */
@@ -790,7 +788,6 @@ run_test ()
       .label = "do-deposit-5a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 8,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 10 }, /* 10s */
       .details.deposit.amount_with_fee = "EUR:0.2",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -800,7 +797,6 @@ run_test ()
       .label = "do-deposit-5b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 9,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.2",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -827,7 +823,8 @@ run_test ()
       .label = "expect-deposit-5",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.2"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.19"
     },
 
     /* Test NEVER running 'tiny' unless they make up minimum unit */
@@ -836,7 +833,6 @@ run_test ()
       .label = "do-deposit-6a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 10,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.102",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -854,7 +850,6 @@ run_test ()
       .label = "do-deposit-6b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 11,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.102",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -864,7 +859,6 @@ run_test ()
       .label = "do-deposit-6c",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 12,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.102",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -882,7 +876,6 @@ run_test ()
       .label = "do-deposit-6d",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 13,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.102",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -900,9 +893,8 @@ run_test ()
       .label = "do-deposit-6e",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 14,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
-      .details.deposit.amount_with_fee = "EUR:0.102",
+      .details.deposit.amount_with_fee = "EUR:0.112",
       .details.deposit.deposit_fee = "EUR:0.1"
     },
     {
@@ -914,6 +906,7 @@ run_test ()
       .label = "expect-deposit-6",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
       .details.expect_transaction.amount = "EUR:0.01"
     },
 
@@ -923,7 +916,6 @@ run_test ()
       .label = "do-deposit-7a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 15,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.109",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -941,9 +933,8 @@ run_test ()
       .label = "do-deposit-7b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 16,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
-      .details.deposit.amount_with_fee = "EUR:0.109",
+      .details.deposit.amount_with_fee = "EUR:0.119",
       .details.deposit.deposit_fee = "EUR:0.1"
     },
     {
@@ -955,6 +946,7 @@ run_test ()
       .label = "expect-deposit-7",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
       .details.expect_transaction.amount = "EUR:0.01"
     },
     /* Now check profit was actually taken */
@@ -963,7 +955,6 @@ run_test ()
       .label = "do-deposit-7c",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 17,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.122",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -977,7 +968,8 @@ run_test ()
       .label = "expect-deposit-7",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.02"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.01"
     },
 
     /* Test that aggregation would happen fully if wire deadline is long */
@@ -986,7 +978,6 @@ run_test ()
       .label = "do-deposit-8a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 18,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.109",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -1004,7 +995,6 @@ run_test ()
       .label = "do-deposit-8b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 19,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.109",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -1023,7 +1013,6 @@ run_test ()
       .label = "do-deposit-8c",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 20,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.122",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -1037,7 +1026,8 @@ run_test ()
       .label = "expect-deposit-8",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.04"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.03"
     },
 
 
@@ -1047,7 +1037,6 @@ run_test ()
       .label = "do-deposit-9a",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 21,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.104",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -1065,7 +1054,6 @@ run_test ()
       .label = "do-deposit-9b",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 22,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 5 }, /* 5s */
       .details.deposit.amount_with_fee = "EUR:0.105",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -1084,7 +1072,6 @@ run_test ()
       .label = "do-deposit-9c",
       .details.deposit.merchant_name = "bob",
       .details.deposit.merchant_account = 4,
-      .details.deposit.transaction_id = 23,
       .details.deposit.wire_deadline = { 1000LL * 1000 * 0 }, /* 0s */
       .details.deposit.amount_with_fee = "EUR:0.112",
       .details.deposit.deposit_fee = "EUR:0.1"
@@ -1099,7 +1086,8 @@ run_test ()
       .label = "expect-deposit-9",
       .details.expect_transaction.debit_account = 3,
       .details.expect_transaction.credit_account = 4,
-      .details.expect_transaction.amount = "EUR:0.02"
+      .details.expect_transaction.exchange_base_url = "https://exchange.taler.net/",
+      .details.expect_transaction.amount = "EUR:0.01"
     },
 
     /* Everything tested, terminate with success */
@@ -1150,6 +1138,8 @@ run (void *cls)
   GNUNET_assert (NULL != session);
   fake_issue (&issue);
   dpk.rsa_public_key = coin_pub;
+  GNUNET_CRYPTO_rsa_public_key_hash (dpk.rsa_public_key,
+				     &issue.properties.denom_hash);
   if ( (GNUNET_OK !=
         plugin->start (plugin->cls,
                        session)) ||
@@ -1172,7 +1162,8 @@ run (void *cls)
     GNUNET_SCHEDULER_add_read_file (GNUNET_TIME_UNIT_FOREVER_REL,
 				    GNUNET_DISK_pipe_handle (sigpipe,
 							     GNUNET_DISK_PIPE_END_READ),
-				    &maint_child_death, NULL);
+				    &maint_child_death,
+                                    NULL);
   GNUNET_SCHEDULER_add_shutdown (&shutdown_action,
                                  NULL);
   timeout_task = GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_MINUTES,
@@ -1214,6 +1205,7 @@ main (int argc,
 {
   const char *plugin_name;
   char *testname;
+  struct GNUNET_OS_Process *proc;
   struct GNUNET_CONFIGURATION_Handle *cfg;
   struct GNUNET_SIGNAL_Context *shc_chld;
 
@@ -1225,15 +1217,41 @@ main (int argc,
   }
   plugin_name++;
   (void) GNUNET_asprintf (&testname,
-                          "test-taler-exchange-aggregator-%s", plugin_name);
+                          "test-taler-exchange-aggregator-%s",
+                          plugin_name);
   (void) GNUNET_asprintf (&config_filename,
-                          "%s.conf", testname);
+                          "%s.conf",
+                          testname);
   /* these might get in the way */
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
   GNUNET_log_setup ("test_taler_exchange_aggregator",
                     "WARNING",
                     NULL);
+  proc = GNUNET_OS_start_process (GNUNET_NO,
+                                  GNUNET_OS_INHERIT_STD_ALL,
+                                  NULL, NULL, NULL,
+                                  "taler-exchange-keyup",
+                                  "taler-exchange-keyup",
+                                  "-c", config_filename,
+                                  NULL);
+  if (NULL == proc)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+		"Failed to run `taler-exchange-keyup`, is your PATH correct?\n");
+    return 77;
+  }
+  GNUNET_OS_process_wait (proc);
+  GNUNET_OS_process_destroy (proc);
+  if (GNUNET_OK !=
+      GNUNET_NETWORK_test_port_free (IPPROTO_TCP,
+				     8082))
+  {
+    fprintf (stderr,
+             "Required port %u not available, skipping.\n",
+	     8082);
+    return 77;
+  }
   cfg = GNUNET_CONFIGURATION_create ();
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_parse (cfg,
@@ -1244,13 +1262,16 @@ main (int argc,
     GNUNET_free (testname);
     return 2;
   }
-  sigpipe = GNUNET_DISK_pipe (GNUNET_NO, GNUNET_NO, GNUNET_NO, GNUNET_NO);
+  sigpipe = GNUNET_DISK_pipe (GNUNET_NO, GNUNET_NO,
+                              GNUNET_NO, GNUNET_NO);
   GNUNET_assert (NULL != sigpipe);
   shc_chld =
-    GNUNET_SIGNAL_handler_install (GNUNET_SIGCHLD, &sighandler_child_death);
+    GNUNET_SIGNAL_handler_install (GNUNET_SIGCHLD,
+                                   &sighandler_child_death);
   coin_pk = GNUNET_CRYPTO_rsa_private_key_create (1024);
   coin_pub = GNUNET_CRYPTO_rsa_private_key_get_public (coin_pk);
-  GNUNET_SCHEDULER_run (&run, cfg);
+  GNUNET_SCHEDULER_run (&run,
+                        cfg);
   GNUNET_CRYPTO_rsa_private_key_free (coin_pk);
   GNUNET_CRYPTO_rsa_public_key_free (coin_pub);
   GNUNET_SIGNAL_handler_uninstall (shc_chld);
