@@ -32,7 +32,9 @@ common_free_reserve_history (void *cls,
 {
   struct TALER_EXCHANGEDB_BankTransfer *bt;
   struct TALER_EXCHANGEDB_CollectableBlindcoin *cbc;
+  struct TALER_EXCHANGEDB_Payback *payback;
   struct TALER_EXCHANGEDB_ReserveHistory *backref;
+  struct TALER_EXCHANGEDB_ClosingTransfer *closing;
 
   while (NULL != rh)
   {
@@ -42,8 +44,7 @@ common_free_reserve_history (void *cls,
       bt = rh->details.bank;
       if (NULL != bt->sender_account_details)
         json_decref (bt->sender_account_details);
-      if (NULL != bt->transfer_details)
-        json_decref (bt->transfer_details);
+      GNUNET_free_non_null (bt->wire_reference);
       GNUNET_free (bt);
       break;
     case TALER_EXCHANGEDB_RO_WITHDRAW_COIN:
@@ -52,6 +53,18 @@ common_free_reserve_history (void *cls,
       GNUNET_CRYPTO_rsa_public_key_free (cbc->denom_pub.rsa_public_key);
       GNUNET_free (cbc);
       break;
+    case TALER_EXCHANGEDB_RO_PAYBACK_COIN:
+      payback = rh->details.payback;
+      GNUNET_CRYPTO_rsa_signature_free (payback->coin.denom_sig.rsa_signature);
+      GNUNET_CRYPTO_rsa_public_key_free (payback->coin.denom_pub.rsa_public_key);
+      GNUNET_free (payback);
+      break;
+    case TALER_EXCHANGEDB_RO_EXCHANGE_TO_BANK:
+      closing = rh->details.closing;
+      if (NULL != closing->receiver_account_details)
+        json_decref (closing->receiver_account_details);
+      GNUNET_free (closing);
+      break;
     }
     backref = rh;
     rh = rh->next;
@@ -125,6 +138,13 @@ common_free_coin_transaction_list (void *cls,
         GNUNET_CRYPTO_rsa_signature_free (list->details.refund->coin.denom_sig.rsa_signature);
       GNUNET_free (list->details.refund);
       break;
+    case TALER_EXCHANGEDB_TT_PAYBACK:
+      if (NULL != list->details.payback->coin.denom_pub.rsa_public_key)
+        GNUNET_CRYPTO_rsa_public_key_free (list->details.payback->coin.denom_pub.rsa_public_key);
+      if (NULL != list->details.payback->coin.denom_sig.rsa_signature)
+        GNUNET_CRYPTO_rsa_signature_free (list->details.payback->coin.denom_sig.rsa_signature);
+      GNUNET_free (list->details.payback);
+      break;
     }
     GNUNET_free (list);
     list = next;
