@@ -151,6 +151,16 @@ TALER_amount_get_zero (const char *cur,
                        struct TALER_Amount *denom);
 
 
+/**
+ * Test if the given amount is valid.
+ *
+ * @param amount amount to check
+ * @return #GNUNET_OK if @a amount is valid
+ */
+int
+TALER_amount_is_valid (const struct TALER_Amount *amount);
+
+
 /**
  * Convert amount from host to network representation.
  *
@@ -251,6 +261,20 @@ TALER_amount_add (struct TALER_Amount *sum,
                   const struct TALER_Amount *a2);
 
 
+/**
+ * Divide an amount by a float.  Note that this function
+ * may introduce a rounding error!
+ *
+ * @param result where to store @a dividend / @a divisor
+ * @param dividend amount to divide
+ * @param divisor by what to divide, must be positive
+ */
+void
+TALER_amount_divide (struct TALER_Amount *result,
+                     const struct TALER_Amount *dividend,
+                     uint32_t divisor);
+
+
 /**
  * Normalize the given amount.
  *
@@ -273,6 +297,18 @@ TALER_amount_normalize (struct TALER_Amount *amount);
 char *
 TALER_amount_to_string (const struct TALER_Amount *amount);
 
+
+/**
+ * Convert amount to string.
+ *
+ * @param amount amount to convert to string
+ * @return statically allocated buffer with string representation,
+ *         NULL if the @a amount was invalid
+ */
+const char *
+TALER_amount2s (const struct TALER_Amount *amount);
+
+
 #if 0                           /* keep Emacsens' auto-indent happy */
 {
 #endif
