@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2016 GNUnet e.V.
+  Copyright (C) 2014-2017 Inria and GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -53,10 +53,6 @@ typedef int
  * @param revenue_balance what was the total profit made from
  *                        deposit fees, melting fees, refresh fees
  *                        and coins that were never returned?
- * @param deposit_fee_balance total profits from deposit fees
- * @param melt_fee_balance total profits from melting fees
- * @param refund_fee_balance total profits from refund fees
- *
  * @return sets the return value of select_denomination_info(),
  *         #GNUNET_OK to continue,
  *         #GNUNET_NO to stop processing further rows
@@ -66,10 +62,7 @@ typedef int
 (*TALER_AUDITORDB_HistoricDenominationRevenueDataCallback)(void *cls,
                                                            const struct GNUNET_HashCode *denom_pub_hash,
                                                            struct GNUNET_TIME_Absolute revenue_timestamp,
-                                                           const struct TALER_Amount *revenue_balance,
-                                                           const struct TALER_Amount *deposit_fee_balance,
-                                                           const struct TALER_Amount *melt_fee_balance,
-                                                           const struct TALER_Amount *refund_fee_balance);
+                                                           const struct TALER_Amount *revenue_balance);
 
 
 /**
@@ -112,6 +105,62 @@ typedef int
                                                       const struct TALER_Amount *reserve_profits);
 
 
+/**
+ * Structure for remembering the auditor's progress over the
+ * various tables and (auditor) transactions.
+ */
+struct TALER_AUDITORDB_ProgressPoint
+{
+  /**
+   * last_reserve_in_serial_id serial ID of the last reserve_in transfer the auditor processed
+   */
+  uint64_t last_reserve_in_serial_id;
+
+  /**
+   * last_reserve_out_serial_id serial ID of the last reserve_out the auditor processed
+   */
+  uint64_t last_reserve_out_serial_id;
+
+  /**
+   * last_payback_serial_id serial ID of the last payback entry the auditor processed when
+   * considering reserves.
+   */
+  uint64_t last_reserve_payback_serial_id;
+
+  /**
+   * last_reserve_close_serial_id serial ID of the last reserve_close
+   * entry the auditor processed.
+   */
+  uint64_t last_reserve_close_serial_id;
+
+  /**
+   * last_reserve_out_serial_id serial ID of the last withdraw the auditor processed
+   */
+  uint64_t last_withdraw_serial_id;
+
+  /**
+   * last_deposit_serial_id serial ID of the last deposit the auditor processed
+   */
+  uint64_t last_deposit_serial_id;
+
+  /**
+   * last_melt_serial_id serial ID of the last refresh the auditor processed
+   */
+  uint64_t last_melt_serial_id;
+
+  /**
+   * last_prewire_serial_id serial ID of the last prewire transfer the auditor processed
+   */
+  uint64_t last_refund_serial_id;
+
+  /**
+   * last_prewire_serial_id serial ID of the last prewire transfer the auditor processed
+   */
+  uint64_t last_wire_out_serial_id;
+
+};
+
+
 /**
  * Handle for one session with the database.
  */
@@ -259,23 +308,14 @@ struct TALER_AUDITORDB_Plugin
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session connection to use
    * @param master_pub master key of the exchange
-   * @param last_reserve_in_serial_id serial ID of the last reserve_in transfer the auditor processed
-   * @param last_reserve_out_serial_id serial ID of the last withdraw the auditor processed
-   * @param last_deposit_serial_id serial ID of the last deposit the auditor processed
-   * @param last_melt_serial_id serial ID of the last refresh the auditor processed
-   * @param last_prewire_serial_id serial ID of the last prewire transfer the auditor processed
+   * @param pp where is the auditor in processing
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
   (*insert_auditor_progress)(void *cls,
                              struct TALER_AUDITORDB_Session *session,
                              const struct TALER_MasterPublicKeyP *master_pub,
-                             uint64_t last_reserve_in_serial_id,
-                             uint64_t last_reserve_out_serial_id,
-                             uint64_t last_deposit_serial_id,
-                             uint64_t last_melt_serial_id,
-                             uint64_t last_refund_serial_id,
-                             uint64_t last_prewire_serial_id);
+                             const struct TALER_AUDITORDB_ProgressPoint *pp);
 
 
   /**
@@ -285,23 +325,14 @@ struct TALER_AUDITORDB_Plugin
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session connection to use
    * @param master_pub master key of the exchange
-   * @param last_reserve_in_serial_id serial ID of the last reserve_in transfer the auditor processed
-   * @param last_reserve_out_serial_id serial ID of the last withdraw the auditor processed
-   * @param last_deposit_serial_id serial ID of the last deposit the auditor processed
-   * @param last_melt_serial_id serial ID of the last refresh the auditor processed
-   * @param last_prewire_serial_id serial ID of the last prewire transfer the auditor processed
+   * @param pp where is the auditor in processing
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
   (*update_auditor_progress)(void *cls,
                              struct TALER_AUDITORDB_Session *session,
                              const struct TALER_MasterPublicKeyP *master_pub,
-                             uint64_t last_reserve_in_serial_id,
-                             uint64_t last_reserve_out_serial_id,
-                             uint64_t last_deposit_serial_id,
-                             uint64_t last_melt_serial_id,
-                             uint64_t last_refund_serial_id,
-                             uint64_t last_prewire_serial_id);
+                             const struct TALER_AUDITORDB_ProgressPoint *pp);
 
 
   /**
@@ -310,11 +341,7 @@ struct TALER_AUDITORDB_Plugin
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session connection to use
    * @param master_pub master key of the exchange
-   * @param[out] last_reserve_in_serial_id serial ID of the last reserve_in transfer the auditor processed
-   * @param[out] last_reserve_out_serial_id serial ID of the last withdraw the auditor processed
-   * @param[out] last_deposit_serial_id serial ID of the last deposit the auditor processed
-   * @param[out] last_melt_serial_id serial ID of the last refresh the auditor processed
-   * @param[out] last_prewire_serial_id serial ID of the last prewire transfer the auditor processed
+   * @param[out] pp set to where the auditor is in processing
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure;
    *         #GNUNET_NO if we have no records for the @a master_pub
    */
@@ -322,12 +349,7 @@ struct TALER_AUDITORDB_Plugin
   (*get_auditor_progress)(void *cls,
                           struct TALER_AUDITORDB_Session *session,
                           const struct TALER_MasterPublicKeyP *master_pub,
-                          uint64_t *last_reserve_in_serial_id,
-                          uint64_t *last_reserve_out_serial_id,
-                          uint64_t *last_deposit_serial_id,
-                          uint64_t *last_melt_serial_id,
-                          uint64_t *last_refund_serial_id,
-                          uint64_t *last_prewire_serial_id);
+                          struct TALER_AUDITORDB_ProgressPoint *pp);
 
 
   /**
@@ -342,10 +364,6 @@ struct TALER_AUDITORDB_Plugin
    * @param withdraw_fee_balance amount the exchange gained in withdraw fees
    *                             due to withdrawals from this reserve
    * @param expiration_date expiration date of the reserve
-   * @param last_reserve_in_serial_id up to which point did we consider
-   *                 incoming transfers for the above information
-   * @param last_reserve_out_serial_id up to which point did we consider
-   *                 withdrawals for the above information
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
@@ -355,9 +373,7 @@ struct TALER_AUDITORDB_Plugin
                          const struct TALER_MasterPublicKeyP *master_pub,
                          const struct TALER_Amount *reserve_balance,
                          const struct TALER_Amount *withdraw_fee_balance,
-                         struct GNUNET_TIME_Absolute expiration_date,
-                         uint64_t last_reserve_in_serial_id,
-                         uint64_t last_reserve_out_serial_id);
+                         struct GNUNET_TIME_Absolute expiration_date);
 
 
   /**
@@ -372,10 +388,6 @@ struct TALER_AUDITORDB_Plugin
    * @param withdraw_fee_balance amount the exchange gained in withdraw fees
    *                             due to withdrawals from this reserve
    * @param expiration_date expiration date of the reserve
-   * @param last_reserve_in_serial_id up to which point did we consider
-   *                 incoming transfers for the above information
-   * @param last_reserve_out_serial_id up to which point did we consider
-   *                 withdrawals for the above information
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
@@ -385,9 +397,7 @@ struct TALER_AUDITORDB_Plugin
                          const struct TALER_MasterPublicKeyP *master_pub,
                          const struct TALER_Amount *reserve_balance,
                          const struct TALER_Amount *withdraw_fee_balance,
-                         struct GNUNET_TIME_Absolute expiration_date,
-                         uint64_t last_reserve_in_serial_id,
-                         uint64_t last_reserve_out_serial_id);
+                         struct GNUNET_TIME_Absolute expiration_date);
 
 
   /**
@@ -397,14 +407,11 @@ struct TALER_AUDITORDB_Plugin
    * @param session connection to use
    * @param reserve_pub public key of the reserve
    * @param master_pub master public key of the exchange
+   * @param[out] rowid which row did we get the information from
    * @param[out] reserve_balance amount stored in the reserve
    * @param[out] withdraw_fee_balance amount the exchange gained in withdraw fees
    *                             due to withdrawals from this reserve
    * @param[out] expiration_date expiration date of the reserve
-   * @param[out] last_reserve_in_serial_id up to which point did we consider
-   *                 incoming transfers for the above information
-   * @param[out] last_reserve_out_serial_id up to which point did we consider
-   *                 withdrawals for the above information
    * @return #GNUNET_OK on success; #GNUNET_NO if there is no known
    *         record about this reserve; #GNUNET_SYSERR on failure
    */
@@ -413,13 +420,28 @@ struct TALER_AUDITORDB_Plugin
                       struct TALER_AUDITORDB_Session *session,
                       const struct TALER_ReservePublicKeyP *reserve_pub,
                       const struct TALER_MasterPublicKeyP *master_pub,
+                      uint64_t *rowid,
                       struct TALER_Amount *reserve_balance,
                       struct TALER_Amount *withdraw_fee_balance,
-                      struct GNUNET_TIME_Absolute *expiration_date,
-                      uint64_t *last_reserve_in_serial_id,
-                      uint64_t *last_reserve_out_serial_id);
+                      struct GNUNET_TIME_Absolute *expiration_date);
 
 
+  /**
+   * Delete information about a reserve.
+   *
+   * @param cls the @e cls of this struct with the plugin-specific state
+   * @param session connection to use
+   * @param reserve_pub public key of the reserve
+   * @param master_pub master public key of the exchange
+   * @return #GNUNET_OK on success; #GNUNET_NO if there is no known
+   *         record about this reserve; #GNUNET_SYSERR on failure
+   */
+  int
+  (*del_reserve_info)(void *cls,
+                      struct TALER_AUDITORDB_Session *session,
+                      const struct TALER_ReservePublicKeyP *reserve_pub,
+                      const struct TALER_MasterPublicKeyP *master_pub);
+
 
   /**
    * Insert information about all reserves.  There must not be an
@@ -481,6 +503,57 @@ struct TALER_AUDITORDB_Plugin
                          struct TALER_Amount *withdraw_fee_balance);
 
 
+  /**
+   * Insert information about exchange's wire fee balance. There must not be an
+   * existing record for the same @a master_pub.
+   *
+   * @param cls the @e cls of this struct with the plugin-specific state
+   * @param session connection to use
+   * @param master_pub master public key of the exchange
+   * @param wire_fee_balance amount the exchange gained in wire fees
+   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
+   */
+  int
+  (*insert_wire_fee_summary)(void *cls,
+                             struct TALER_AUDITORDB_Session *session,
+                             const struct TALER_MasterPublicKeyP *master_pub,
+                             const struct TALER_Amount *wire_fee_balance);
+
+
+  /**
+   * Insert information about exchange's wire fee balance.  Destructively updates an
+   * existing record, which must already exist.
+   *
+   * @param cls the @e cls of this struct with the plugin-specific state
+   * @param session connection to use
+   * @param master_pub master public key of the exchange
+   * @param wire_fee_balance amount the exchange gained in wire fees
+   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
+   */
+  int
+  (*update_wire_fee_summary)(void *cls,
+                             struct TALER_AUDITORDB_Session *session,
+                             const struct TALER_MasterPublicKeyP *master_pub,
+                             const struct TALER_Amount *wire_fee_balance);
+
+
+  /**
+   * Get summary information about an exchanges wire fee balance.
+   *
+   * @param cls the @e cls of this struct with the plugin-specific state
+   * @param session connection to use
+   * @param master_pub master public key of the exchange
+   * @param[out] wire_fee_balance set amount the exchange gained in wire fees
+   * @return #GNUNET_OK on success; #GNUNET_NO if there is no known
+   *         record about this exchange; #GNUNET_SYSERR on failure
+   */
+  int
+  (*get_wire_fee_summary)(void *cls,
+                          struct TALER_AUDITORDB_Session *session,
+                          const struct TALER_MasterPublicKeyP *master_pub,
+                          struct TALER_Amount *wire_fee_balance);
+
+
   /**
    * Insert information about a denomination key's balances.  There
    * must not be an existing record for the denomination key.
@@ -489,17 +562,7 @@ struct TALER_AUDITORDB_Plugin
    * @param session connection to use
    * @param denom_pub_hash hash of the denomination public key
    * @param denom_balance value of coins outstanding with this denomination key
-   * @param deposit_fee_balance total deposit fees collected for this DK
-   * @param melt_fee_balance total melt fees collected for this DK
-   * @param refund_fee_balance total refund fees collected for this DK
-   * @param last_reserve_out_serial_id up to which point did we consider
-   *                 withdrawals for the above information
-   * @param last_deposit_serial_id up to which point did we consider
-   *                 deposits for the above information
-   * @param last_melt_serial_id up to which point did we consider
-   *                 melts for the above information
-   * @param last_refund_serial_id up to which point did we consider
-   *                 refunds for the above information
+   * @param denom_risk value of coins issued with this denomination key
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
@@ -507,13 +570,7 @@ struct TALER_AUDITORDB_Plugin
                                  struct TALER_AUDITORDB_Session *session,
                                  const struct GNUNET_HashCode *denom_pub_hash,
                                  const struct TALER_Amount *denom_balance,
-                                 const struct TALER_Amount *deposit_fee_balance,
-                                 const struct TALER_Amount *melt_fee_balance,
-                                 const struct TALER_Amount *refund_fee_balance,
-                                 uint64_t last_reserve_out_serial_id,
-                                 uint64_t last_deposit_serial_id,
-                                 uint64_t last_melt_serial_id,
-                                 uint64_t last_refund_serial_id);
+                                 const struct TALER_Amount *denom_risk);
 
 
   /**
@@ -524,17 +581,7 @@ struct TALER_AUDITORDB_Plugin
    * @param session connection to use
    * @param denom_pub_hash hash of the denomination public key
    * @param denom_balance value of coins outstanding with this denomination key
-   * @param deposit_fee_balance total deposit fees collected for this DK
-   * @param melt_fee_balance total melt fees collected for this DK
-   * @param refund_fee_balance total refund fees collected for this DK
-   * @param last_reserve_out_serial_id up to which point did we consider
-   *                 withdrawals for the above information
-   * @param last_deposit_serial_id up to which point did we consider
-   *                 deposits for the above information
-   * @param last_melt_serial_id up to which point did we consider
-   *                 melts for the above information
-   * @param last_refund_serial_id up to which point did we consider
-   *                 refunds for the above information
+   * @param denom_risk value of coins issued with this denomination key
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
@@ -542,13 +589,7 @@ struct TALER_AUDITORDB_Plugin
                                  struct TALER_AUDITORDB_Session *session,
                                  const struct GNUNET_HashCode *denom_pub_hash,
                                  const struct TALER_Amount *denom_balance,
-                                 const struct TALER_Amount *deposit_fee_balance,
-                                 const struct TALER_Amount *melt_fee_balance,
-                                 const struct TALER_Amount *refund_fee_balance,
-                                 uint64_t last_reserve_out_serial_id,
-                                 uint64_t last_deposit_serial_id,
-                                 uint64_t last_melt_serial_id,
-                                 uint64_t last_refund_serial_id);
+                                 const struct TALER_Amount *denom_risk);
 
 
   /**
@@ -558,31 +599,29 @@ struct TALER_AUDITORDB_Plugin
    * @param session connection to use
    * @param denom_pub_hash hash of the denomination public key
    * @param[out] denom_balance value of coins outstanding with this denomination key
-   * @param[out] deposit_fee_balance total deposit fees collected for this DK
-   * @param[out] melt_fee_balance total melt fees collected for this DK
-   * @param[out] refund_fee_balance total refund fees collected for this DK
-   * @param[out] last_reserve_out_serial_id up to which point did we consider
-   *                 withdrawals for the above information
-   * @param[out] last_deposit_serial_id up to which point did we consider
-   *                 deposits for the above information
-   * @param[out] last_melt_serial_id up to which point did we consider
-   *                 melts for the above information
-   * @param[out] last_refund_serial_id up to which point did we consider
-   *                 refunds for the above information
-   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
+   * @param[out] denom_risk value of coins issued with this denomination key
+   * @return #GNUNET_OK on success; #GNUNET_NO if no record found, #GNUNET_SYSERR on failure
    */
   int
   (*get_denomination_balance)(void *cls,
                               struct TALER_AUDITORDB_Session *session,
                               const struct GNUNET_HashCode *denom_pub_hash,
                               struct TALER_Amount *denom_balance,
-                              struct TALER_Amount *deposit_fee_balance,
-                              struct TALER_Amount *melt_fee_balance,
-                              struct TALER_Amount *refund_fee_balance,
-                              uint64_t *last_reserve_out_serial_id,
-                              uint64_t *last_deposit_serial_id,
-                              uint64_t *last_melt_serial_id,
-                              uint64_t *last_refund_serial_id);
+                              struct TALER_Amount *denom_risk);
+
+
+  /**
+   * Delete information about a denomination key's balances.
+   *
+   * @param cls the @e cls of this struct with the plugin-specific state
+   * @param session connection to use
+   * @param denom_pub_hash hash of the denomination public key
+   * @return #GNUNET_OK on success; #GNUNET_NO if no record found, #GNUNET_SYSERR on failure
+   */
+  int
+  (*del_denomination_balance)(void *cls,
+                              struct TALER_AUDITORDB_Session *session,
+                              const struct GNUNET_HashCode *denom_pub_hash);
 
 
   /**
@@ -596,16 +635,18 @@ struct TALER_AUDITORDB_Plugin
    * @param deposit_fee_balance total deposit fees collected for this DK
    * @param melt_fee_balance total melt fees collected for this DK
    * @param refund_fee_balance total refund fees collected for this DK
+   * @param risk maximum risk exposure of the exchange
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
-  (*insert_denomination_summary)(void *cls,
-                                 struct TALER_AUDITORDB_Session *session,
-                                 const struct TALER_MasterPublicKeyP *master_pub,
-                                 const struct TALER_Amount *denom_balance,
-                                 const struct TALER_Amount *deposit_fee_balance,
-                                 const struct TALER_Amount *melt_fee_balance,
-                                 const struct TALER_Amount *refund_fee_balance);
+  (*insert_balance_summary)(void *cls,
+                            struct TALER_AUDITORDB_Session *session,
+                            const struct TALER_MasterPublicKeyP *master_pub,
+                            const struct TALER_Amount *denom_balance,
+                            const struct TALER_Amount *deposit_fee_balance,
+                            const struct TALER_Amount *melt_fee_balance,
+                            const struct TALER_Amount *refund_fee_balance,
+                            const struct TALER_Amount *risk);
 
 
   /**
@@ -619,16 +660,18 @@ struct TALER_AUDITORDB_Plugin
    * @param deposit_fee_balance total deposit fees collected for this DK
    * @param melt_fee_balance total melt fees collected for this DK
    * @param refund_fee_balance total refund fees collected for this DK
+   * @param risk maximum risk exposure of the exchange
    * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
-  (*update_denomination_summary)(void *cls,
-                                 struct TALER_AUDITORDB_Session *session,
-                                 const struct TALER_MasterPublicKeyP *master_pub,
-                                 const struct TALER_Amount *denom_balance,
-                                 const struct TALER_Amount *deposit_fee_balance,
-                                 const struct TALER_Amount *melt_fee_balance,
-                                 const struct TALER_Amount *refund_fee_balance);
+  (*update_balance_summary)(void *cls,
+                            struct TALER_AUDITORDB_Session *session,
+                            const struct TALER_MasterPublicKeyP *master_pub,
+                            const struct TALER_Amount *denom_balance,
+                            const struct TALER_Amount *deposit_fee_balance,
+                            const struct TALER_Amount *melt_fee_balance,
+                            const struct TALER_Amount *refund_fee_balance,
+                            const struct TALER_Amount *risk);
 
 
   /**
@@ -641,68 +684,19 @@ struct TALER_AUDITORDB_Plugin
    * @param[out] deposit_fee_balance total deposit fees collected for this DK
    * @param[out] melt_fee_balance total melt fees collected for this DK
    * @param[out] refund_fee_balance total refund fees collected for this DK
+   * @param[out] risk maximum risk exposure of the exchange
    * @return #GNUNET_OK on success; #GNUNET_NO if there is no entry
    *           for this @a master_pub; #GNUNET_SYSERR on failure
    */
   int
-  (*get_denomination_summary)(void *cls,
-                              struct TALER_AUDITORDB_Session *session,
-                              const struct TALER_MasterPublicKeyP *master_pub,
-                              struct TALER_Amount *denom_balance,
-                              struct TALER_Amount *deposit_fee_balance,
-                              struct TALER_Amount *melt_fee_balance,
-                              struct TALER_Amount *refund_fee_balance);
-
-
-  /**
-   * Insert information about an exchange's risk exposure.  There
-   * must not be an existing record for the exchange.
-   *
-   * @param cls the @e cls of this struct with the plugin-specific state
-   * @param session connection to use
-   * @param master_pub master key of the exchange
-   * @param risk maximum risk exposure of the exchange
-   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
-   */
-  int
-  (*insert_risk_summary)(void *cls,
+  (*get_balance_summary)(void *cls,
                          struct TALER_AUDITORDB_Session *session,
                          const struct TALER_MasterPublicKeyP *master_pub,
-                         const struct TALER_Amount *risk);
-
-
-  /**
-   * Update information about an exchange's risk exposure.  There
-   * must be an existing record for the exchange.
-   *
-   * @param cls the @e cls of this struct with the plugin-specific state
-   * @param session connection to use
-   * @param master_pub master key of the exchange
-   * @param risk maximum risk exposure of the exchange
-   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
-   */
-  int
-  (*update_risk_summary)(void *cls,
-                         struct TALER_AUDITORDB_Session *session,
-                         const struct TALER_MasterPublicKeyP *master_pub,
-                         const struct TALER_Amount *risk);
-
-
-  /**
-   * Get information about an exchange's risk exposure.
-   *
-   * @param cls the @e cls of this struct with the plugin-specific state
-   * @param session connection to use
-   * @param master_pub master key of the exchange
-   * @param[out] risk maximum risk exposure of the exchange
-   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure;
-   *         #GNUNET_NO if we have no records for the @a master_pub
-   */
-  int
-  (*get_risk_summary)(void *cls,
-                      struct TALER_AUDITORDB_Session *session,
-                      const struct TALER_MasterPublicKeyP *master_pub,
-                      struct TALER_Amount *risk);
+                         struct TALER_Amount *denom_balance,
+                         struct TALER_Amount *deposit_fee_balance,
+                         struct TALER_Amount *melt_fee_balance,
+                         struct TALER_Amount *refund_fee_balance,
+                         struct TALER_Amount *risk);
 
 
   /**
@@ -717,10 +711,7 @@ struct TALER_AUDITORDB_Plugin
    * @param revenue_balance what was the total profit made from
    *                        deposit fees, melting fees, refresh fees
    *                        and coins that were never returned?
-   * @param deposit_fee_balance total profits from deposit fees
-   * @param melt_fee_balance total profits from melting fees
-   * @param refund_fee_balance total profits from refund fees
-   * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
+     * @return #GNUNET_OK on success; #GNUNET_SYSERR on failure
    */
   int
   (*insert_historic_denom_revenue)(void *cls,
@@ -728,10 +719,8 @@ struct TALER_AUDITORDB_Plugin
                                    const struct TALER_MasterPublicKeyP *master_pub,
                                    const struct GNUNET_HashCode *denom_pub_hash,
                                    struct GNUNET_TIME_Absolute revenue_timestamp,
-                                   const struct TALER_Amount *revenue_balance,
-                                   const struct TALER_Amount *deposit_fee_balance,
-                                   const struct TALER_Amount *melt_fee_balance,
-                                   const struct TALER_Amount *refund_fee_balance);
+                                   const struct TALER_Amount *revenue_balance);
+
 
   /**
    * Obtain all of the historic denomination key revenue
