@@ -128,6 +128,17 @@ enum TALER_ErrorCode
    */
   TALER_EC_PARAMETER_MALFORMED = 1009,
 
+  /**
+   * The exchange failed to obtain the transaction history of the
+   * given coin from the database while generating an insufficient
+   * funds errors.  This can happen during /deposit or /payback requests.
+   * This response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_COIN_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS = 1010,
+
+
+
   /* ********** request-specific error codes ************* */
 
   /**
@@ -230,7 +241,7 @@ enum TALER_ErrorCode
   /**
    * The exchange failed to obtain the transaction history of the
    * given reserve from the database while generating an insufficient
-   * funds errors.
+   * funds error.
    * This response is provided with HTTP status code
    * MHD_HTTP_INTERNAL_SERVER_ERROR.
    */
@@ -343,21 +354,12 @@ enum TALER_ErrorCode
 
   /**
    * The hash of the given wire address does not match the hash
-   * specified in the contract.
+   * specified in the proposal data.
    * This response is provided
    * with HTTP status code MHD_HTTP_BAD_REQUEST.
    */
   TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_CONTRACT_HASH_CONFLICT = 1211,
 
-  /**
-   * The exchange failed to obtain the transaction history of the
-   * given coin from the database while generating an insufficient
-   * funds errors.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_INTERNAL_SERVER_ERROR.
-   */
-  TALER_EC_DEPOSIT_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS = 1212,
-
   /**
    * The exchange detected that the given account number
    * is invalid for the selected wire format type.
@@ -779,6 +781,19 @@ enum TALER_ErrorCode
    */
   TALER_EC_TRACK_TRANSFER_WTID_NOT_FOUND = 1702,
 
+  /**
+   * The exchange did not find information about the wire transfer
+   * fees it charged. This response is
+   * provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_TRACK_TRANSFER_WIRE_FEE_NOT_FOUND = 1703,
+
+  /**
+   * The exchange found a wire fee that was above the total transfer
+   * value (and thus could not have been charged). This response is
+   * provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_TRACK_TRANSFER_WIRE_FEE_INCONSISTENT = 1704,
 
   /**
    * The exchange found internally inconsistent fee data when
@@ -818,6 +833,77 @@ enum TALER_ErrorCode
    */
   TALER_EC_TRACK_TRANSACTION_MERCHANT_SIGNATURE_INVALID = 1804,
 
+  /**
+   * The given denomination key is not in the "payback" set of the
+   * exchange right now.  This response is provided with an
+   * HTTP status code of MHD_HTTP_NOT_FOUND.
+   */
+  TALER_EC_PAYBACK_DENOMINATION_KEY_UNKNOWN = 1850,
+
+  /**
+   * The given coin signature is invalid for the request.
+   * This response is provided with an
+   * HTTP status code of MHD_HTTP_UNAUTHORIZED.
+   */
+  TALER_EC_PAYBACK_SIGNATURE_INVALID = 1851,
+
+  /**
+   * The signature of the denomination key over the coin is not valid.
+   * This response is provided with HTTP status code
+   * MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_PAYBACK_DENOMINATION_SIGNATURE_INVALID = 1852,
+
+  /**
+   * The exchange failed to access its own database about reserves.
+   * This response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAYBACK_DB_FETCH_FAILED = 1853,
+
+  /**
+   * The exchange could not find the corresponding withdraw operation.
+   * The request is denied.  This response is provided with an HTTP
+   * status code of MHD_HTTP_NOT_FOUND.
+   */
+  TALER_EC_PAYBACK_WITHDRAW_NOT_FOUND = 1854,
+
+  /**
+   * The exchange obtained an internally inconsistent transaction
+   * history for the given coin. This response is provided with HTTP
+   * status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAYBACK_HISTORY_DB_ERROR = 1855,
+
+  /**
+   * The exchange failed to store information about the payback to be
+   * performed in the database. This response is provided with HTTP
+   * status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAYBACK_DB_PUT_FAILED = 1856,
+
+  /**
+   * The coin's remaining balance is zero.  The request is denied.
+   * This response is provided with an HTTP status code of
+   * MHD_HTTP_FORBIDDEN.
+   */
+  TALER_EC_PAYBACK_COIN_BALANCE_ZERO = 1857,
+
+  /**
+   * The exchange failed to reproduce the coin's blinding.
+   * This response is provided with an HTTP status code of
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAYBACK_BLINDING_FAILED = 1858,
+
+  /**
+   * The coin's remaining balance is zero.  The request is denied.
+   * This response is provided with an HTTP status code of
+   * MHD_HTTP_INTERNAL_SERVER_ERROR
+   */
+  TALER_EC_PAYBACK_COIN_BALANCE_NEGATIVE = 1857,
+
+
 
   /* *********** Merchant backend error codes ********* */
 
@@ -978,6 +1064,35 @@ enum TALER_ErrorCode
    */
   TALER_EC_PAY_OFFER_EXPIRED = 2121,
 
+  /**
+   * The "merchant" field is missing in the proposal data.
+   * This response is
+   * provided with HTTP status code MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_PAY_MERCHANT_FIELD_MISSING = 2122,
+
+  /**
+   * Failed computing a hash code (likely server out-of-memory).
+   * This response is
+   * provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAY_FAILED_COMPUTE_PROPOSAL_HASH = 2123,
+
+  /**
+   * Failed to locate merchant's account information matching the
+   * wire hash given in the proposal.
+   * This response is
+   * provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAY_WIRE_HASH_UNKNOWN = 2124,
+
+  /**
+   * We got different currencies for the wire fee and the maximum wire
+   * fee.  This response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PAY_WIRE_FEE_CURRENCY_MISSMATCH = 2125,
+
 
   /**
    * Integer overflow with sepcified timestamp argument detected.
@@ -993,6 +1108,11 @@ enum TALER_ErrorCode
    */
   TALER_EC_HISTORY_DB_FETCH_ERROR = 2201,
 
+  /**
+   * Instance used to retrieve history is unknown to the
+   * merchant.
+   */
+  TALER_EC_HISTORY_INSTANCE_UNKNOWN = 2202,
 
   /**
    * We failed to contact the exchange for the /track/transaction
@@ -1128,11 +1248,40 @@ enum TALER_ErrorCode
   /**
    * The exchange gave conflicting information about a coin which has
    * been wire transferred.
-   * The response is
-   * provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   * The response is provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
    */
   TALER_EC_TRACK_TRANSFER_CONFLICTING_REPORTS = 2408,
 
+  /**
+   * The merchant backend had problems in creating the JSON response.
+   */
+  TALER_EC_TRACK_TRANSFER_JSON_RESPONSE_ERROR = 2409,
+
+  /**
+   * The hash provided in the request of /map/in does not match
+   * the contract sent alongside in the same request.
+   */
+  TALER_EC_MAP_IN_UNMATCHED_HASH = 2500,
+
+  /**
+   * The backend encountered an error while trying to store the
+   * h_contract_terms into the database.
+   * The response is provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_PROPOSAL_STORE_DB_ERROR = 2501,
+
+  /**
+   * The backend encountered an error while trying to retrieve the
+   * proposal data from database.  Likely to be an internal error.
+   */
+  TALER_EC_PROPOSAL_LOOKUP_DB_ERROR = 2502,
+
+
+  /**
+   * The proposal being looked up is not found on this merchant.
+   */
+  TALER_EC_PROPOSAL_LOOKUP_NOT_FOUND = 2503,
+
   /* ********** /test API error codes ************* */
 
   /**
