@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+  Copyright (C) 2014-2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -154,7 +154,7 @@ struct TALER_EXCHANGE_AuditorInformation
    * should also be an auditor API for automated submission about
    * claims of misbehaving exchange providers.
    */
-  const char *auditor_url;
+  char *auditor_url;
 
   /**
    * Number of denomination keys audited by this auditor.
@@ -197,6 +197,14 @@ struct TALER_EXCHANGE_Keys
    */
   struct TALER_EXCHANGE_AuditorInformation *auditors;
 
+  /**
+   * Supported Taler protocol version by the exchange.
+   * String in the format current:revision:age using the
+   * semantics of GNU libtool.  See
+   * https://www.gnu.org/software/libtool/manual/html_node/Versioning.html#Versioning
+   */
+  char *version;
+
   /**
    * Length of the @e sign_keys array.
    */
@@ -340,6 +348,75 @@ TALER_EXCHANGE_get_denomination_key_by_hash (const struct TALER_EXCHANGE_Keys *k
 /* *********************  /wire *********************** */
 
 
+/**
+ * Sorted list of fees to be paid for aggregate wire transfers.
+ */
+struct TALER_EXCHANGE_WireAggregateFees
+{
+  /**
+   * This is a linked list.
+   */
+  struct TALER_EXCHANGE_WireAggregateFees *next;
+
+  /**
+   * Fee to be paid whenever the exchange wires funds to the merchant.
+   */
+  struct TALER_Amount wire_fee;
+
+  /**
+   * Fee to be paid when the exchange closes a reserve and wires funds
+   * back to a customer.
+   */
+  struct TALER_Amount closing_fee;
+
+  /**
+   * Time when this fee goes into effect (inclusive)
+   */
+  struct GNUNET_TIME_Absolute start_date;
+
+  /**
+   * Time when this fee stops being in effect (exclusive).
+   */
+  struct GNUNET_TIME_Absolute end_date;
+
+  /**
+   * Signature affirming the above fee structure.
+   */
+  struct TALER_MasterSignatureP master_sig;
+};
+
+
+/**
+ * Function called with information about the wire fees
+ * for each wire method.
+ *
+ * @param cls closure
+ * @param wire_method name of the wire method (i.e. "sepa")
+ * @param fees fee structure for this method
+ */
+typedef void
+(*TALER_EXCHANGE_WireFeeCallback)(void *cls,
+                                  const char *wire_method,
+                                  const struct TALER_EXCHANGE_WireAggregateFees *fees);
+
+
+/**
+ * Obtain information about wire fees encoded in @a obj
+ * by wire method.
+ *
+ * @param master_pub public key to use to verify signatures, NULL to not verify
+ * @param obj wire information as encoded in the #TALER_EXCHANGE_WireResultCallback
+ * @param cb callback to invoke for the fees
+ * @param cb_cls closure for @a cb
+ * @return #GNUNET_OK in success, #GNUNET_SYSERR if @a obj is ill-formed
+ */
+int
+TALER_EXCHANGE_wire_get_fees (const struct TALER_MasterPublicKeyP *master_pub,
+                              const json_t *obj,
+                              TALER_EXCHANGE_WireFeeCallback cb,
+                              void *cb_cls);
+
+
 /**
  * @brief A Wire format inquiry handle
  */
@@ -451,12 +528,11 @@ typedef void
  * @param wire_deadline execution date, until which the merchant would like the exchange to settle the balance (advisory, the exchange cannot be
  *        forced to settle in the past or upon very short notice, but of course a well-behaved exchange will limit aggregation based on the advice received)
  * @param wire_details the merchant’s account details, in a format supported by the exchange
- * @param h_contract hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
+ * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the exchange)
  * @param coin_pub coin’s public key
  * @param denom_pub denomination key with which the coin is signed
  * @param denom_sig exchange’s unblinded signature of the coin
  * @param timestamp timestamp when the contract was finalized, must match approximately the current time of the exchange
- * @param transaction_id transaction id for the transaction between merchant and customer
  * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the exchange (can be zero if refunds are not allowed); must not be after the @a wire_deadline
  * @param coin_sig the signature made with purpose #TALER_SIGNATURE_WALLET_COIN_DEPOSIT made by the customer with the coin’s private key.
@@ -470,12 +546,11 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
                         const struct TALER_Amount *amount,
                         struct GNUNET_TIME_Absolute wire_deadline,
                         json_t *wire_details,
-                        const struct GNUNET_HashCode *h_contract,
+                        const struct GNUNET_HashCode *h_contract_terms,
                         const struct TALER_CoinSpendPublicKeyP *coin_pub,
                         const struct TALER_DenominationSignature *denom_sig,
                         const struct TALER_DenominationPublicKey *denom_pub,
                         struct GNUNET_TIME_Absolute timestamp,
-                        uint64_t transaction_id,
                         const struct TALER_MerchantPublicKeyP *merchant_pub,
                         struct GNUNET_TIME_Absolute refund_deadline,
                         const struct TALER_CoinSpendSignatureP *coin_sig,
@@ -539,8 +614,7 @@ typedef void
  *        (as that fee is still being subtracted), and smaller than the amount
  *        (with deposit fee) of the original deposit contribution of this coin
  * @param refund_fee fee applicable to this coin for the refund
- * @param h_contract hash of the contact of the merchant with the customer that is being refunded
- * @param transaction_id transaction id for the transaction being refunded, must match @a h_contract
+ * @param h_contract_terms hash of the contact of the merchant with the customer that is being refunded
  * @param coin_pub coin’s public key of the coin from the original deposit operation
  * @param rtransaction_id transaction id for the transaction between merchant and customer (of refunding operation);
  *                        this is needed as we may first do a partial refund and later a full refund.  If both
@@ -556,8 +630,7 @@ struct TALER_EXCHANGE_RefundHandle *
 TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
 		       const struct TALER_Amount *amount,
 		       const struct TALER_Amount *refund_fee,
-		       const struct GNUNET_HashCode *h_contract,
-		       uint64_t transaction_id,
+		       const struct GNUNET_HashCode *h_contract_terms,
 		       const struct TALER_CoinSpendPublicKeyP *coin_pub,
 		       uint64_t rtransaction_id,
 		       const struct TALER_MerchantPrivateKeyP *merchant_priv,
@@ -599,7 +672,17 @@ enum TALER_EXCHANGE_ReserveTransactionType {
   /**
    * Withdrawal from the reserve.
    */
-  TALER_EXCHANGE_RTT_WITHDRAWAL
+  TALER_EXCHANGE_RTT_WITHDRAWAL,
+
+  /**
+   * /payback operation.
+   */
+  TALER_EXCHANGE_RTT_PAYBACK,
+
+  /**
+   * Reserve closed operation.
+   */
+  TALER_EXCHANGE_RTT_CLOSE
 
 };
 
@@ -625,6 +708,10 @@ struct TALER_EXCHANGE_ReserveHistory
    */
   union {
 
+    /**
+     * Information about a deposit that filled this reserve.
+     * @e type is #TALER_EXCHANGE_RTT_DEPOSIT.
+     */
     struct {
       /**
        * Sender account information for the incoming transfer.
@@ -632,18 +719,85 @@ struct TALER_EXCHANGE_ReserveHistory
       json_t *sender_account_details;
 
       /**
-       * Wire transfer details for the incoming transfer.
+       * Information that uniquely identifies the wire transfer.
        */
-      json_t *transfer_details;
+      void *wire_reference;
 
+      /**
+       * Number of bytes stored in @e wire_reference.
+       */
+      size_t wire_reference_size;
 
     } in_details;
 
     /**
      * Signature authorizing the withdrawal for outgoing transaction.
+     * @e type is #TALER_EXCHANGE_RTT_WITHDRAWAL.
      */
     json_t *out_authorization_sig;
 
+    /**
+     * Information provided if the reserve was filled via /payback.
+     * @e type is #TALER_EXCHANGE_RTT_PAYBACK.
+     */
+    struct {
+
+      /**
+       * Public key of the coin that was paid back.
+       */
+      struct TALER_CoinSpendPublicKeyP coin_pub;
+
+      /**
+       * Signature of the coin of type
+       * #TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK.
+       */
+      struct TALER_ExchangeSignatureP exchange_sig;
+
+      /**
+       * Public key of the exchange that was used for @e exchange_sig.
+       */
+      struct TALER_ExchangePublicKeyP exchange_pub;
+
+      /**
+       * When did the /payback operation happen?
+       */
+      struct GNUNET_TIME_Absolute timestamp;
+
+    } payback_details;
+
+    /**
+     * Information about a close operation of the reserve.
+     * @e type is #TALER_EXCHANGE_RTT_CLOSE.
+     */
+    struct {
+      /**
+       * Receiver account information for the outgoing wire transfer.
+       */
+      json_t *receiver_account_details;
+
+      /**
+       * Wire transfer details for the outgoing wire transfer.
+       */
+      struct TALER_WireTransferIdentifierRawP wtid;
+
+      /**
+       * Signature of the coin of type
+       * #TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED.
+       */
+      struct TALER_ExchangeSignatureP exchange_sig;
+
+      /**
+       * Public key of the exchange that was used for @e exchange_sig.
+       */
+      struct TALER_ExchangePublicKeyP exchange_pub;
+
+      /**
+       * When did the wire transfer happen?
+       */
+      struct GNUNET_TIME_Absolute timestamp;
+
+    } close_details;
+
   } details;
 
 };
@@ -696,13 +850,13 @@ TALER_EXCHANGE_reserve_status (struct TALER_EXCHANGE_Handle *exchange,
 
 
 /**
- * Cancel a withdraw status request.  This function cannot be used
+ * Cancel a reserve status request.  This function cannot be used
  * on a request handle if a response is already served for it.
  *
- * @param wsh the withdraw status request handle
+ * @param rsh the reserve status request handle
  */
 void
-TALER_EXCHANGE_reserve_status_cancel (struct TALER_EXCHANGE_ReserveStatusHandle *wsh);
+TALER_EXCHANGE_reserve_status_cancel (struct TALER_EXCHANGE_ReserveStatusHandle *rsh);
 
 
 /* ********************* /reserve/withdraw *********************** */
@@ -816,7 +970,7 @@ TALER_EXCHANGE_reserve_withdraw_cancel (struct TALER_EXCHANGE_ReserveWithdrawHan
  *         if the inputs are invalid (i.e. denomination key not with this exchange).
  *         Otherwise, pointer to a buffer of @a res_size to store persistently
  *         before proceeding to #TALER_EXCHANGE_refresh_melt().
- *         Non-null results should be freed using #GNUNET_free().
+ *         Non-null results should be freed using GNUNET_free().
  */
 char *
 TALER_EXCHANGE_refresh_prepare (const struct TALER_CoinSpendPrivateKeyP *melt_priv,
@@ -1134,6 +1288,7 @@ struct TALER_EXCHANGE_TrackTransferHandle;
  * @param execution_time time when the exchange claims to have performed the wire transfer
  * @param total_amount total amount of the wire transfer, or NULL if the exchange could
  *             not provide any @a wtid (set only if @a http_status is #MHD_HTTP_OK)
+ * @param wire_fee wire fee that was charged by the exchange
  * @param details_length length of the @a details array
  * @param details array with details about the combined transactions
  */
@@ -1146,6 +1301,7 @@ typedef void
                                         const struct GNUNET_HashCode *h_wire,
                                         struct GNUNET_TIME_Absolute execution_time,
                                         const struct TALER_Amount *total_amount,
+                                        const struct TALER_Amount *wire_fee,
                                         unsigned int details_length,
                                         const struct TALER_TrackTransferDetails *details);
 
@@ -1217,9 +1373,8 @@ typedef void
  * @param exchange the exchange to query
  * @param merchant_priv the merchant's private key
  * @param h_wire hash of merchant's wire transfer details
- * @param h_contract hash of the contract
+ * @param h_contract_terms hash of the proposal data
  * @param coin_pub public key of the coin
- * @param transaction_id transaction identifier
  * @param cb function to call with the result
  * @param cb_cls closure for @a cb
  * @return handle to abort request
@@ -1228,9 +1383,8 @@ struct TALER_EXCHANGE_TrackTransactionHandle *
 TALER_EXCHANGE_track_transaction (struct TALER_EXCHANGE_Handle *exchange,
                                   const struct TALER_MerchantPrivateKeyP *merchant_priv,
                                   const struct GNUNET_HashCode *h_wire,
-                                  const struct GNUNET_HashCode *h_contract,
+                                  const struct GNUNET_HashCode *h_contract_terms,
                                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                                  uint64_t transaction_id,
                                   TALER_EXCHANGE_TrackTransactionCallback cb,
                                   void *cb_cls);
 
@@ -1263,4 +1417,78 @@ TALER_EXCHANGE_verify_coin_history (const char *currency,
                                     struct TALER_Amount *total);
 
 
+/* ********************* /payback *********************** */
+
+
+/**
+ * @brief A /payback Handle
+ */
+struct TALER_EXCHANGE_PaybackHandle;
+
+
+/**
+ * Callbacks of this type are used to return the final result of
+ * submitting a refresh request to a exchange.  If the operation was
+ * successful, this function returns the signatures over the coins
+ * that were remelted.  The @a coin_privs and @a sigs arrays give the
+ * coins in the same order (and should have the same length) in which
+ * the original request specified the respective denomination keys.
+ *
+ * @param cls closure
+ * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
+ *                    0 if the exchange's reply is bogus (fails to follow the protocol)
+ * @param ec taler-specific error code, #TALER_EC_NONE on success
+ * @param amount amount the exchange will wire back for this coin,
+ *        on error the total balance remaining, or NULL
+ * @param timestamp what time did the exchange receive the /payback request
+ * @param reserve_pub public key of the reserve receiving the payback
+ * @param full_response full response from the exchange (for logging, in case of errors)
+ */
+typedef void
+(*TALER_EXCHANGE_PaybackResultCallback) (void *cls,
+                                         unsigned int http_status,
+                                         enum TALER_ErrorCode ec,
+                                         const struct TALER_Amount *amount,
+                                         struct GNUNET_TIME_Absolute timestamp,
+                                         const struct TALER_ReservePublicKeyP *reserve_pub,
+                                         const json_t *full_response);
+
+
+/**
+ * Ask the exchange to pay back a coin due to the exchange triggering
+ * the emergency payback protocol for a given denomination.  The value
+ * of the coin will be refunded to the original customer (without fees).
+ *
+ * @param exchange the exchange handle; the exchange must be ready to operate
+ * @param pk kind of coin to pay back
+ * @param denom_sig signature over the coin by the exchange using @a pk
+ * @param coin_priv the coin's private key,
+ * @param blinding_key where to fetch the coin's blinding key
+ * @param payback_cb the callback to call when the final result for this request is available
+ * @param payback_cb_cls closure for @a payback_cb
+ * @return NULL
+ *         if the inputs are invalid (i.e. denomination key not with this exchange).
+ *         In this case, the callback is not called.
+ */
+struct TALER_EXCHANGE_PaybackHandle *
+TALER_EXCHANGE_payback (struct TALER_EXCHANGE_Handle *exchange,
+                        const struct TALER_EXCHANGE_DenomPublicKey *pk,
+                        const struct TALER_DenominationSignature *denom_sig,
+                        const struct TALER_CoinSpendPrivateKeyP *coin_priv,
+                        const struct TALER_DenominationBlindingKeyP *blinding_key,
+                        TALER_EXCHANGE_PaybackResultCallback payback_cb,
+                        void *payback_cb_cls);
+
+
+/**
+ * Cancel a payback request.  This function cannot be used on a
+ * request handle if the callback was already invoked.
+ *
+ * @param ph the payback handle
+ */
+void
+TALER_EXCHANGE_payback_cancel (struct TALER_EXCHANGE_PaybackHandle *ph);
+
+
+
 #endif  /* _TALER_EXCHANGE_SERVICE_H */
