@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+  Copyright (C) 2014-2017 GNUnet e.V.
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -46,11 +46,6 @@
  */
 #define TALER_CNC_KAPPA 3
 
-/**
- * After what time do idle reserves "expire"?  We might want to make
- * this a configuration option (eventually).
- */
-#define TALER_IDLE_RESERVE_EXPIRATION_TIME GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_YEARS, 5)
 
 /*********************************************/
 /* Exchange offline signatures (with master key) */
@@ -78,6 +73,15 @@
  */
 #define TALER_SIGNATURE_MASTER_TEST_DETAILS 1027
 
+/**
+ * Fees charged per (aggregate) wire transfer to the merchant.
+ */
+#define TALER_SIGNATURE_MASTER_WIRE_FEES 1028
+
+/**
+ * The given revocation key was revoked and must no longer be used.
+ */
+#define TALER_SIGNATURE_MASTER_DENOMINATION_KEY_REVOKED 1029
 
 /*********************************************/
 /* Exchange online signatures (with signing key) */
@@ -121,6 +125,16 @@
  */
 #define TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND 1038
 
+/**
+ * Signature where the Exchange confirms a payback.
+ */
+#define TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK 1039
+
+/**
+ * Signature where the Exchange confirms it closed a reserve.
+ */
+#define TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED 1040
+
 
 /*********************/
 /* Wallet signatures */
@@ -179,6 +193,11 @@
  */
 #define TALER_SIGNATURE_WALLET_COIN_MELT 1202
 
+/**
+ * Signature using a coin key requesting payback.
+ */
+#define TALER_SIGNATURE_WALLET_COIN_PAYBACK 1203
+
 
 /*******************/
 /* Test signatures */
@@ -266,7 +285,7 @@ struct TALER_DepositRequestPS
   /**
    * Hash over the contract for which this deposit is made.
    */
-  struct GNUNET_HashCode h_contract GNUNET_PACKED;
+  struct GNUNET_HashCode h_contract_terms GNUNET_PACKED;
 
   /**
    * Hash over the wiring information of the merchant.
@@ -295,18 +314,6 @@ struct TALER_DepositRequestPS
    */
   struct GNUNET_TIME_AbsoluteNBO refund_deadline;
 
-  /**
-   * Merchant-generated transaction ID to detect duplicate
-   * transactions.  The merchant must communicate a merchant-unique ID
-   * to the customer for each transaction.  Note that different coins
-   * that are part of the same transaction can use the same
-   * transaction ID.  The transaction ID is useful for later disputes,
-   * and the merchant's contract offer (@e h_contract) with the
-   * customer should include the offer's term and transaction ID
-   * signed with a key from the merchant.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
-
   /**
    * Amount to be deposited, including deposit fee charged by the
    * exchange.  This is the total amount that the coin's value at the exchange
@@ -356,19 +363,13 @@ struct TALER_DepositConfirmationPS
   /**
    * Hash over the contract for which this deposit is made.
    */
-  struct GNUNET_HashCode h_contract GNUNET_PACKED;
+  struct GNUNET_HashCode h_contract_terms GNUNET_PACKED;
 
   /**
    * Hash over the wiring information of the merchant.
    */
   struct GNUNET_HashCode h_wire GNUNET_PACKED;
 
-  /**
-   * Merchant-generated transaction ID to detect duplicate
-   * transactions.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
-
   /**
    * Time when this confirmation was generated.
    */
@@ -417,14 +418,10 @@ struct TALER_RefundRequestPS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Hash over the contract which is being refunded.
+   * Hash over the proposal data to identify the contract
+   * which is being refunded.
    */
-  struct GNUNET_HashCode h_contract GNUNET_PACKED;
-
-  /**
-   * Merchant-generated transaction ID of the orginal transaction.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
+  struct GNUNET_HashCode h_contract_terms GNUNET_PACKED;
 
   /**
    * The coin's public key.  This is the value that must have been
@@ -475,14 +472,10 @@ struct TALER_RefundConfirmationPS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Hash over the contract which is being refunded.
+   * Hash over the proposal data to identify the contract
+   * which is being refunded.
    */
-  struct GNUNET_HashCode h_contract GNUNET_PACKED;
-
-  /**
-   * Merchant-generated transaction ID of the orginal transaction.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
+  struct GNUNET_HashCode h_contract_terms GNUNET_PACKED;
 
   /**
    * The coin's public key.  This is the value that must have been
@@ -905,6 +898,67 @@ struct TALER_MasterWireDetailsPS
 };
 
 
+
+/**
+ * @brief Information signed by the exchange's master
+ * key stating the wire fee to be paid per wire transfer.
+ */
+struct TALER_MasterWireFeePS
+{
+
+  /**
+   * Purpose is #TALER_SIGNATURE_MASTER_WIRE_FEES.
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * Hash over the wire method (yes, H("test") or H("sepa")), in lower
+   * case, including 0-terminator.  Used to uniquely identify which
+   * wire method these fees apply to.
+   */
+  struct GNUNET_HashCode h_wire_method;
+
+  /**
+   * Start date when the fee goes into effect.
+   */
+  struct GNUNET_TIME_AbsoluteNBO start_date;
+
+  /**
+   * End date when the fee stops being in effect (exclusive)
+   */
+  struct GNUNET_TIME_AbsoluteNBO end_date;
+
+  /**
+   * Fee charged to the merchant per wire transfer.
+   */
+  struct TALER_AmountNBO wire_fee;
+
+  /**
+   * Closing fee charged when we wire back funds of a reserve.
+   */
+  struct TALER_AmountNBO closing_fee;
+
+};
+
+
+/**
+ * @brief Message confirming that a denomination key was revoked.
+ */
+struct TALER_MasterDenominationKeyRevocation
+{
+  /**
+   * Purpose is #TALER_SIGNATURE_MASTER_DENOMINATION_KEY_REVOKED.
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * Hash of the denomination key.
+   */
+  struct GNUNET_HashCode h_denom_pub;
+
+};
+
+
 /**
  * @brief Format used to generate the signature on a request to obtain
  * the wire transfer identifier associated with a deposit.
@@ -917,27 +971,15 @@ struct TALER_DepositTrackPS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Hash over the contract for which this deposit is made.
+   * Hash over the proposal data of the contract for which this deposit is made.
    */
-  struct GNUNET_HashCode h_contract GNUNET_PACKED;
+  struct GNUNET_HashCode h_contract_terms GNUNET_PACKED;
 
   /**
    * Hash over the wiring information of the merchant.
    */
   struct GNUNET_HashCode h_wire GNUNET_PACKED;
 
-  /**
-   * Merchant-generated transaction ID to detect duplicate
-   * transactions.  The merchant must communicate a merchant-unique ID
-   * to the customer for each transaction.  Note that different coins
-   * that are part of the same transaction can use the same
-   * transaction ID.  The transaction ID is useful for later disputes,
-   * and the merchant's contract offer (@e h_contract) with the
-   * customer should include the offer's term and transaction ID
-   * signed with a key from the merchant.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
-
   /**
    * The Merchant's public key.  The deposit inquiry request is to be
    * signed by the corresponding private key (using EdDSA).
@@ -963,18 +1005,13 @@ struct TALER_WireDepositDetailP
   /**
    * Hash of the contract
    */
-  struct GNUNET_HashCode h_contract;
+  struct GNUNET_HashCode h_contract_terms;
 
   /**
    * Time when the wire transfer was performed by the exchange.
    */
   struct GNUNET_TIME_AbsoluteNBO execution_time;
 
-  /**
-   * Merchant's transaction ID in NBO.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
-
   /**
    * Coin's public key.
    */
@@ -1010,6 +1047,11 @@ struct TALER_WireDepositDataPS
    */
   struct TALER_AmountNBO total;
 
+  /**
+   * Wire fee that was charged.
+   */
+  struct TALER_AmountNBO wire_fee;
+
   /**
    * Public key of the merchant (for all aggregated transactions).
    */
@@ -1031,51 +1073,19 @@ struct TALER_WireDepositDataPS
 /**
  * The contract sent by the merchant to the wallet.
  */
-struct TALER_ContractPS
+struct TALER_ProposalDataPS
 {
   /**
-   * Purpose header for the signature over the contract with
-   * purpose #TALER_SIGNATURE_MERCHANT_CONTRACT.
+   * Purpose header for the signature over the proposal data
+   * with purpose #TALER_SIGNATURE_MERCHANT_CONTRACT.
    */
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
-  /**
-   * Merchant-generated transaction ID to detect duplicate
-   * transactions, in big endian.  The merchant must communicate a
-   * merchant-unique ID to the customer for each transaction.  Note
-   * that different coins that are part of the same transaction can
-   * use the same transaction ID.  The transaction ID is useful for
-   * later disputes, and the merchant's contract offer (@e h_contract)
-   * with the customer should include the offer's term and transaction
-   * ID signed with a key from the merchant.  This field must match
-   * the corresponding field in the JSON contract.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
-
-  /**
-   * The total amount to be paid to the merchant. Note that if deposit
-   * fees are higher than @e max_fee, the actual total must be higher
-   * to cover the additional fees.  This field must match the
-   * corresponding field in the JSON contract.
-   */
-  struct TALER_AmountNBO total_amount;
-
-  /**
-   * The maximum fee the merchant is willing to cover.  This field
-   * must match the corresponding field in the JSON contract.
-   */
-  struct TALER_AmountNBO max_fee;
-
   /**
    * Hash of the JSON contract in UTF-8 including 0-termination,
    * using JSON_COMPACT | JSON_SORT_KEYS
    */
-  struct GNUNET_HashCode h_contract;
-
-  /**
-   * Key of the merchant who is signing this contract
-   */
-  struct TALER_MerchantPublicKeyP merchant_pub;
+  struct GNUNET_HashCode hash;
 };
 
 /**
@@ -1093,9 +1103,9 @@ struct PaymentResponsePS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Hash of the contract associated with this confirmation
+   * Hash of the proposal data associated with this confirmation
    */
-  struct GNUNET_HashCode h_contract;
+  struct GNUNET_HashCode h_contract_terms;
 };
 
 
@@ -1119,7 +1129,7 @@ struct TALER_ConfirmWirePS
   /**
    * Hash over the contract for which this deposit is made.
    */
-  struct GNUNET_HashCode h_contract GNUNET_PACKED;
+  struct GNUNET_HashCode h_contract_terms GNUNET_PACKED;
 
   /**
    * Raw value (binary encoding) of the wire transfer subject.
@@ -1132,18 +1142,6 @@ struct TALER_ConfirmWirePS
    */
   struct TALER_CoinSpendPublicKeyP coin_pub;
 
-  /**
-   * Merchant-generated transaction ID to detect duplicate
-   * transactions, in big endian.  The merchant must communicate a
-   * merchant-unique ID to the customer for each transaction.  Note
-   * that different coins that are part of the same transaction can
-   * use the same transaction ID.  The transaction ID is useful for
-   * later disputes, and the merchant's contract offer (@e h_contract)
-   * with the customer should include the offer's term and transaction
-   * ID signed with a key from the merchant.
-   */
-  uint64_t transaction_id GNUNET_PACKED;
-
   /**
    * When did the exchange execute this transfer? Note that the
    * timestamp may not be exactly the same on the wire, i.e.
@@ -1159,6 +1157,119 @@ struct TALER_ConfirmWirePS
 
 };
 
+
+/**
+ * Signed data to request that a coin should be refunded as part of
+ * the "emergency" /payback protocol.  The refund will go back to the bank
+ * account that created the reserve.
+ */
+struct TALER_PaybackRequestPS
+{
+  /**
+   * Purpose is #TALER_SIGNATURE_WALLET_COIN_PAYBACK
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * Public key of the coin to be refunded.
+   */
+  struct TALER_CoinSpendPublicKeyP coin_pub;
+
+  /**
+   * Hash of the denomination public key of the coin.
+   */
+  struct GNUNET_HashCode h_denom_pub;
+
+  /**
+   * Blinding factor that was used to withdraw the coin.
+   */
+  struct TALER_DenominationBlindingKeyP coin_blind;
+};
+
+
+/**
+ * Response by which the exchange affirms that it will
+ * refund a coin as part of the emergency /payback
+ * protocol.  The refund will go back to the bank
+ * account that created the reserve.
+ */
+struct TALER_PaybackConfirmationPS
+{
+
+  /**
+   * Purpose is #TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * When did the exchange receive the payback request?
+   * Indirectly determines when the wire transfer is (likely)
+   * to happen.
+   */
+  struct GNUNET_TIME_AbsoluteNBO timestamp;
+
+  /**
+   * How much of the coin's value will the exchange transfer?
+   * (Needed in case the coin was partially spent.)
+   */
+  struct TALER_AmountNBO payback_amount;
+
+  /**
+   * Public key of the coin.
+   */
+  struct TALER_CoinSpendPublicKeyP coin_pub;
+
+  /**
+   * Public key of the reserve that will receive the payback.
+   */
+  struct TALER_ReservePublicKeyP reserve_pub;
+};
+
+
+/**
+ * Response by which the exchange affirms that it has
+ * closed a reserve and send back the funds.
+ */
+struct TALER_ReserveCloseConfirmationPS
+{
+
+  /**
+   * Purpose is #TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * When did the exchange initiate the wire transfer.
+   */
+  struct GNUNET_TIME_AbsoluteNBO timestamp;
+
+  /**
+   * How much did the exchange send?
+   */
+  struct TALER_AmountNBO closing_amount;
+
+  /**
+   * How much did the exchange charge for closing the reserve?
+   */
+  struct TALER_AmountNBO closing_fee;
+
+  /**
+   * Public key of the reserve that received the payback.
+   */
+  struct TALER_ReservePublicKeyP reserve_pub;
+
+  /**
+   * Hash of the receiver's bank account.
+   */
+  struct GNUNET_HashCode h_wire;
+
+  /**
+   * Wire transfer subject.
+   */
+  struct TALER_WireTransferIdentifierRawP wtid;
+};
+
+
 GNUNET_NETWORK_STRUCT_END
 
 #endif
