@@ -252,16 +252,15 @@ invalidate (struct TALER_Amount *a)
 
 
 /**
- * Test if @a a is valid
+ * Test if the given amount is valid.
  *
- * @param a amount to test
- * @return #GNUNET_YES if valid,
- *         #GNUNET_NO if invalid
+ * @param amount amount to check
+ * @return #GNUNET_OK if @a amount is valid
  */
-static int
-test_valid (const struct TALER_Amount *a)
+int
+TALER_amount_is_valid (const struct TALER_Amount *amount)
 {
-  return ('\0' != a->currency[0]);
+  return ('\0' != amount->currency[0]);
 }
 
 
@@ -292,8 +291,8 @@ int
 TALER_amount_cmp_currency (const struct TALER_Amount *a1,
                            const struct TALER_Amount *a2)
 {
-  if ( (GNUNET_NO == test_valid (a1)) ||
-       (GNUNET_NO == test_valid (a2)) )
+  if ( (GNUNET_NO == TALER_amount_is_valid (a1)) ||
+       (GNUNET_NO == TALER_amount_is_valid (a2)) )
     return GNUNET_SYSERR;
   if (0 == strcasecmp (a1->currency,
 		       a2->currency))
@@ -499,7 +498,7 @@ TALER_amount_normalize (struct TALER_Amount *amount)
 {
   int ret;
 
-  if (GNUNET_YES != test_valid (amount))
+  if (GNUNET_YES != TALER_amount_is_valid (amount))
     return GNUNET_SYSERR;
   ret = GNUNET_NO;
   while ( (amount->value != UINT64_MAX) &&
@@ -530,12 +529,12 @@ char *
 TALER_amount_to_string (const struct TALER_Amount *amount)
 {
   char *result;
+  unsigned int i;
   uint32_t n;
   char tail[TALER_AMOUNT_FRAC_LEN + 1];
-  unsigned int i;
   struct TALER_Amount norm;
 
-  if (GNUNET_YES != test_valid (amount))
+  if (GNUNET_YES != TALER_amount_is_valid (amount))
     return NULL;
   norm = *amount;
   GNUNET_break (GNUNET_SYSERR !=
@@ -565,4 +564,80 @@ TALER_amount_to_string (const struct TALER_Amount *amount)
 }
 
 
+/**
+ * Convert amount to string.
+ *
+ * @param amount amount to convert to string
+ * @return statically allocated buffer with string representation,
+ *         NULL if the @a amount was invalid
+ */
+const char *
+TALER_amount2s (const struct TALER_Amount *amount)
+{
+  static char result[TALER_AMOUNT_FRAC_LEN + TALER_CURRENCY_LEN + 3 + 12];
+  unsigned int i;
+  uint32_t n;
+  char tail[TALER_AMOUNT_FRAC_LEN + 1];
+  struct TALER_Amount norm;
+
+  if (GNUNET_YES != TALER_amount_is_valid (amount))
+    return NULL;
+  norm = *amount;
+  GNUNET_break (GNUNET_SYSERR !=
+                TALER_amount_normalize (&norm));
+  if (0 != (n = norm.fraction))
+  {
+    for (i = 0; (i < TALER_AMOUNT_FRAC_LEN) && (0 != n); i++)
+    {
+      tail[i] = '0' + (n / (TALER_AMOUNT_FRAC_BASE / 10));
+      n = (n * 10) % (TALER_AMOUNT_FRAC_BASE);
+    }
+    tail[i] = '\0';
+    GNUNET_snprintf (result,
+                     sizeof (result),
+                     "%s:%llu.%s",
+                     norm.currency,
+                     (unsigned long long) norm.value,
+                     tail);
+  }
+  else
+  {
+    GNUNET_snprintf (result,
+                     sizeof (result),
+                     "%s:%llu",
+                     norm.currency,
+                     (unsigned long long) norm.value);
+  }
+  return result;
+}
+
+
+/**
+ * Divide an amount by a float.  Note that this function
+ * may introduce a rounding error!
+ *
+ * @param result where to store @a dividend / @a divisor
+ * @param dividend amount to divide
+ * @param divisor by what to divide, must be positive
+ */
+void
+TALER_amount_divide (struct TALER_Amount *result,
+                     const struct TALER_Amount *dividend,
+                     uint32_t divisor)
+{
+  uint64_t modr;
+
+  GNUNET_assert (0 != divisor);
+  *result = *dividend;
+  if (1 == divisor)
+    return;
+  modr = result->value % divisor;
+  result->value /= divisor;
+  /* modr is a 32-bit value, so we can safely multiply by (<32-bit) base and add fraction! */
+  modr = (modr * TALER_AMOUNT_FRAC_BASE) + result->fraction;
+  GNUNET_assert (modr < TALER_AMOUNT_FRAC_BASE * divisor);
+  result->fraction = (uint32_t) (modr / divisor);
+}
+
+
 /* end of amount.c */
