@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016 GNUnet e.V. & Inria
+  Copyright (C) 2016, 2017 GNUnet e.V. & Inria
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -72,7 +72,7 @@ sepa_amount_round (void *cls,
   if (0 == delta)
     return GNUNET_NO;
   amount->fraction -= delta;
-  return GNUNET_SYSERR;
+  return GNUNET_OK;
 }
 
 
@@ -357,7 +357,7 @@ validate_iban (const char *iban)
  * @param sepa_name name of the account holder
  * @param iban bank account number in IBAN format
  * @param bic bank identifier
- * @param[out] mp purpose to be signed
+ * @param[out] wsd purpose to be signed
  */
 static void
 compute_purpose (const char *sepa_name,
@@ -451,7 +451,7 @@ verify_wire_sepa_signature_ok (const json_t *json,
  * @param cls the @e cls of this struct with the plugin-specific state
  * @param wire the JSON wire format object
  * @param master_pub public key of the exchange to verify against
- * @param[OUT] emsg set to an error message, unless we return #TALER_EC_NONE;
+ * @param[out] emsg set to an error message, unless we return #TALER_EC_NONE;
  *             error message must be freed by the caller using GNUNET_free()
  * @return #TALER_EC_NONE if correctly formatted
  */
@@ -658,6 +658,7 @@ sepa_sign_wire_details (void *cls,
  * @param cls the @e cls of this struct with the plugin-specific state
  * @param wire valid wire account information
  * @param amount amount to transfer, already rounded
+ * @param exchange_base_url base URL of the exchange (for tracking)
  * @param wtid wire transfer identifier to use
  * @param psc function to call with the prepared data to persist
  * @param psc_cls closure for @a psc
@@ -667,6 +668,7 @@ static struct TALER_WIRE_PrepareHandle *
 sepa_prepare_wire_transfer (void *cls,
                             const json_t *wire,
                             const struct TALER_Amount *amount,
+                            const char *exchange_base_url,
                             const struct TALER_WireTransferIdentifierRawP *wtid,
                             TALER_WIRE_PrepareTransactionCallback psc,
                             void *psc_cls)
@@ -733,6 +735,54 @@ sepa_execute_wire_transfer_cancel (void *cls,
 }
 
 
+/**
+ * Query transfer history of an account.  We use the variable-size
+ * @a start_off to indicate which transfers we are interested in as
+ * different banking systems may have different ways to identify
+ * transfers.  The @a start_off value must thus match the value of
+ * a `row_off` argument previously given to the @a hres_cb.  Use
+ * NULL to query transfers from the beginning of time (with
+ * positive @a num_results) or from the latest committed transfers
+ * (with negative @a num_results).
+ *
+ * @param cls the @e cls of this struct with the plugin-specific state
+ * @param direction what kinds of wire transfers should be returned
+ * @param start_off from which row on do we want to get results, use NULL for the latest; exclusive
+ * @param start_off_len number of bytes in @a start_off; must be `sizeof(uint64_t)`.
+ * @param num_results how many results do we want; negative numbers to go into the past,
+ *                    positive numbers to go into the future starting at @a start_row;
+ *                    must not be zero.
+ * @param hres_cb the callback to call with the transaction history
+ * @param hres_cb_cls closure for the above callback
+ */
+static struct TALER_WIRE_HistoryHandle *
+sepa_get_history (void *cls,
+                  enum TALER_BANK_Direction direction,
+                  const void *start_off,
+                  size_t start_off_len,
+                  int64_t num_results,
+                  TALER_WIRE_HistoryResultCallback hres_cb,
+                  void *hres_cb_cls)
+{
+  GNUNET_break (0);
+  return NULL;
+}
+
+
+/**
+ * Cancel going over the account's history.
+ *
+ * @param cls the @e cls of this struct with the plugin-specific state
+ * @param whh operation to cancel
+ */
+static void
+sepa_get_history_cancel (void *cls,
+			 struct TALER_WIRE_HistoryHandle *whh)
+{
+  GNUNET_break (0);
+}
+
+
 /**
  * Initialize sepa-wire subsystem.
  *
@@ -772,6 +822,8 @@ libtaler_plugin_wire_sepa_init (void *cls)
   plugin->prepare_wire_transfer_cancel = &sepa_prepare_wire_transfer_cancel;
   plugin->execute_wire_transfer = &sepa_execute_wire_transfer;
   plugin->execute_wire_transfer_cancel = &sepa_execute_wire_transfer_cancel;
+  plugin->get_history = &sepa_get_history;
+  plugin->get_history_cancel = &sepa_get_history_cancel;
   return plugin;
 }
 
