@@ -45,6 +45,11 @@ struct TestClosure
    */
   char *bank_uri;
 
+  /**
+   * Authentication information.
+   */
+  struct TALER_BANK_AuthenticationData auth;
+
   /**
    * Handle to the context for sending funds to the bank.
    */
@@ -57,9 +62,9 @@ struct TestClosure
 
   /**
    * Number of the account that the exchange has at the bank for
-   * outgoing transfers.
+   * transfers.
    */
-  unsigned long long exchange_account_outgoing_no;
+  unsigned long long exchange_account_no;
 
 };
 
@@ -85,6 +90,11 @@ struct TALER_WIRE_PrepareHandle
    */
   json_t *wire;
 
+  /**
+   * Base URL to use for the exchange.
+   */
+  char *exchange_base_url;
+
   /**
    * Function to call with the serialized data.
    */
@@ -178,7 +188,7 @@ test_amount_round (void *cls,
  *
  * @param account number of the account
  * @param bank_uri URI of the bank
- * @param[out] mp purpose to be signed
+ * @param[out] wsd purpose to be signed
  */
 static void
 compute_purpose (uint64_t account,
@@ -213,7 +223,7 @@ compute_purpose (uint64_t account,
  * @param cls the @e cls of this struct with the plugin-specific state
  * @param wire the JSON wire format object
  * @param master_pub public key of the exchange to verify against
- * @param[OUT] emsg set to an error message, unless we return #TALER_EC_NONE;
+ * @param[out] emsg set to an error message, unless we return #TALER_EC_NONE;
  *             error message must be freed by the caller using GNUNET_free()
  * @return #TALER_EC_NONE if correctly formatted
  */
@@ -397,7 +407,9 @@ struct BufFormatP
    */
   struct TALER_AmountNBO amount;
 
-  /* followed by serialized 'wire' JSON data */
+  /* followed by serialized 'wire' JSON data (0-terminated) */
+
+  /* followed by 0-terminated base URL */
 
 };
 GNUNET_NETWORK_STRUCT_END
@@ -417,6 +429,7 @@ test_prepare_wire_transfer_cancel (void *cls,
   if (NULL != pth->task)
     GNUNET_SCHEDULER_cancel (pth->task);
   json_decref (pth->wire);
+  GNUNET_free (pth->exchange_base_url);
   GNUNET_free (pth);
 }
 
@@ -432,7 +445,8 @@ do_prepare (void *cls)
 {
   struct TALER_WIRE_PrepareHandle *pth = cls;
   char *wire_enc;
-  size_t len;
+  size_t len_w;
+  size_t len_b;
   struct BufFormatP bf;
 
   pth->task = NULL;
@@ -449,19 +463,23 @@ do_prepare (void *cls)
                                        pth);
     return;
   }
-  len = strlen (wire_enc) + 1;
+  len_w = strlen (wire_enc) + 1;
+  len_b = strlen (pth->exchange_base_url) + 1;
   bf.wtid = pth->wtid;
   TALER_amount_hton (&bf.amount,
                      &pth->amount);
   {
-    char buf[sizeof (struct BufFormatP) + len];
+    char buf[sizeof (struct BufFormatP) + len_w + len_b];
 
     memcpy (buf,
             &bf,
             sizeof (struct BufFormatP));
     memcpy (&buf[sizeof (struct BufFormatP)],
             wire_enc,
-            len);
+            len_w);
+    memcpy (&buf[sizeof (struct BufFormatP) + len_w],
+            pth->exchange_base_url,
+            len_b);
 
     /* finally give the state back */
     pth->ptc (pth->ptc_cls,
@@ -485,6 +503,7 @@ do_prepare (void *cls)
  * @param cls the @e cls of this struct with the plugin-specific state
  * @param wire valid wire account information
  * @param amount amount to transfer, already rounded
+ * @param exchange_base_url base URL of this exchange
  * @param wtid wire transfer identifier to use
  * @param ptc function to call with the prepared data to persist
  * @param ptc_cls closure for @a ptc
@@ -494,6 +513,7 @@ static struct TALER_WIRE_PrepareHandle *
 test_prepare_wire_transfer (void *cls,
                             const json_t *wire,
                             const struct TALER_Amount *amount,
+                            const char *exchange_base_url,
                             const struct TALER_WireTransferIdentifierRawP *wtid,
                             TALER_WIRE_PrepareTransactionCallback ptc,
                             void *ptc_cls)
@@ -515,6 +535,7 @@ test_prepare_wire_transfer (void *cls,
   pth = GNUNET_new (struct TALER_WIRE_PrepareHandle);
   pth->tc = tc;
   pth->wire = json_incref ((json_t *) wire);
+  pth->exchange_base_url = GNUNET_strdup (exchange_base_url);
   pth->wtid = *wtid;
   pth->ptc = ptc;
   pth->ptc_cls = ptc_cls;
@@ -532,11 +553,13 @@ test_prepare_wire_transfer (void *cls,
  * @param cls closure with the `struct TALER_WIRE_ExecuteHandle`
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
  *                    0 if the bank's reply is bogus (fails to follow the protocol)
+ * @param serial_id unique ID of the wire transfer in the bank's records; UINT64_MAX on error
  * @param json detailed response from the HTTPD, or NULL if reply was not JSON
  */
 static void
 execute_cb (void *cls,
             unsigned int http_status,
+            uint64_t serial_id,
             const json_t *json)
 {
   struct TALER_WIRE_ExecuteHandle *eh = cls;
@@ -564,6 +587,7 @@ execute_cb (void *cls,
                      http_status);
   eh->cc (eh->cc_cls,
           (MHD_HTTP_OK == http_status) ? GNUNET_OK : GNUNET_SYSERR,
+          serial_id,
           (MHD_HTTP_OK == http_status) ? NULL : s);
   GNUNET_free (s);
   GNUNET_free (eh);
@@ -650,6 +674,8 @@ test_execute_wire_transfer (void *cls,
   json_int_t account_no;
   struct BufFormatP bf;
   char *emsg;
+  const char *json_s;
+  const char *exchange_base_url;
 
   if (NULL == tc->ctx)
   {
@@ -658,7 +684,14 @@ test_execute_wire_transfer (void *cls,
     return NULL; /* not initialized with configuration, cannot do transfers */
   }
   if ( (buf_size <= sizeof (struct BufFormatP)) ||
-       ('\0' != buf[buf_size -1]) )
+       ('\0' != buf[buf_size - 1]) )
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  json_s = &buf[sizeof (struct BufFormatP)];
+  exchange_base_url = &json_s[strlen (json_s) + 1];
+  if (exchange_base_url > &buf[buf_size - 1])
   {
     GNUNET_break (0);
     return NULL;
@@ -668,7 +701,7 @@ test_execute_wire_transfer (void *cls,
           sizeof (bf));
   TALER_amount_ntoh (&amount,
                      &bf.amount);
-  wire = json_loads (&buf[sizeof (struct BufFormatP)],
+  wire = json_loads (json_s,
                      JSON_REJECT_DUPLICATES,
                      NULL);
   if (NULL == wire)
@@ -697,9 +730,11 @@ test_execute_wire_transfer (void *cls,
   eh->cc_cls = cc_cls;
   eh->aaih = TALER_BANK_admin_add_incoming (tc->ctx,
                                             tc->bank_uri,
+                                            &tc->auth,
+                                            exchange_base_url,
                                             &bf.wtid,
                                             &amount,
-                                            (uint64_t) tc->exchange_account_outgoing_no,
+                                            (uint64_t) tc->exchange_account_no,
 					    (uint64_t) account_no,
                                             &execute_cb,
                                             eh);
@@ -735,6 +770,211 @@ test_execute_wire_transfer_cancel (void *cls,
 }
 
 
+/**
+ * Handle for a #test_get_history() request.
+ */
+struct TALER_WIRE_HistoryHandle
+{
+
+  /**
+   * Function to call with results.
+   */
+  TALER_WIRE_HistoryResultCallback hres_cb;
+
+  /**
+   * Closure for @e hres_cb.
+   */
+  void *hres_cb_cls;
+
+  /**
+   * Request to the bank.
+   */
+  struct TALER_BANK_HistoryHandle *hh;
+
+};
+
+
+/**
+ * Function called with results from the bank about the transaction history.
+ *
+ * @param cls the `struct TALER_WIRE_HistoryHandle`
+ * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
+ *                    0 if the bank's reply is bogus (fails to follow the protocol),
+ *                    #MHD_HTTP_NO_CONTENT if there are no more results; on success the
+ *                    last callback is always of this status (even if `abs(num_results)` were
+ *                    already returned).
+ * @param dir direction of the transfer
+ * @param serial_id monotonically increasing counter corresponding to the transaction
+ * @param details details about the wire transfer
+ * @param json detailed response from the HTTPD, or NULL if reply was not in JSON
+ */
+static void
+bhist_cb (void *cls,
+          unsigned int http_status,
+          enum TALER_BANK_Direction dir,
+          uint64_t serial_id,
+          const struct TALER_BANK_TransferDetails *details,
+          const json_t *json)
+{
+  struct TALER_WIRE_HistoryHandle *whh = cls;
+  uint64_t bserial_id = GNUNET_htonll (serial_id);
+  struct TALER_WIRE_TransferDetails wd;
+
+  if (MHD_HTTP_OK == http_status)
+  {
+    char *subject;
+    char *space;
+
+    wd.amount = details->amount;
+    wd.execution_date = details->execution_date;
+    subject = GNUNET_strdup (details->wire_transfer_subject);
+    space = strchr (subject, (int) ' ');
+    if (NULL != space)
+    {
+      /* Space separates the actual wire transfer subject from the
+         exchange base URL (if present, expected only for outgoing
+         transactions).  So we cut the string off at the space. */
+      *space = '\0';
+    }
+    /* NOTE: For a real bank, the subject should include a checksum! */
+    if (GNUNET_OK !=
+        GNUNET_STRINGS_string_to_data (subject,
+                                       strlen (subject),
+                                       &wd.reserve_pub,
+                                       sizeof (wd.reserve_pub)))
+    {
+      GNUNET_break (0);
+      GNUNET_free (subject);
+      /* NOTE: for a "real" bank, we would want to trigger logic to undo the
+         wire transfer. However, for the "demo" bank, it should currently
+         be "impossible" to do wire transfers with invalid subjects, and
+         equally we thus don't need to undo them (and there is no API to do
+         that nicely either right now). So we don't handle this case for now. */
+      return;
+    }
+    GNUNET_free (subject);
+    wd.account_details = details->account_details;
+
+    if ( (NULL != whh->hres_cb) &&
+	 (GNUNET_OK !=
+	  whh->hres_cb (whh->hres_cb_cls,
+			dir,
+			&bserial_id,
+			sizeof (bserial_id),
+			&wd)) )
+      whh->hres_cb = NULL;
+  }
+  else
+  {
+    if (NULL != whh->hres_cb)
+      (void) whh->hres_cb (whh->hres_cb_cls,
+                           TALER_BANK_DIRECTION_NONE,
+                           NULL,
+                           0,
+                           NULL);
+    whh->hh = NULL;
+    GNUNET_free (whh);
+  }
+}
+
+
+/**
+ * Query transfer history of an account.  We use the variable-size
+ * @a start_off to indicate which transfers we are interested in as
+ * different banking systems may have different ways to identify
+ * transfers.  The @a start_off value must thus match the value of
+ * a `row_off` argument previously given to the @a hres_cb.  Use
+ * NULL to query transfers from the beginning of time (with
+ * positive @a num_results) or from the latest committed transfers
+ * (with negative @a num_results).
+ *
+ * @param cls the @e cls of this struct with the plugin-specific state
+ * @param direction what kinds of wire transfers should be returned
+ * @param start_off from which row on do we want to get results, use NULL for the latest; exclusive
+ * @param start_off_len number of bytes in @a start_off; must be `sizeof(uint64_t)`.
+ * @param num_results how many results do we want; negative numbers to go into the past,
+ *                    positive numbers to go into the future starting at @a start_row;
+ *                    must not be zero.
+ * @param hres_cb the callback to call with the transaction history
+ * @param hres_cb_cls closure for the above callback
+ */
+static struct TALER_WIRE_HistoryHandle *
+test_get_history (void *cls,
+                  enum TALER_BANK_Direction direction,
+                  const void *start_off,
+                  size_t start_off_len,
+                  int64_t num_results,
+                  TALER_WIRE_HistoryResultCallback hres_cb,
+                  void *hres_cb_cls)
+{
+  struct TestClosure *tc = cls;
+  struct TALER_WIRE_HistoryHandle *whh;
+  const uint64_t *start_off_b64;
+  uint64_t start_row;
+
+  if (0 == num_results)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  if (TALER_BANK_DIRECTION_NONE == direction)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  if ( (NULL != start_off) &&
+       (sizeof (uint64_t) != start_off_len) )
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  if (NULL == start_off)
+  {
+    start_row = UINT64_MAX; /* no start row */
+  }
+  else
+  {
+    start_off_b64 = start_off;
+    start_row = GNUNET_ntohll (*start_off_b64);
+  }
+
+  whh = GNUNET_new (struct TALER_WIRE_HistoryHandle);
+  whh->hres_cb = hres_cb;
+  whh->hres_cb_cls = hres_cb_cls;
+  whh->hh = TALER_BANK_history (tc->ctx,
+                                tc->bank_uri,
+                                &tc->auth,
+                                (uint64_t) tc->exchange_account_no,
+                                direction,
+                                start_row,
+                                num_results,
+                                &bhist_cb,
+                                whh);
+  if (NULL == whh->hh)
+  {
+    GNUNET_break (0);
+    GNUNET_free (whh);
+    return NULL;
+  }
+  return whh;
+}
+
+
+/**
+ * Cancel going over the account's history.
+ *
+ * @param cls the @e cls of this struct with the plugin-specific state
+ * @param whh operation to cancel
+ */
+static void
+test_get_history_cancel (void *cls,
+			 struct TALER_WIRE_HistoryHandle *whh)
+{
+  TALER_BANK_history_cancel (whh->hh);
+  GNUNET_free (whh);
+}
+
+
 /**
  * Initialize test-wire subsystem.
  *
@@ -747,30 +987,32 @@ libtaler_plugin_wire_test_init (void *cls)
   struct GNUNET_CONFIGURATION_Handle *cfg = cls;
   struct TestClosure *tc;
   struct TALER_WIRE_Plugin *plugin;
+  char *user;
+  char *pass;
 
   tc = GNUNET_new (struct TestClosure);
   if (NULL != cfg)
   {
     if (GNUNET_OK !=
         GNUNET_CONFIGURATION_get_value_string (cfg,
-                                               "exchange-wire-outgoing-test",
+                                               "exchange-wire-test",
                                                "BANK_URI",
                                                &tc->bank_uri))
     {
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                                 "exchange-wire-outgoing-test",
+                                 "exchange-wire-test",
                                  "BANK_URI");
       GNUNET_free (tc);
       return NULL;
     }
     if (GNUNET_OK !=
         GNUNET_CONFIGURATION_get_value_number (cfg,
-                                               "exchange-wire-outgoing-test",
+                                               "exchange-wire-test",
                                                "EXCHANGE_ACCOUNT_NUMBER",
-                                               &tc->exchange_account_outgoing_no))
+                                               &tc->exchange_account_no))
     {
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                                 "exchange-wire-outgoing-test",
+                                 "exchange-wire-test",
                                  "EXCHANGE_ACCOUNT_NUMBER");
       GNUNET_free (tc->bank_uri);
       GNUNET_free (tc);
@@ -789,6 +1031,36 @@ libtaler_plugin_wire_test_init (void *cls)
       GNUNET_free (tc);
       return NULL;
     }
+    if (GNUNET_OK !=
+        GNUNET_CONFIGURATION_get_value_string (cfg,
+                                               "exchange-wire-test",
+                                               "USERNAME",
+                                               &user))
+    {
+      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                                 "exchange-wire-test",
+                                 "USERNAME");
+      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc);
+      return NULL;
+    }
+    if (GNUNET_OK !=
+        GNUNET_CONFIGURATION_get_value_string (cfg,
+                                               "exchange-wire-test",
+                                               "PASSWORD",
+                                               &pass))
+    {
+      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                                 "exchange-wire-test",
+                                 "PASSWORD");
+      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc);
+      GNUNET_free (user);
+      return NULL;
+    }
+    tc->auth.method = TALER_BANK_AUTH_BASIC;
+    tc->auth.details.basic.username = user;
+    tc->auth.details.basic.password = pass;
     tc->ctx = GNUNET_CURL_init (&GNUNET_CURL_gnunet_scheduler_reschedule,
                                 &tc->rc);
     tc->rc = GNUNET_CURL_gnunet_rc_create (tc->ctx);
@@ -797,6 +1069,8 @@ libtaler_plugin_wire_test_init (void *cls)
       GNUNET_break (0);
       GNUNET_free (tc->currency);
       GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc->auth.details.basic.username);
+      GNUNET_free (tc->auth.details.basic.password);
       GNUNET_free (tc);
       return NULL;
     }
@@ -811,6 +1085,8 @@ libtaler_plugin_wire_test_init (void *cls)
   plugin->prepare_wire_transfer_cancel = &test_prepare_wire_transfer_cancel;
   plugin->execute_wire_transfer = &test_execute_wire_transfer;
   plugin->execute_wire_transfer_cancel = &test_execute_wire_transfer_cancel;
+  plugin->get_history = &test_get_history;
+  plugin->get_history_cancel = &test_get_history_cancel;
   return plugin;
 }
 
@@ -837,6 +1113,23 @@ libtaler_plugin_wire_test_done (void *cls)
     GNUNET_CURL_gnunet_rc_destroy (tc->rc);
     tc->rc = NULL;
   }
+  switch (tc->auth.method)
+  {
+  case TALER_BANK_AUTH_NONE:
+    break;
+  case TALER_BANK_AUTH_BASIC:
+    if (NULL != tc->auth.details.basic.username)
+    {
+      GNUNET_free (tc->auth.details.basic.username);
+      tc->auth.details.basic.username = NULL;
+    }
+    if (NULL != tc->auth.details.basic.password)
+    {
+      GNUNET_free (tc->auth.details.basic.password);
+      tc->auth.details.basic.password = NULL;
+    }
+    break;
+  }
   GNUNET_free_non_null (tc->currency);
   GNUNET_free_non_null (tc->bank_uri);
   GNUNET_free (tc);
